@@ -3,7 +3,7 @@
 usage(){
 echo "
 Written by Brian Bushnell
-Last modified April 11, 2019
+Last modified November 7, 2019
 
 Description:  Compares reads to the kmers in a reference dataset, optionally 
 allowing an edit distance. Splits the reads into two outputs - those that 
@@ -279,13 +279,19 @@ entropy=-1          Set between 0 and 1 to filter reads with entropy below
 entropywindow=50    Calculate entropy using a sliding window of this length.
 entropyk=5          Calculate entropy using kmers of this length.
 minbasefrequency=0  Discard reads with a minimum base frequency below this.
+entropytrim=f       Values:
+                       f:  (false) Do not entropy-trim.
+                       r:  (right) Trim low entropy on the right end only.
+                       l:  (left) Trim low entropy on the left end only.
+                       rl: (both) Trim low entropy on both ends.
 entropymask=f       Values:
-                       f:  Discard low-entropy sequences.
-                       t:  Mask low-entropy parts of sequences with N.
+                       f:  (filter) Discard low-entropy sequences.
+                       t:  (true) Mask low-entropy parts of sequences with N.
                        lc: Change low-entropy parts of sequences to lowercase.
 entropymark=f       Mark each base with its entropy value.  This is on a scale
                     of 0-41 and is reported as quality scores, so the output
                     should be fastq or fasta+qual.
+NOTE: If set, entropytrim overrides entropymask.
 
 Cardinality estimation:
 cardinality=f       (loglog) Count unique kmers using the LogLog algorithm.
