@@ -164,7 +164,7 @@ public class BBSplitter {
 				}else if(a.equals("rebuild")){
 					forceRebuild=Tools.parseBoolean(b);
 					unparsed.add(args[i]);
-				}else if(a.equals("fastawrap")){
+				}else if(a.equals("fastawrap") || a.equals("wrap")){
 					Shared.FASTA_WRAP=Integer.parseInt(b);
 				}else{
 					unparsed.add(args[i]);
