@@ -3,6 +3,8 @@ package assemble;
 import java.util.ArrayList;
 import java.util.concurrent.atomic.AtomicInteger;
 
+import structures.ByteBuilder;
+
 /**
  * Thread for exploring connectivity graph between contigs.
  * @author Brian Bushnell
@@ -24,14 +26,14 @@ public abstract class AbstractProcessContigThread extends Thread {
 	public final void processContigs(ArrayList<Contig> contigs){
 		for(int cnum=next.getAndIncrement(); cnum<contigs.size(); cnum=next.getAndIncrement()){
 			Contig c=contigs.get(cnum);
-			processContigLeft(c, leftCounts, rightCounts, extraCounts);
-			processContigRight(c, leftCounts, rightCounts, extraCounts);
+			processContigLeft(c, leftCounts, rightCounts, extraCounts, bb);
+			processContigRight(c, leftCounts, rightCounts, extraCounts, bb);
 		}
 	}
 
-	abstract void processContigLeft(Contig c, int[] leftCounts, int[] rightCounts, int[] extraCounts);
+	abstract void processContigLeft(Contig c, int[] leftCounts, int[] rightCounts, int[] extraCounts, ByteBuilder bb);
 
-	abstract void processContigRight(Contig c, int[] leftCounts, int[] rightCounts, int[] extraCounts);
+	abstract void processContigRight(Contig c, int[] leftCounts, int[] rightCounts, int[] extraCounts, ByteBuilder bb);
 
 	final int[] leftCounts=new int[4];
 	final int[] rightCounts=new int[4];
@@ -44,6 +46,7 @@ public abstract class AbstractProcessContigThread extends Thread {
 	int lastTarget=-1;
 	int lastExitCondition=-1;
 	int lastOrientation=-1;
+	ByteBuilder bb=new ByteBuilder();
 	long edgesMadeT=0;
 
 }
