@@ -279,7 +279,7 @@ public class BloomFilterWrapper {
 		{
 			Timer t=new Timer(outstream, true);
 			if(serialIn==null){
-				filter=new BloomFilter(null, null, ref, k, bits, hashes, minConsecutiveMatches,
+				filter=new BloomFilter(null, null, ref, k, k, bits, hashes, minConsecutiveMatches,
 						rcomp, ecco, merge, memFraction);
 				if(serialOut!=null){
 					ReadWrite.writeObjectInThread(filter, serialOut, true);
