@@ -273,6 +273,7 @@ public class Clump extends ArrayList<Read> implements Comparable<Clump> {
 								float errB=b.expectedErrorsIncludingMate(true);
 								if(markAll){
 									b.setDiscarded(true);
+									assert(!a.discarded() || markAll);
 									dupePairs++;
 									dupeReads+=1+b.mateCount();
 									unequals=0;
@@ -283,12 +284,14 @@ public class Clump extends ArrayList<Read> implements Comparable<Clump> {
 									}
 								}else if(containment || errB>=errA){
 									b.setDiscarded(true);
+									assert(!a.discarded() || markAll);
 									a.copies+=b.copies+parseExtraCopies(b);
 									dupePairs++;
 									dupeReads+=1+b.mateCount();
 									unequals=0;
 								}else{
 									a.setDiscarded(true);
+									assert(!b.discarded() || markAll);
 									b.copies+=a.copies+parseExtraCopies(a);
 									dupePairs++;
 									dupeReads+=1+a.mateCount();
@@ -382,6 +385,7 @@ public class Clump extends ArrayList<Read> implements Comparable<Clump> {
 								float errB=b.expectedErrorsIncludingMate(true);
 								if(markAll){
 									b.setDiscarded(true);
+									assert(!a.discarded() || markAll);
 									dupePairs++;
 									dupeReads+=1+b.mateCount();
 									unequals=0;
@@ -392,12 +396,14 @@ public class Clump extends ArrayList<Read> implements Comparable<Clump> {
 									}
 								}else if(containment || errB>=errA){
 									b.setDiscarded(true);
+									assert(!a.discarded() || markAll);
 									a.copies+=b.copies+parseExtraCopies(b);
 									dupePairs++;
 									dupeReads+=1+b.mateCount();
 									unequals=0;
 								}else{
 									a.setDiscarded(true);
+									assert(!b.discarded() || markAll);
 									b.copies+=a.copies+parseExtraCopies(a);
 									dupePairs++;
 									dupeReads+=1+a.mateCount();
@@ -481,8 +487,11 @@ public class Clump extends ArrayList<Read> implements Comparable<Clump> {
 //	}
 	
 	public static boolean equals(Read a, Read b, int maxSubs, float dupeSubRate){
+		if(a.numericID==b.numericID){return false;}
 		if(dupeSubRate>0){maxSubs=Tools.max(maxSubs, (int)(dupeSubRate*Tools.min(a.length(), b.length())));}
-		if(containment){return contains(a, b, maxSubs);}
+		if(containment){
+			return contains(a, b, maxSubs);
+		}
 		if(!equals(a.bases, b.bases, maxSubs)){return false;}
 		if(a.mate!=null && !equals(a.mate.bases, b.mate.bases, maxSubs)){return false;}
 		return true;
@@ -521,10 +530,11 @@ public class Clump extends ArrayList<Read> implements Comparable<Clump> {
 		ReadKey rkb1=(ReadKey)b.obj;
 		ReadKey rka2=(ReadKey)a.mate.obj; //TODO: In containment mode, mates need to always get keys.
 		ReadKey rkb2=(ReadKey)b.mate.obj;
-		return ((rka1.kmerMinusStrand==rkb1.kmerMinusStrand)!=(rka2.kmerMinusStrand==rkb2.kmerMinusStrand)); //Ensures that both reads have the same directionality.
+		return ((rka1.kmerMinusStrand==rkb1.kmerMinusStrand) && (rka2.kmerMinusStrand==rkb2.kmerMinusStrand)); //Ensures that both reads have the same directionality.
 	}
 	
 	public static boolean contains_inner(Read a, Read b, int maxSubs){
+//		if(a.length()==b.length()){return equals(a.bases, b.bases, maxSubs);}
 		ReadKey rka=(ReadKey)a.obj;
 		ReadKey rkb=(ReadKey)b.obj;
 		if(affix ? !rka.physicalAffix(rkb, a.length(), b.length()) : !rka.physicallyContains(rkb, a.length(), b.length())){return false;}
