@@ -240,6 +240,8 @@ public final class AminoAcid {
 	public static final byte[] lowerCaseToNocall=new byte[256];
 	/** Lowercase to ., everything else unchanged */
 	public static final byte[] lowerCaseToNocallAA=new byte[256];
+	/** Non-acgtACGT alphabet letters to N */
+	public static final byte[] iupacToNocall=new byte[256];
 	
 	/** Element i is the bitwise OR of constituent IUPAC base numbers in baseToNumber.<br>
 	 * For example, baseToNumberExtended['M'] = (baseToNumber['A'] | baseToNumber['C']) = (1 | 2) = 3 <br>
@@ -301,7 +303,11 @@ public final class AminoAcid {
 	}
 	public static final void complementBasesInPlace(final byte[] in){
 		if(in==null){return;}
-		for(int i=0; i<in.length; i++){
+		complementBasesInPlace(in, in.length);
+	}
+	public static final void complementBasesInPlace(final byte[] in, final int length){
+		if(in==null){return;}
+		for(int i=0; i<length; i++){
 			in[i]=baseToComplementExtended[in[i]];
 		}
 	}
@@ -522,6 +528,15 @@ public final class AminoAcid {
 		}
 		return x;
 	}
+	
+	public static int countDefined(byte[] s){
+		if(s==null){return 0;}
+		int x=0;
+		for(int i=0; i<s.length; i++){
+			if(isFullyDefined(s[i])){x++;}
+		}
+		return x;
+	}
 
 	public static final byte toNumber(String s){
 		assert(s.length()==3);
@@ -710,7 +725,10 @@ public final class AminoAcid {
 		tToU['t']='u';
 		tToU['T']='U';
 		
-		for(int i=0; i<dotDashXToNocall.length; i++){dotDashXToNocall[i]=(byte)i;}
+		for(int i=0; i<dotDashXToNocall.length; i++){
+			dotDashXToNocall[i]=(byte)i;
+			iupacToNocall[i]=(byte)i;
+		}
 		dotDashXToNocall['.']='N';
 		dotDashXToNocall['-']='N';
 		dotDashXToNocall['X']='N';
@@ -903,6 +921,12 @@ public final class AminoAcid {
 			codonToString[i]=kmerToString(i, 3);
 		}
 		
+		for(int i='A'; i<='z'; i++){
+			if(baseToNumber[i]<0 && baseToNumberExtended[i]>=0){
+				iupacToNocall[i]='N';
+			}
+		}
+		
 	}
 	
 }
