@@ -1190,7 +1190,9 @@ public class Data {
 		}
 	}
 	
-	public static final String findPath(String fname){
+	public static final String findPath(String fname){return findPath(fname, true);}
+	
+	public static final String findPath(String fname, boolean warn){
 		assert(fname!=null);
 		if(fname.startsWith("?")){//Look in standard locations
 			fname=fname.substring(1);
@@ -1239,8 +1241,10 @@ public class Data {
 				else{if(vb){System.err.println("Did not find "+fname+" at "+hardlink);}}
 			}
 			if(!f.exists() && !path.startsWith("jar:")){
-				System.err.println("Warning!  Cannot find "+fname+" "+path);
-				new Exception().printStackTrace();
+				if(warn){
+					System.err.println("Warning!  Cannot find "+fname+" "+path);
+					new Exception().printStackTrace();
+				}
 				return null;
 			}
 		}
@@ -1523,7 +1527,10 @@ public class Data {
 										BGZIP_VERSION_threadsFlag=false;
 										BGZIP_VERSION_levelFlag=false;
 										
-										String[] ss=BGZIP_VERSION.split("\\.");
+										String[] ss=BGZIP_VERSION.split("[-\\.]");
+										
+//										System.err.println(Arrays.toString("1.7-8".split("[-\\.]")));
+										
 										int[] is=new int[3];
 										for(int i=0; i<3 && i<ss.length; i++){
 											String s=ss[i];
