@@ -341,6 +341,7 @@ public final class FileFormat {
 		else if(ext.equals("embl")){r[0]=EMBL;}
 		else if(ext.equals("gbk")){r[0]=GBK;}
 		else if(ext.equals("gbff")){r[0]=GBFF;}
+		else if(ext.equals("alm")){r[0]=ALM;}
 		
 		if(comp!=null){
 			r[1]=Tools.find(comp, COMPRESSION_ARRAY);
@@ -714,6 +715,7 @@ public final class FileFormat {
 	public final boolean embl(){return format==EMBL;}
 	public final boolean gbk(){return format==GBK;}
 	public final boolean gbff(){return format==GBFF;}
+	public final boolean alm(){return format==ALM;}
 	
 	public final boolean preferShreds(){
 		return preferShreds;
@@ -832,13 +834,15 @@ public final class FileFormat {
 	public static final int EMBL=27;
 	public static final int GBK=28;
 	public static final int GBFF=29;//TODO: this may be the same as GBK...
+	//Alignment Model, from Consensus package
+	public static final int ALM=30;
 	
 	public static final String[] FORMAT_ARRAY=new String[] {
 		"unknown", "fasta", "fastq", "bread", "sam", "csfasta",
 		"qual", "sequential", "random", "sites", "attachment",
 		"bam", "scarf", "text", "phylip", "header", "int1d",
 		"long1d", "bitset", "sketch", "oneline", "fastr",
-		"vcf", "var", "gff", "bed", "pgm", "embl", "gbk", "gbff"
+		"vcf", "var", "gff", "bed", "pgm", "embl", "gbk", "gbff", "alm"
 	};
 	
 	public static final String[] EXTENSION_LIST=new String[] {
@@ -848,7 +852,7 @@ public final class FileFormat {
 		"scarf", "phylip", "txt",
 		"gz", "gzip", "bz2", "zip", "xz", "dsrc", "header", "headers",
 		"int1d", "long1d", "bitset", "sketch", "oneline", "flat", "fqz",
-		"gff", "gff3", "var", "vcf", "bed", "pgm", "embl", "gbk", "gbff"
+		"gff", "gff3", "var", "vcf", "bed", "pgm", "embl", "gbk", "gbff", "alm"
 	};
 	
 	/* Compression */
