@@ -138,6 +138,8 @@ public class AnalyzeFlowCell {
 			
 			if(a.equals("verbose")){
 				verbose=Tools.parseBoolean(b);
+			}else if(a.equals("seed")){
+				seed=Long.parseLong(b);
 			}else if(a.equals("divisor") || a.equals("size")){
 				Tile.xSize=Tile.ySize=Integer.parseInt(b);
 			}else if(a.equals("xdivisor") || a.equals("xsize")){
@@ -991,8 +993,9 @@ public class AnalyzeFlowCell {
 	private float minprob=0;
 	private static final int WAYS=31;
 	private static final int k=31, k2=30;
+	private long seed=-1;
 	
-	private final Random randy=Shared.threadLocalRandom();
+	private final Random randy=Shared.threadLocalRandom(seed);
 	private FlowCell flowcell;
 	
 //	private int[] avgQualityArray;
