@@ -255,7 +255,7 @@ public final class AssemblyStats2 {
 			try {
 				ff=FileFormat.testInput(in, FileFormat.FA, null, false, true, true);
 			} catch (Throwable e) {
-				//Ignore
+				e.printStackTrace();
 			}
 			if(ff!=null){
 				fastqMode=ff.fastq();
@@ -1587,13 +1587,13 @@ public final class AssemblyStats2 {
 //			sb.append("ctg_L50\t");
 			sb.append("scaf_max\t");
 //			sb.append("ctg_max\t");
-			sb.append("scaf_n_gt50K\t");
+			sb.append("scaf_n_gt50K");
 			if(printExtended){
-				sb.append("scaf_l_gt50k\t");
+				sb.append("\tscaf_l_gt50k");
 			}
-			sb.append("scaf_pct_gt50K");
-			sb.append("gc_avg");
-			sb.append("gc_std");
+			sb.append("\tscaf_pct_gt50K");
+			sb.append("\tgc_avg");
+			sb.append("\tgc_std");
 			if(addfilename){sb.append("\tfilename");}
 
 			sb.append("\n");
@@ -1622,14 +1622,14 @@ public final class AssemblyStats2 {
 //			sb.append(formatKB(cn50, 3, 0)+"\t");
 			sb.append(formatKB(maxScaf, 3, 0)+"\t");
 //			sb.append(formatKB(maxContig, 3, 0)+"\t");
-			sb.append(numOver50+"\t");
+			sb.append(numOver50);
 			if(printExtended){
-				sb.append(basesOver50+"\t");
+				sb.append('\t').append(basesOver50);
 			}
-			sb.append(String.format(Locale.ROOT, "%.3f", fractionOver50));
+			sb.append(String.format(Locale.ROOT, "\t%.3f", fractionOver50));
 
-			sb.append(String.format(Locale.ROOT, "%.5f", (counts[1]+counts[2])*1.0/(counts[0]+counts[1]+counts[2]+counts[3]))+"\t");
-			sb.append(String.format(Locale.ROOT, "%.5f", gc_std));
+			sb.append(String.format(Locale.ROOT, "\t%.5f", (counts[1]+counts[2])*1.0/(counts[0]+counts[1]+counts[2]+counts[3]))+"\t");
+			sb.append(String.format(Locale.ROOT, "\t%.5f", gc_std));
 			
 			if(addfilename){sb.append('\t').append(name);}
 			tsw.println(sb);
