@@ -328,6 +328,7 @@ public class CoveragePileup {
 		refBases=0;
 		mappedBases=0;
 		mappedNonClippedBases=0;
+		mappedBasesWithDels=0;
 		mappedReads=0;
 		properPairs=0;
 		readsProcessed=0;
@@ -345,6 +346,7 @@ public class CoveragePileup {
 		refBases=0;
 		mappedBases=0;
 		mappedNonClippedBases=0;
+		mappedBasesWithDels=0;
 		mappedReads=0;
 		properPairs=0;
 		readsProcessed=0;
@@ -390,7 +392,9 @@ public class CoveragePileup {
 		createDataStructures();
 
 //		System.err.println("A");
-		if(Data.SAMTOOLS() && useStreamer){ReadWrite.USE_SAMBAMBA=false;} //Disable because it takes forever to read the header
+		if(in1!=null && FileFormat.isBamFile(in1)){
+			if(Data.SAMTOOLS() && useStreamer){ReadWrite.USE_SAMBAMBA=false;} //Disable because it takes forever to read the header
+		}
 		ByteFile tf=ByteFile.makeByteFile(in1, false);
 
 //		System.err.println("B");
@@ -625,6 +629,7 @@ public class CoveragePileup {
 			String pound=(headerPound ? "#" : "");
 			tsw.print(pound+"mappedBases="+mappedBases+"\n");
 			tsw.print(pound+"mappedNonClippedBases="+mappedNonClippedBases+"\n");
+			tsw.print(pound+"mappedBasesWithDels="+mappedBasesWithDels+"\n");
 			tsw.print(pound+"mappedReads="+mappedReads+"\n");
 			tsw.print(pound+"name\tlength\tdepthSum\tavgDepth\tavgDepth/mappedBases\tminDepth\tmaxDepth\tmedianDepth\tstdDevDepth\tfractionCovered\n");
 		}
@@ -713,8 +718,8 @@ public class CoveragePileup {
 	/*----------------         Inner Methods        ----------------*/
 	/*--------------------------------------------------------------*/
 	
-	public boolean addCoverage(final String scafName, final byte[] seq, byte[] match, final int start0, final int stop0, final int readlen, final int strand, int incrementFrags,
-			boolean properPair, SamLine sl){//sl is optional
+	public boolean addCoverage(final String scafName, final byte[] seq, byte[] match, final int start0, final int stop0, final int readlen, 
+			final int nonClippedBases, final int strand, int incrementFrags, boolean properPair, SamLine sl){//sl is optional
 		Scaffold scaf=table.get(scafName);
 		if(scaf==null){
 			if(ADD_FROM_READS){
@@ -728,7 +733,7 @@ public class CoveragePileup {
 				if(COUNT_GC){scaf.basecount=new long[8];}
 				table.put(scafName, scaf);
 				list.add(scaf);
-				return addCoverage(scaf, seq, match, start0, stop0, readlen, strand, incrementFrags, properPair, sl);
+				return addCoverage(scaf, seq, match, start0, stop0, readlen, nonClippedBases, strand, incrementFrags, properPair, sl);
 			}else if(EA){
 				KillSwitch.kill("ERROR: A read was mapped to unknown reference sequence "+scafName);
 			}else if(!warned){
@@ -738,11 +743,11 @@ public class CoveragePileup {
 			}
 			return false;
 		}
-		return addCoverage(scaf, seq, match, start0, stop0, readlen, strand, incrementFrags, properPair, sl);
+		return addCoverage(scaf, seq, match, start0, stop0, readlen, nonClippedBases, strand, incrementFrags, properPair, sl);
 	}
 	
-	public boolean addCoverage(final Scaffold scaf, final byte[] seq, byte match[], final int start0, final int stop0, final int readlen, final int strand, int incrementFrags,
-			boolean properPair, SamLine sl){//sl is optional
+	public boolean addCoverage(final Scaffold scaf, final byte[] seq, byte match[], final int start0, final int stop0, final int readlen, final int nonClippedBases, 
+			final int strand, int incrementFrags, boolean properPair, SamLine sl){//sl is optional
 		if(scaf==null){
 			assert(false) : "Adding coverage to a null Scaffold.";
 			return false;
@@ -764,7 +769,8 @@ public class CoveragePileup {
 			"\nstop="+stop+"\nreadlen="+readlen+"\nstrand="+strand+"\nscaf.length="+scaf.length+"\nscaf="+scaf;
 		
 		mappedBases+=readlen;
-		mappedNonClippedBases+=(stop-start+1);
+		mappedNonClippedBases+=nonClippedBases;
+		mappedBasesWithDels+=(stop-start+1);
 		mappedReads++;
 		if(properPair){properPairs++;}
 
@@ -934,7 +940,7 @@ public class CoveragePileup {
 				final int strand=(sl.pairnum()==0 ? sl.strand() : mate.strand());
 				final int length=USE_TLEN ? sl.tlen : Tools.max(stop1, stop2)-Tools.min(start1, start2)+1;
 				mappedKmers+=kmers;
-				addCoverage(sl.rnameS(), null, null, Tools.min(start1, start2), Tools.max(stop1, stop2), length, strand, 2, sl.properPair(), sl);
+				addCoverage(sl.rnameS(), null, null, Tools.min(start1, start2), Tools.max(stop1, stop2), length, sl.mappedNonClippedBases(), strand, 2, sl.properPair(), sl);
 			}
 		}else if(sl.mapped() && (USE_SECONDARY || sl.primary()) && sl.mapq>=minMapq){
 			assert(sl.seq!=null || sl.cigar!=null) : "This program requires bases or a cigar string for every sam line.  Problem line:\n"+sl+"\n";
@@ -947,7 +953,7 @@ public class CoveragePileup {
 //			assert(false) : "'"+sl.rnameS()+"'";
 			final byte[] match=(INCLUDE_DELETIONS ? null : sl.toShortMatch(true));
 			mappedKmers+=kmers;
-			return addCoverage(sl.rnameS(), sl.seq, match, start, stop, length, sl.strand(), sl.hasMate() ? 1 : 2, sl.properPair(), sl);
+			return addCoverage(sl.rnameS(), sl.seq, match, start, stop, length, sl.mappedNonClippedBases(), sl.strand(), sl.hasMate() ? 1 : 2, sl.properPair(), sl);
 		}
 		return false;
 	}
@@ -978,11 +984,13 @@ public class CoveragePileup {
 					final int strand=r.strand();
 					final int length=Tools.max(stop1, stop2)-Tools.min(start1, start2)+1;
 					mappedKmers+=kmers;
-					addCoverage(new String(coords.name), null, null, Tools.min(start1, start2), Tools.max(stop1, stop2), length, strand, 2-r.mateCount(), r.paired(), null);
+					addCoverage(new String(coords.name), null, null, Tools.min(start1, start2), Tools.max(stop1, stop2), length, 
+							r.mappedNonClippedBases(), strand, 2-r.mateCount(), r.paired(), null);
 				}else{
 					if(set1){
 						mappedKmers+=kmers;
-						return addCoverage(new String(coords.name), r.bases, r.match, coords.start, coords.stop, r.length(), coords.strand, 2-r.mateCount(), r.paired(), null);
+						return addCoverage(new String(coords.name), r.bases, r.match, coords.start, coords.stop, r.length(), 
+								r.mappedNonClippedBases(), coords.strand, 2-r.mateCount(), r.paired(), null);
 					}
 				}
 			}
@@ -994,7 +1002,8 @@ public class CoveragePileup {
 	public boolean processRead(Read r, SiteScore ss){
 		if(ss!=null && r.bases!=null){
 			if(coords.set(ss)){
-				return addCoverage(new String(coords.name), r.bases, ss.match, coords.start, coords.stop, r.length(), coords.strand, 1, r.paired(), null);
+				return addCoverage(new String(coords.name), r.bases, ss.match, coords.start, coords.stop, r.length(), 
+						r.mappedNonClippedBases(), coords.strand, 1, r.paired(), null);
 			}
 		}
 		return false;
@@ -1070,6 +1079,7 @@ public class CoveragePileup {
 		final double mult=1.0/refBases;
 		double depthCovered=mappedBases*mult;
 		double depthCovered2=mappedNonClippedBases*mult;
+		double depthCovered3=mappedBasesWithDels*mult;
 		double pctScaffoldsWithCoverage=scaffoldsWithCoverage1*100.0/totalScaffolds;
 		double pctCovered=totalCoveredBases1*100*mult;
 		
@@ -1094,7 +1104,9 @@ public class CoveragePileup {
 
 			outstream.println(String.format(Locale.ROOT, "\nPercent mapped:                      \t%.3f", mappedReads*100f/readsProcessed));
 			outstream.println(String.format(Locale.ROOT, "Percent proper pairs:                \t%.3f", properPairs*100f/readsProcessed));
+//			outstream.println(depthCovered);
 			outstream.println(String.format(Locale.ROOT, "Average coverage:                    \t%.3f", depthCovered2));
+			outstream.println(String.format(Locale.ROOT, "Average coverage with deletions:     \t%.3f", depthCovered3));
 			if(USE_COVERAGE_ARRAYS && calcCovStdev){
 				double[] stdev=standardDeviation(list, 0, minscaf);
 				outstream.println(String.format(Locale.ROOT, "Standard deviation:                    \t%.3f", stdev[1]));
@@ -1107,6 +1119,7 @@ public class CoveragePileup {
 			outstream.println("reads="+readsProcessed);
 			outstream.println("mappedReads="+mappedReads);
 			outstream.println("mappedBases="+mappedNonClippedBases);
+			outstream.println("mappedBasesWithDels="+mappedBasesWithDels);
 			outstream.println("refScaffolds="+totalScaffolds);
 			outstream.println("refBases="+refBases);
 			outstream.println(String.format(Locale.ROOT, "percentMapped=%.3f", mappedReads*100f/readsProcessed));
@@ -1952,6 +1965,7 @@ public class CoveragePileup {
 	public long refBases=0;
 	public long mappedBases=0;
 	public long mappedNonClippedBases=0;
+	public long mappedBasesWithDels=0;
 	public long mappedReads=0;
 	public long properPairs=0;
 	public long readsProcessed=0;
