@@ -138,7 +138,17 @@ public class ReformatReads {
 			}else if(a.equals("comp") || a.equals("complement")){
 				complement=Tools.parseBoolean(b);
 				outstream.println("Warning: Setting non-standard parameter 'complement'.");
-			}else if(a.equals("deleteempty") || a.equals("deletempty") || a.equals("delempty") || a.equals("def")){
+			}
+			
+			//Unneeded due to pad flag
+//			else if(a.equals("npad") || a.equals("padns")){
+//				assert(b!=null) : "npad requires an int value, e.g. npad=10";
+//				assert(Character.isDigit(b.charAt(0)) || b.charAt(0)=='f') : "npad requires an int value, e.g. npad=10";
+//				if(Character.isDigit(b.charAt(0)){npad=Integer.parseInt(b);}
+//				else {npad=0;}
+//			}
+			
+			else if(a.equals("deleteempty") || a.equals("deletempty") || a.equals("delempty") || a.equals("def")){
 				deleteEmptyFiles=Tools.parseBoolean(b);
 			}else if(a.equals("mappedonly")){
 				mappedOnly=Tools.parseBoolean(b);
@@ -256,8 +266,9 @@ public class ReformatReads {
 			usePairGC=parser.usePairGC;
 			tossJunk=parser.tossJunk;
 			recalibrateQuality=parser.recalibrateQuality;
-			
-			idFilter=parser.idFilter;
+
+			minIdFilter=parser.minIdFilter;
+			maxIdFilter=parser.maxIdFilter;
 			subfilter=parser.subfilter;
 			clipfilter=parser.clipfilter;
 			delfilter=parser.delfilter;
@@ -286,7 +297,7 @@ public class ReformatReads {
 			extout=parser.extout;
 			
 			loglog=(parser.loglog ? new LogLog(parser) : null);
-			silent=parser.silent;
+			silent=Parser.silent;
 		}
 		
 		if(recalibrateQuality){CalcTrueQuality.initializeMatrices();}
@@ -750,10 +761,10 @@ public class ReformatReads {
 						}
 					}
 					
-					if(idFilter>=0 || USE_EDIT_FILTER){
+					if(minIdFilter>=0 || maxIdFilter<1 || USE_EDIT_FILTER){
 						if(r1!=null && !r1.discarded()){
 							assert(r1.match!=null || r1.samline!=null) : "idfilter requires sam/bam input.";
-							boolean pass=passesIDFilter(r1, idFilter, false);
+							boolean pass=passesIDFilter(r1, minIdFilter, maxIdFilter, false);
 							if(USE_EDIT_FILTER){
 								pass=pass&&passesEditFilter(r1, false);
 							}
@@ -765,7 +776,7 @@ public class ReformatReads {
 						}
 						if(r2!=null && !r2.discarded()){
 							assert(r2.match!=null || r2.samline!=null) : "idfilter requires sam/bam input.";
-							boolean pass=passesIDFilter(r2, idFilter, false);
+							boolean pass=passesIDFilter(r2, minIdFilter, maxIdFilter, false);
 							if(USE_EDIT_FILTER){
 								pass=pass&&passesEditFilter(r2, false);
 							}
@@ -1204,7 +1215,7 @@ public class ReformatReads {
 			outstream.println("Low quality discards:   \t"+lowqReadsT+" reads ("+String.format(Locale.ROOT, "%.2f",lowqReadsT*rpmult)+"%) \t"+
 					lowqBasesT+" bases ("+String.format(Locale.ROOT, "%.2f",lowqBasesT*bpmult)+"%)");
 		}
-		if(idFilter>=0 || USE_EDIT_FILTER){
+		if(minIdFilter>=0 || maxIdFilter<1 || USE_EDIT_FILTER){
 			outstream.println("Identity/edit discards: \t"+idfilteredReadsT+" reads ("+String.format(Locale.ROOT, "%.2f",idfilteredReadsT*rpmult)+"%) \t"+
 					idfilteredBasesT+" bases ("+String.format(Locale.ROOT, "%.2f",idfilteredBasesT*bpmult)+"%)");
 		}
@@ -1280,9 +1291,13 @@ public class ReformatReads {
 		for(int i=0; i<old.length; i++){array[i+padLeft]=old[i];}
 		return array;
 	}
-		
-		
-	public static final boolean passesIDFilter(Read r, float minId, boolean requireMapped){
+	
+	public static final boolean passesIDFilter(Read r, float minId, float maxId, boolean requireMapped){
+		if(!passesMinIDFilter(r, minId, requireMapped)){return false;}
+		return passesMaxIDFilter(r, maxId);
+	}
+	
+	public static final boolean passesMinIDFilter(Read r, float minId, boolean requireMapped){
 		if(minId<=0 || r.perfect()){return true;}
 		if(r.match==null && r.samline!=null){
 			r.match=r.samline.toShortMatch(false);
@@ -1291,6 +1306,15 @@ public class ReformatReads {
 		return Read.identityFlat(r.match, true)>=minId;
 	}
 	
+	public static final boolean passesMaxIDFilter(Read r, float maxId){
+		if(maxId>=1){return true;}
+		if(r.match==null && r.samline!=null){
+			r.match=r.samline.toShortMatch(false);
+		}
+		if(r.match==null){return true;}
+		return Read.identityFlat(r.match, true)<=maxId;
+	}
+	
 	public final boolean passesEditFilter(Read r, boolean requireMapped){
 		if(r.perfect()){return true;}
 		if(r.match==null && r.samline!=null){
@@ -1547,8 +1571,9 @@ public class ReformatReads {
 	private boolean addunderscore=false;
 	private boolean stoptag=false;
 	private boolean iupacToN=false;
-	
-	private float idFilter=-1;
+
+	private float minIdFilter=-1;
+	private float maxIdFilter=-1;
 	private int subfilter=-1;
 	private int clipfilter=-1;
 	private int delfilter=-1;
