@@ -16,6 +16,7 @@ import stream.ConcurrentReadInputStream;
 import stream.ConcurrentReadOutputStream;
 import stream.Read;
 import structures.ListNum;
+import template.BBTool_ST;
 
 /**
  * @author Brian Bushnell
@@ -31,7 +32,7 @@ public class SplitNexteraLMP extends BBTool_ST {
 	}
 	
 	@Override
-	void setDefaults(){
+	protected void setDefaults(){
 		outStats="stderr";
 		minReadLength=40;
 		mask=false;
@@ -149,7 +150,7 @@ public class SplitNexteraLMP extends BBTool_ST {
 	}
 	
 	@Override
-	void startupSubclass(){
+	protected void startupSubclass(){
 		
 		if(!Tools.testOutputFiles(overwrite, append, false, out1, out2, outFrag1, outFrag2, outUnk1, outUnk2)){
 			throw new RuntimeException("\noverwrite="+overwrite+", append="+append+"\n" +
@@ -195,7 +196,7 @@ public class SplitNexteraLMP extends BBTool_ST {
 	@Override
 	/** Iterate through the reads.
 	 * This may optionally be overridden. */
-	void processInner(final ConcurrentReadInputStream cris, final ConcurrentReadOutputStream rosLmp){
+	protected void processInner(final ConcurrentReadInputStream cris, final ConcurrentReadOutputStream rosLmp){
 		
 		if(mask){
 			final TableLoaderLockFree loader=new TableLoaderLockFree(tables, k, mink, 0, hdist, edist, rcomp, maskMiddle);
@@ -316,7 +317,7 @@ public class SplitNexteraLMP extends BBTool_ST {
 	}
 	
 	@Override
-	void showStatsSubclass(final Timer t, long readsIn, long basesIn){
+	protected void showStatsSubclass(final Timer t, long readsIn, long basesIn){
 		
 		TextStreamWriter tsw=new TextStreamWriter(outStats==null ? "stderr" : outStats, overwrite, append, false);
 		tsw.start();
@@ -350,7 +351,7 @@ public class SplitNexteraLMP extends BBTool_ST {
 	}
 	
 	@Override
-	boolean processReadPair(Read r1, Read r2) {
+	protected boolean processReadPair(Read r1, Read r2) {
 		throw new RuntimeException("Do not use.");
 	}
 	
@@ -591,7 +592,7 @@ public class SplitNexteraLMP extends BBTool_ST {
 	}
 	
 	@Override
-	void shutdownSubclass(){
+	protected void shutdownSubclass(){
 		errorState|=ReadWrite.closeStreams(null, rosFrag, rosUnk, rosSingle);
 	}
 	
