@@ -500,6 +500,11 @@ public class KmerTableSet extends AbstractKmerTableSet {
 						basesInT+=r2.length();
 					}
 					
+					if(maxNs<Integer.MAX_VALUE){
+						if(r1!=null && r1.countUndefined()>maxNs){r1.setDiscarded(true);}
+						if(r2!=null && r2.countUndefined()>maxNs){r2.setDiscarded(true);}
+					}
+					
 					//Determine whether to discard the reads based on average quality
 					if(minAvgQuality>0){
 						if(r1!=null && r1.quality!=null && r1.avgQuality(false, minAvgQualityBases)<minAvgQuality){r1.setDiscarded(true);}
@@ -535,6 +540,11 @@ public class KmerTableSet extends AbstractKmerTableSet {
 							BBMerge.findOverlapStrict(r1, r2, true);
 						}
 					}
+					
+					if(minLen>0){
+						if(r1!=null && r1.length()<minLen){r1.setDiscarded(true);}
+						if(r2!=null && r2.length()<minLen){r2.setDiscarded(true);}
+					}
 
 					if(r1!=null){
 						if(r1.discarded()){
