@@ -3,6 +3,8 @@ package pacbio;
 import java.util.ArrayList;
 import java.util.Locale;
 
+import aligner.MultiStateAligner9PacBioAdapter;
+import aligner.MultiStateAligner9PacBioAdapter2;
 import dna.AminoAcid;
 import dna.Data;
 import fileIO.FileFormat;
@@ -278,7 +280,7 @@ public class RemoveAdapters2 {
 			assert(window<ALIGN_COLUMNS);
 
 			msa=new MultiStateAligner9PacBioAdapter(ALIGN_ROWS, ALIGN_COLUMNS);
-			msa2=USE_ALT_MSA ? new MultiStateAligner9PacBioAdapter2(ALIGN_ROWS, ALIGN_COLUMNS) : null;
+			msa2=USE_ALT_MSA ? new MultiStateAligner9PacBioAdapter2() : null;
 
 			maxSwScore=msa.maxQuality(query1.length);
 			minSwScore=(int)(maxSwScore*MINIMUM_ALIGNMENT_SCORE_RATIO);
