@@ -3,6 +3,7 @@ package prok;
 import java.io.File;
 import java.io.PrintStream;
 import java.util.ArrayList;
+import java.util.Arrays;
 
 import dna.AminoAcid;
 import dna.Data;
@@ -12,6 +13,7 @@ import fileIO.FileFormat;
 import fileIO.ReadWrite;
 import gff.CompareGff;
 import jgi.BBMerge;
+import json.JsonObject;
 import shared.Parser;
 import shared.PreParser;
 import shared.ReadStats;
@@ -30,7 +32,7 @@ import structures.ListNum;
  * @date Sep 24, 2018
  *
  */
-public class CallGenes {
+public class CallGenes extends ProkObject {
 	
 	/*--------------------------------------------------------------*/
 	/*----------------        Initialization        ----------------*/
@@ -61,7 +63,7 @@ public class CallGenes {
 	public CallGenes(String[] args){
 		
 		{//Preparse block for help, config files, and outstream
-			PreParser pp=new PreParser(args, getClass(), false);
+			PreParser pp=new PreParser(args, (args.length>40 ? null : getClass()), false);
 			args=pp.args;
 			outstream=pp.outstream;
 		}
@@ -74,7 +76,7 @@ public class CallGenes {
 			final Parser parser=parse(args);
 			overwrite=parser.overwrite;
 			append=parser.append;
-
+			
 			outGff=parser.out1;
 			maxReads=parser.maxReads;
 		}
@@ -82,16 +84,31 @@ public class CallGenes {
 		fixExtensions(); //Add or remove .gz or .bz2 as needed
 		checkFileExistence(); //Ensure files can be read and written
 		checkStatics(); //Adjust file-related static fields as needed for this program
-
+		
 		ffoutGff=FileFormat.testOutput(outGff, FileFormat.GFF, null, true, overwrite, append, ordered);
 		ffoutAmino=FileFormat.testOutput(outAmino, FileFormat.FA, null, true, overwrite, append, ordered);
-
+		ffout16S=FileFormat.testOutput(out16S, FileFormat.FA, null, true, overwrite, append, ordered);
+		ffout18S=FileFormat.testOutput(out18S, FileFormat.FA, null, true, overwrite, append, ordered);
+		
 		if(ffoutGff!=null){
 			assert(!ffoutGff.isSequence()) : "\nout is for gff files.  To output sequence, please use outa.";
 		}
 		if(ffoutAmino!=null){
 			assert(!ffoutAmino.gff()) : "\nouta is for sequence data.  To output gff, please use out.";
 		}
+		if(ffout16S!=null){
+			assert(!ffout16S.gff()) : "\nout16S is for sequence data.  To output gff, please use out.";
+		}
+		if(ffout18S!=null){
+			assert(!ffout18S.gff()) : "\nout18S is for sequence data.  To output gff, please use out.";
+		}
+		
+		if(geneHistFile==null){geneHistBins=0;}
+		else{
+			assert(geneHistBins>1) : "geneHistBins="+geneHistBins+"; should be >1";
+			assert(geneHistDiv>=1) : "geneHistDiv="+geneHistDiv+"; should be >=1";
+		}
+		geneHist=geneHistBins>1 ? new long[geneHistBins] : null;
 	}
 	
 	/*--------------------------------------------------------------*/
@@ -129,14 +146,28 @@ public class CallGenes {
 				Tools.addFiles(arg, pgmList);
 			}else if(a.equals("outamino") || a.equals("aminoout") || a.equals("outa") || a.equals("outaa") || a.equals("aaout") || a.equals("amino")){
 				outAmino=b;
+			}else if(a.equalsIgnoreCase("out16s") || a.equalsIgnoreCase("16sout")){
+				out16S=b;
+			}else if(a.equalsIgnoreCase("out18s") || a.equalsIgnoreCase("18sout")){
+				out18S=b;
 			}else if(a.equals("verbose")){
 				verbose=Tools.parseBoolean(b);
 				//ReadWrite.verbose=verbose;
 				GeneCaller.verbose=verbose;
-			}else if(a.equals("plus")){
-				GeneModel.PROCESS_PLUS_STRAND=Tools.parseBoolean(b);
-			}else if(a.equals("minus")){
-				GeneModel.PROCESS_MINUS_STRAND=Tools.parseBoolean(b);
+			}
+			
+			else if(a.equals("json_out") || a.equalsIgnoreCase("json")){
+				json_out=Tools.parseBoolean(b);
+			}else if(a.equals("stats") || a.equalsIgnoreCase("outstats")){
+				outStats=b;
+			}else if(a.equals("hist") || a.equalsIgnoreCase("outhist") || a.equalsIgnoreCase("lengthhist") || a.equalsIgnoreCase("lhist") || a.equalsIgnoreCase("genehist")){
+				geneHistFile=b;
+			}else if(a.equals("bins")){
+				geneHistBins=Integer.parseInt(b);
+			}else if(a.equals("binlen") || a.equals("binlength") || a.equals("histdiv")){
+				geneHistBins=Integer.parseInt(b);
+			}else if(a.equals("printzero") || a.equals("pz")){
+				printZeroCountHist=Tools.parseBoolean(b);
 			}
 			
 			else if(a.equals("merge")){
@@ -144,28 +175,19 @@ public class CallGenes {
 			}else if(a.equals("ecco")){
 				ecco=Tools.parseBoolean(b);
 			}
-			
-			else if(a.equalsIgnoreCase("load16skmers")){
-				GeneModel.load16Skmers=Tools.parseBoolean(b);
-			}else if(a.equalsIgnoreCase("load23skmers")){
-				GeneModel.load23Skmers=Tools.parseBoolean(b);
-			}else if(a.equalsIgnoreCase("load5skmers")){
-				GeneModel.load5Skmers=Tools.parseBoolean(b);
-			}else if(a.equalsIgnoreCase("loadtrnakmers")){
-				GeneModel.loadtRNAkmers=Tools.parseBoolean(b);
-			}else if(a.equalsIgnoreCase("klongtrna")){
-				GeneModel.kLongTRna=Integer.parseInt(b);
-			}else if(a.equalsIgnoreCase("longkmers")){
-				GeneModel.load16Skmers=GeneModel.load23Skmers=
-						GeneModel.load5Skmers=GeneModel.loadtRNAkmers=Tools.parseBoolean(b);
-			}else if(a.equalsIgnoreCase("klong5s")){
-				GeneModel.kLong5s=Integer.parseInt(b);
-			}else if(a.equalsIgnoreCase("klong16s")){
-				GeneModel.kLong16s=Integer.parseInt(b);
-			}else if(a.equalsIgnoreCase("klong23s")){
-				GeneModel.kLong23s=Integer.parseInt(b);
-			}else if(a.equalsIgnoreCase("klongtrna")){
-				GeneModel.kLongTRna=Integer.parseInt(b);
+
+			else if(a.equalsIgnoreCase("setbias16s")) {
+				GeneCaller.biases[r16S]=Float.parseFloat(b);
+			}else if(a.equalsIgnoreCase("setbias18s")) {
+				GeneCaller.biases[r18S]=Float.parseFloat(b);
+			}else if(a.equalsIgnoreCase("setbias23s")) {
+				GeneCaller.biases[r23S]=Float.parseFloat(b);
+			}else if(a.equalsIgnoreCase("setbias5s")) {
+				GeneCaller.biases[r5S]=Float.parseFloat(b);
+			}else if(a.equalsIgnoreCase("setbiastRNA")) {
+				GeneCaller.biases[tRNA]=Float.parseFloat(b);
+			}else if(a.equalsIgnoreCase("setbiasCDS")) {
+				GeneCaller.biases[CDS]=Float.parseFloat(b);
 			}
 			
 			else if(a.equals("ordered")){
@@ -180,11 +202,11 @@ public class CallGenes {
 				mode=RECODE;
 			}
 			
-			else if(a.equals("minlen") || a.equals("minlength")){
+			else if(a.equalsIgnoreCase("minlen") || a.equals("minlength")){
 				minLen=Integer.parseInt(b);
-			}else if(a.equals("maxoverlapss") || a.equals("overlapss") || a.equals("overlapsamestrand") || a.equals("moss")){
+			}else if(a.equals("maxoverlapss") || a.equals("overlapss") || a.equals("overlapsamestrand") || a.equals("moss") || a.equalsIgnoreCase("maxOverlapSameStrand")){
 				maxOverlapSameStrand=Integer.parseInt(b);
-			}else if(a.equals("maxoverlapos") || a.equals("overlapos") || a.equals("overlapoppositestrand") || a.equals("moos")){
+			}else if(a.equals("maxoverlapos") || a.equals("overlapos") || a.equals("overlapoppositestrand") || a.equals("moos") || a.equalsIgnoreCase("maxOverlapOppositeStrand")){
 				maxOverlapOppositeStrand=Integer.parseInt(b);
 			}else if(a.equalsIgnoreCase("minStartScore")){
 				minStartScore=Float.parseFloat(b);
@@ -259,6 +281,8 @@ public class CallGenes {
 				compareToGff=b;
 			}
 			
+			else if(ProkObject.parse(arg, a, b)){}
+			
 			else if(parser.parse(arg, a, b)){
 				//do nothing
 			}else{
@@ -295,9 +319,10 @@ public class CallGenes {
 	/** Ensure files can be read and written */
 	private void checkFileExistence(){
 		//Ensure output files can be written
-		if(!Tools.testOutputFiles(overwrite, append, false, outGff, outAmino)){
+		if(!Tools.testOutputFiles(overwrite, append, false, outGff, outAmino, out16S, out18S, outStats, geneHistFile)){
 			outstream.println((outGff==null)+", "+outGff);
-			throw new RuntimeException("\n\noverwrite="+overwrite+"; Can't write to output files "+outGff+", "+outAmino+"\n");
+			throw new RuntimeException("\n\noverwrite="+overwrite+"; Can't write to output files "
+					+outGff+", "+outAmino+", "+out16S+", "+out18S+", "+outStats+", "+geneHistFile+"\n");
 		}
 		
 		//Ensure input files can be read
@@ -311,6 +336,10 @@ public class CallGenes {
 		//Ensure that no file was specified multiple times
 		foo.add(outGff);
 		foo.add(outAmino);
+		foo.add(out16S);
+		foo.add(out18S);
+		foo.add(outStats);
+		foo.add(geneHistFile);
 		if(!Tools.testForDuplicateFiles(true, foo.toArray(new String[0]))){
 			throw new RuntimeException("\nSome file names were specified multiple times.\n");
 		}
@@ -333,16 +362,19 @@ public class CallGenes {
 		
 		GeneModel pgm=PGMTools.loadAndMerge(pgmList);
 		
-		if(GeneCaller.call16S || GeneCaller.call23S || GeneCaller.calltRNA || GeneCaller.call5S){
-			pgm.loadLongKmers();
+		if(call16S || call18S || call23S || calltRNA || call5S){
+			loadLongKmers();
+			loadConsensusSequenceFromFile(false, false);
 		}
 		
 		ByteStreamWriter bsw=makeBSW(ffoutGff);
 		if(bsw!=null){
 			bsw.forcePrint("##gff-version 3\n");
 		}
-		
-		ConcurrentReadOutputStream ros=makeCros(ffoutAmino);
+
+		ConcurrentReadOutputStream rosAmino=makeCros(ffoutAmino);
+		ConcurrentReadOutputStream ros16S=makeCros(ffout16S);
+		ConcurrentReadOutputStream ros18S=makeCros(ffout18S);
 		
 		//Turn off read validation in the input threads to increase speed
 		final boolean vic=Read.VALIDATE_IN_CONSTRUCTOR;
@@ -357,12 +389,15 @@ public class CallGenes {
 			final ConcurrentReadInputStream cris=makeCris(fname);
 
 			//Process the reads in separate threads
-			spawnThreads(cris, bsw, ros, pgm);
+			spawnThreads(cris, bsw, rosAmino, ros16S, ros18S, pgm);
 			
 			//Close the input stream
-			errorState|=ReadWrite.closeStreams(cris, ros);
+			errorState|=ReadWrite.closeStream(cris);
 		}
 		
+		//Close the input stream
+		errorState|=ReadWrite.closeStreams(null, rosAmino, ros16S, ros18S);
+		
 		if(verbose){outstream.println("Finished; closing streams.");}
 		
 		//Write anything that was accumulated by ReadStats
@@ -377,25 +412,17 @@ public class CallGenes {
 		t.stop();
 		outstream.println(Tools.timeReadsBasesProcessed(t, readsIn, basesIn, 8));
 		outstream.println(Tools.linesBytesOut(readsIn, basesIn, genesOut, bytesOut, 8, false));
+		outstream.println();
 		
-		outstream.println();
-		outstream.println("Gene Stops Made:      \t "+Tools.padLeft(geneStopsMade, 12));
-		outstream.println("Gene Starts Made:     \t "+Tools.padLeft(geneStartsMade, 12));
-		outstream.println("Gene Starts Retained: \t "+Tools.padLeft(geneStartsRetained, 12));
-		outstream.println("Gene Stops Retained:  \t "+Tools.padLeft(geneStopsRetained, 12));
-		outstream.println("Genes Out:            \t "+Tools.padLeft(geneStartsOut, 12));
-
-		outstream.println();
-		outstream.println("All ORF Stats:");
-		outstream.println(stCds);
+		if(json_out){
+			printStatsJson(outStats);
+		}else{
+			printStats(outStats);
+		}
 		
-		outstream.println();
-		outstream.println("Retained ORF Stats:");
-		outstream.println(stCds2);
-		
-		outstream.println();
-		outstream.println("Called ORF Stats:");
-		outstream.println(stCdsPass);
+		if(geneHistFile!=null){
+			printHist(geneHistFile);
+		}
 		
 		//Throw an exception of there was an error in a thread
 		if(errorState){
@@ -412,6 +439,130 @@ public class CallGenes {
 		}
 	}
 	
+	private void printStats(String fname){
+		if(fname==null){return;}
+		ByteStreamWriter bsw=new ByteStreamWriter(fname, overwrite, append, false);
+		bsw.start();
+		
+		if(callCDS){
+			bsw.println("Gene Starts Made:     \t "+Tools.padLeft(geneStartsMade, 12));
+			bsw.println("Gene Stops Made:      \t "+Tools.padLeft(geneStopsMade, 12));
+			bsw.println("Gene Starts Retained: \t "+Tools.padLeft(geneStartsRetained, 12));
+			bsw.println("Gene Stops Retained:  \t "+Tools.padLeft(geneStopsRetained, 12));
+			bsw.println("CDS Out:              \t "+Tools.padLeft(geneStartsOut, 12));
+		}
+		if(call16S){bsw.println("16S Out:              \t "+Tools.padLeft(r16SOut, 12));}
+		if(call18S){bsw.println("18S Out:              \t "+Tools.padLeft(r18SOut, 12));}
+		if(call23S){bsw.println("23S Out:              \t "+Tools.padLeft(r23SOut, 12));}
+		if(call5S){bsw.println("5S Out:               \t "+Tools.padLeft(r5SOut, 12));}
+		if(calltRNA){bsw.println("tRNA Out:             \t "+Tools.padLeft(tRNAOut, 12));}
+		
+		if(callCDS){
+			bsw.println();
+			bsw.println("All ORF Stats:");
+			bsw.print(stCds.toString());
+
+			bsw.println();
+			bsw.println("Retained ORF Stats:");
+			bsw.print(stCds2.toString());
+
+			bsw.println();
+			bsw.println("Called ORF Stats:");
+			stCdsPass.genomeSize=basesIn;
+			bsw.print(stCdsPass.toString());
+		}
+
+		if(call16S){
+			bsw.println();
+			bsw.println("Called 16S Stats:");
+			bsw.print(st16s.toString());
+		}
+		if(call23S){
+			bsw.println();
+			bsw.println("Called 23S Stats:");
+			bsw.print(st23s.toString());
+		}
+		if(call5S){
+			bsw.println();
+			bsw.println("Called 5S Stats:");
+			bsw.print(st5s.toString());
+		}
+		if(call18S){
+			bsw.println();
+			bsw.println("Called 18S Stats:");
+			bsw.print(st18s.toString());
+		}
+		bsw.poisonAndWait();
+	}
+	
+	private void printStatsJson(String fname){
+		if(fname==null){return;}
+		
+		JsonObject outer=new JsonObject();
+		
+		{
+			JsonObject jo=new JsonObject();
+			if(callCDS){
+				jo.add("Gene Starts Made", geneStartsMade);
+				jo.add("Gene Stops Made", geneStopsMade);
+				jo.add("Gene Starts Retained", geneStartsRetained);
+				jo.add("Gene Stops Retained", geneStopsRetained);
+				jo.add("CDS Out", geneStartsOut);
+			}
+			if(call16S){jo.add("16S Out", r16SOut);}
+			if(call18S){jo.add("18S Out", r18SOut);}
+			if(call23S){jo.add("23S Out", r23SOut);}
+			if(call5S){jo.add("5S Out", r5SOut);}
+			if(calltRNA){jo.add("tRNA Out", tRNAOut);}
+			outer.add("Overall", jo);
+		}
+		
+		if(callCDS){
+			outer.add("All ORF Stats", stCds.toJson());
+			outer.add("Retained ORF Stats", stCds2.toJson());
+			stCdsPass.genomeSize=basesIn;
+			outer.add("Called ORF Stats", stCdsPass.toJson());
+		}
+
+		if(call16S){outer.add("Called 16S Stats", st16s.toJson());}
+		if(call18S){outer.add("Called 18S Stats", st18s.toJson());}
+		if(call23S){outer.add("Called 23S Stats", st23s.toJson());}
+		if(call5S){outer.add("Called 5S Stats", st5s.toJson());}
+		if(calltRNA){outer.add("Called tRNA Stats", sttRNA.toJson());}
+		
+
+		ByteStreamWriter bsw=new ByteStreamWriter(fname, overwrite, append, false);
+		bsw.start();
+		bsw.println(outer.toText());
+		bsw.poisonAndWait();
+	}
+	
+	private void printHist(String fname){
+		if(fname==null || geneHist==null){return;}
+		ByteStreamWriter bsw=new ByteStreamWriter(fname, overwrite, append, false);
+		bsw.start();
+		long sum=Tools.sum(geneHist);
+		double mean=Tools.averageHistogram(geneHist)*geneHistDiv;
+		int median=Tools.medianHistogram(geneHist)*geneHistDiv;
+		double std=Tools.standardDeviationHistogram(geneHist)*geneHistDiv;
+		bsw.println("#Gene Length Histogram");
+		bsw.print("#Genes:\t").println(sum);
+		bsw.print("#Mean:\t").println(mean, 4);
+		bsw.print("#Median:\t").println(median);
+		bsw.print("#STDDev:\t").println(std, 4);
+		bsw.print("#Length\tCount\n");
+		long cum=0;
+		for(int i=0; i<geneHist.length && cum<sum; i++){
+			int len=i*geneHistDiv;
+			long count=geneHist[i];
+			cum+=count;
+			if(count>0 || printZeroCountHist){
+				bsw.print(len).tab().println(count);
+			}
+		}
+		bsw.poisonAndWait();
+	}
+	
 	private ConcurrentReadInputStream makeCris(String fname){
 		FileFormat ffin=FileFormat.testInput(fname, FileFormat.FA, null, true, true);
 		ConcurrentReadInputStream cris=ConcurrentReadInputStream.getReadInputStream(maxReads, false, ffin, null);
@@ -421,7 +572,8 @@ public class CallGenes {
 	}
 	
 	/** Spawn process threads */
-	private void spawnThreads(final ConcurrentReadInputStream cris, final ByteStreamWriter bsw, ConcurrentReadOutputStream ros, GeneModel pgm){
+	private void spawnThreads(final ConcurrentReadInputStream cris, final ByteStreamWriter bsw, 
+			ConcurrentReadOutputStream rosAmino, ConcurrentReadOutputStream ros16S, ConcurrentReadOutputStream ros18S, GeneModel pgm){
 		
 		//Do anything necessary prior to processing
 		
@@ -431,7 +583,7 @@ public class CallGenes {
 		//Fill a list with ProcessThreads
 		ArrayList<ProcessThread> alpt=new ArrayList<ProcessThread>(threads);
 		for(int i=0; i<threads; i++){
-			alpt.add(new ProcessThread(cris, bsw, ros, pgm, minLen, i));
+			alpt.add(new ProcessThread(cris, bsw, rosAmino, ros16S, ros18S, pgm, minLen, i));
 		}
 		
 		//Start the threads
@@ -475,9 +627,21 @@ public class CallGenes {
 			geneStopsRetained+=pt.caller.geneStopsRetained;
 			geneStartsOut+=pt.caller.geneStartsOut;
 
+			r16SOut+=pt.caller.r16SOut;
+			r18SOut+=pt.caller.r18SOut;
+			r23SOut+=pt.caller.r23SOut;
+			r5SOut+=pt.caller.r5SOut;
+			tRNAOut+=pt.caller.tRNAOut;
+			
 			stCds.add(pt.caller.stCds);
 			stCds2.add(pt.caller.stCds2);
-			stCdsPass.add(pt.caller.stCdsPass);
+//			stCdsPass.add(pt.caller.stCdsPass);
+			
+			for(int i=0; i<trackers.length; i++){
+				trackers[i].add(pt.caller.trackers[i]);
+			}
+			
+			if(geneHist!=null){Tools.add(geneHist, pt.geneHistT);}
 			
 			success&=pt.success;
 		}
@@ -517,13 +681,17 @@ public class CallGenes {
 	private class ProcessThread extends Thread {
 		
 		//Constructor
-		ProcessThread(final ConcurrentReadInputStream cris_, final ByteStreamWriter bsw_, ConcurrentReadOutputStream ros_,
+		ProcessThread(final ConcurrentReadInputStream cris_, final ByteStreamWriter bsw_, 
+				ConcurrentReadOutputStream rosAmino_, ConcurrentReadOutputStream ros16S_, ConcurrentReadOutputStream ros18S_, 
 				GeneModel pgm_, final int minLen, final int tid_){
 			cris=cris_;
 			bsw=bsw_;
-			ros=ros_;
+			rosAmino=rosAmino_;
+			ros16S=ros16S_;
+			ros18S=ros18S_;
 			pgm=pgm_;
 			tid=tid_;
+			geneHistT=(geneHistBins>1 ? new long[geneHistBins] : null);
 			caller=new GeneCaller(minLen, maxOverlapSameStrand, maxOverlapOppositeStrand, 
 					minStartScore, minStopScore, minKmerScore, minOrfScore, minAvgScore, pgm);
 		}
@@ -663,17 +831,41 @@ public class CallGenes {
 		ArrayList<Orf> processRead(final Read r){
 			ArrayList<Orf> list=caller.callGenes(r, pgm);
 			
-			if(ros!=null){
+			if(geneHistT!=null && list!=null){
+				for(Orf o : list){
+					int bin=Tools.min(geneHistT.length-1, o.length()/geneHistDiv);
+					geneHistT[bin]++;
+				}
+			}
+			
+			if(ros16S!=null){
+				if(list!=null && !list.isEmpty()){
+//					System.err.println("Looking for 16S.");
+					ArrayList<Read> ssu=fetchType(r, list, r16S);
+					if(ssu!=null && !ssu.isEmpty()){
+//						System.err.println("Found "+ssu.size()+" 16S.");
+						ros16S.add(ssu, r.numericID);
+					}
+				}
+			}
+			if(ros18S!=null){
+				if(list!=null && !list.isEmpty()){
+					ArrayList<Read> ssu=fetchType(r, list, r18S);
+					if(ssu!=null && !ssu.isEmpty()){ros18S.add(ssu, r.numericID);}
+				}
+			}
+			
+			if(rosAmino!=null){
 				if(mode==TRANSLATE){
 					if(list!=null && !list.isEmpty()){
 						ArrayList<Read> prots=translate(r, list);
-						ros.add(prots, r.numericID);
+						if(prots!=null){rosAmino.add(prots, r.numericID);}
 					}
 				}else if(mode==RETRANSLATE) {
 					if(list!=null && !list.isEmpty()){
 						ArrayList<Read> prots=translate(r, list);
 						ArrayList<Read> ret=detranslate(prots);
-						ros.add(ret, r.numericID);
+						if(ret!=null){rosAmino.add(ret, r.numericID);}
 					}
 				}else if(mode==RECODE) {
 					if(list!=null && !list.isEmpty()){
@@ -681,7 +873,7 @@ public class CallGenes {
 						r.mate=null;
 						ArrayList<Read> rec=new ArrayList<Read>(1);
 						rec.add(recoded);
-						ros.add(rec, r.numericID);
+						if(rec!=null){rosAmino.add(rec, r.numericID);}
 					}
 				}else{
 					assert(false) : mode;
@@ -701,7 +893,11 @@ public class CallGenes {
 		/** Number of bytes written by this thread */
 		protected long bytesOutT=0;
 		
-		protected ConcurrentReadOutputStream ros;
+		final long[] geneHistT;
+
+		protected ConcurrentReadOutputStream rosAmino;
+		protected ConcurrentReadOutputStream ros16S;
+		protected ConcurrentReadOutputStream ros18S;
 		
 		/** True only if this thread has completed successfully */
 		boolean success=false;
@@ -718,26 +914,41 @@ public class CallGenes {
 		final int tid;
 	}
 	
+	public static ArrayList<Read> fetchType(final Read r, final ArrayList<Orf> list, int type){
+		if(list==null || list.isEmpty()){return null;}
+		ArrayList<Read> ret=new ArrayList<Read>(list.size());
+		for(int strand=0; strand<2; strand++){
+			for(Orf orf : list){
+				if(orf.strand==strand && orf.type==type){
+					Read sequence=fetch(orf, r.bases, r.id);
+					ret.add(sequence);
+				}
+			}
+			r.reverseComplement();
+		}
+		return (ret.size()>0 ? ret : null);
+	}
+	
 	public static ArrayList<Read> translate(final Read r, final ArrayList<Orf> list){
 		if(list==null || list.isEmpty()){return null;}
 		ArrayList<Read> prots=new ArrayList<Read>(list.size());
 		for(int strand=0; strand<2; strand++){
 			for(Orf orf : list){
-				if(orf.strand==strand && orf.type==Orf.CDS){
+				if(orf.strand==strand && orf.type==CDS){
 					Read aa=translate(orf, r.bases, r.id);
 					prots.add(aa);
 				}
 			}
 			r.reverseComplement();
 		}
-		return prots;
+		return prots.isEmpty() ? null : prots;
 	}
 	
 	public static Read recode(final Read r, final ArrayList<Orf> list){
 		if(list==null || list.isEmpty()){return r;}
 		for(int strand=0; strand<2; strand++){
 			for(Orf orf : list){
-				if(orf.strand==strand && orf.type==Orf.CDS){
+				if(orf.strand==strand && orf.type==CDS){
 					recode(orf, r.bases);
 				}
 			}
@@ -759,10 +970,30 @@ public class CallGenes {
 	}
 	
 	public static Read translate(Orf orf, byte[] bases, String id){
+//		assert(orf.length()%3==0) : orf.length(); //Happens sometimes on genes that go off the end, perhaps
 		if(orf.strand==1){orf.flip();}
 		byte[] acids=AminoAcid.toAAs(bases, orf.start, orf.stop);
 		if(orf.strand==1){orf.flip();}
 		Read r=new Read(acids, null, id+"\t"+(Shared.strandCodes[orf.strand]+"\t"+orf.start+"-"+orf.stop), 0, Read.AAMASK);
+//		assert((r.length()+1)*3==orf.length());
+		return r;
+	}
+	
+	public static Read fetch(Orf orf, Read source){
+		assert(orf.start>=0 && orf.stop<source.length()) : source.length()+"\n"+orf;
+		if(orf.strand==1){source.reverseComplement();}
+		Read r=fetch(orf, source.bases, source.id);
+		if(orf.strand==1){source.reverseComplement();}
+		return r;
+	}
+	
+	public static Read fetch(Orf orf, byte[] bases, String id){
+		assert(orf.start>=0 && orf.stop<bases.length) : bases.length+"\n"+orf;
+		if(orf.strand==1){orf.flip();}
+		byte[] sub=Arrays.copyOfRange(bases, orf.start, orf.stop+1);
+		if(orf.strand==1){orf.flip();}
+		Read r=new Read(sub, null, id+"\t"+(Shared.strandCodes[orf.strand]+"\t"+orf.start+"-"+orf.stop), 0, 0);
+		assert(r.length()==orf.length()) : r.length()+", "+orf.length();
 		return r;
 	}
 	
@@ -810,15 +1041,23 @@ public class CallGenes {
 	private long genesOut=0;
 	private long bytesOut=0;
 	
-	private static int minLen=60;
+	private static int minLen=80;//Actually a much higher value like 200 seems optimal compared to NCBI
 	private static int maxOverlapSameStrand=80;
-	private static int maxOverlapOppositeStrand=120;
+	private static int maxOverlapOppositeStrand=110;
 	
-	private static float minStartScore=0.00f;
-	private static float minStopScore=-0.02f;
-	private static float minKmerScore=0.06f;
-	private static float minOrfScore=40f; //Higher increases SNR dramatically but reduces TP rate
-	private static float minAvgScore=0.10f; //Not very effective
+	/* for kinnercds=6 */
+//	private static float minStartScore=-0.10f;
+//	private static float minStopScore=-0.5f;//Not useful; disabled
+//	private static float minKmerScore=0.04f;//Does not seem useful.
+//	private static float minOrfScore=40f; //Higher increases SNR dramatically but reduces TP rate
+//	private static float minAvgScore=0.08f; //Not very effective
+
+	/* for kinnercds=7 */
+	private static float minStartScore=-0.10f;
+	private static float minStopScore=-0.5f;//Not useful; disabled
+	private static float minKmerScore=0.02f;//Does not seem useful.
+	private static float minOrfScore=50f; //Higher increases SNR dramatically but reduces TP rate
+	private static float minAvgScore=0.08f; //Not very effective
 	
 	long geneStopsMade=0;
 	long geneStartsMade=0;
@@ -826,9 +1065,27 @@ public class CallGenes {
 	long geneStopsRetained=0;
 	long geneStartsOut=0;
 
-	ScoreTracker stCds=new ScoreTracker(Orf.CDS);
-	ScoreTracker stCds2=new ScoreTracker(Orf.CDS);
-	ScoreTracker stCdsPass=new ScoreTracker(Orf.CDS);
+	long tRNAOut=0;
+	long r16SOut=0;
+	long r23SOut=0;
+	long r5SOut=0;
+	long r18SOut=0;
+	
+	ScoreTracker stCds=new ScoreTracker(CDS);
+	ScoreTracker stCds2=new ScoreTracker(CDS);
+	ScoreTracker stCdsPass=new ScoreTracker(CDS);
+	ScoreTracker sttRNA=new ScoreTracker(tRNA);
+	ScoreTracker st16s=new ScoreTracker(r16S);
+	ScoreTracker st23s=new ScoreTracker(r23S);
+	ScoreTracker st5s=new ScoreTracker(r5S);
+	ScoreTracker st18s=new ScoreTracker(r18S);
+	
+	ScoreTracker[] trackers=new ScoreTracker[] {stCdsPass, sttRNA, st16s, st23s, st5s, st18s};
+	
+	private int geneHistBins=2000;
+	private int geneHistDiv=20;
+	private final long[] geneHist;
+	private boolean printZeroCountHist=false;
 	
 	/*--------------------------------------------------------------*/
 
@@ -836,7 +1093,12 @@ public class CallGenes {
 	private ArrayList<String> pgmList=new ArrayList<String>();
 	private String outGff=null;
 	private String outAmino=null;
+	private String out16S=null;
+	private String out18S=null;
 	private String compareToGff=null;
+	private String outStats="stderr";
+	private String geneHistFile=null;
+	private boolean json_out=false;
 	
 	/*--------------------------------------------------------------*/
 	/*----------------         Final Fields         ----------------*/
@@ -844,6 +1106,8 @@ public class CallGenes {
 	
 	private final FileFormat ffoutGff;
 	private final FileFormat ffoutAmino;
+	private final FileFormat ffout16S;
+	private final FileFormat ffout18S;
 	
 	/** Determines how sequence is processed if it will be output */
 	int mode=TRANSLATE;
@@ -865,5 +1129,6 @@ public class CallGenes {
 	private boolean overwrite=false;
 	private boolean append=false;
 	private boolean ordered=false; //this is OK sometimes, but sometimes hangs (e.g. on RefSeq mito), possibly if a sequence produces nothing.
+	//To fix it, just ensure functions like translate always produce an array, even if it is empty.
 	
 }
