@@ -5,11 +5,14 @@ import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.Collections;
 
+import aligner.SingleStateAlignerFlat2;
+import aligner.SingleStateAlignerFlat3;
+import aligner.SingleStateAlignerFlatFloat;
 import dna.AminoAcid;
 import shared.Tools;
 import stream.Read;
 import structures.FloatList;
-import structures.IntHashSet;
+import structures.LongHashSet;
 import structures.IntList;
 
 
@@ -19,19 +22,12 @@ import structures.IntList;
  * @date Sep 24, 2018
  *
  */
-public class GeneCaller {
+public class GeneCaller extends ProkObject {
 	
 	/*--------------------------------------------------------------*/
 	/*----------------             Init             ----------------*/
 	/*--------------------------------------------------------------*/
 	
-	GeneCaller(int minLen_, int maxOverlapSameStrand_, int maxOverlapOppositeStrand_, 
-			float minStartScore_, float minStopScore_, float minInnerScore_, 
-			float minOrfScore_, float minAvgScore_){
-		this(minLen_, maxOverlapSameStrand_, maxOverlapOppositeStrand_, 
-				minStartScore_, minStopScore_, minInnerScore_, minOrfScore_, minAvgScore_, null);
-	}
-	
 	GeneCaller(int minLen_, int maxOverlapSameStrand_, int maxOverlapOppositeStrand_, 
 			float minStartScore_, float minStopScore_, float minInnerScore_,
 			float minOrfScore_, float minAvgScore_, GeneModel pgm_){
@@ -57,12 +53,6 @@ public class GeneCaller {
 	
 	public ArrayList<Orf> callGenes(Read r, GeneModel pgm_){
 		pgm=pgm_;
-
-		pgm.statsCDS.enabled=callCDS;
-		pgm.statstRNA.enabled=calltRNA;
-		pgm.stats16S.enabled=call16S;
-		pgm.stats23S.enabled=call23S;
-		pgm.stats5S.enabled=call5S;
 		
 		final String name=r.id;
 		final byte[] bases=r.bases;
@@ -73,7 +63,8 @@ public class GeneCaller {
 		ArrayList<Orf>[] brokenLists=breakOrfs(frameLists, bases);
 		
 		ArrayList<Orf>[] rnaLists=null;
-		if(calltRNA || call16S || call23S || call5S){
+		final int rlen=r.length();
+		if(calltRNA || (call16S && rlen>800) || (call23S && rlen>1500) || call5S || (call18S && rlen>1000)){
 			rnaLists=makeRnas(name, bases);
 
 			brokenLists[0].addAll(rnaLists[0]);
@@ -106,9 +97,14 @@ public class GeneCaller {
 		
 		//Find the optimal path through Orfs
 		ArrayList<Orf> path=findPath(brokenLists, bases);
-		geneStartsOut+=path.size();
-		
-		stCdsPass.add(path);
+//		geneStartsOut+=path.size();
+
+		if(callCDS){stCdsPass.add(path);}
+		if(calltRNA){sttRNA.add(path);}
+		if(call16S){st16s.add(path);}
+		if(call23S){st23s.add(path);}
+		if(call5S){st5s.add(path);}
+		if(call18S){st18s.add(path);}
 		
 		return path;
 	}
@@ -128,11 +124,11 @@ public class GeneCaller {
 		array[0]=new ArrayList<Orf>();
 		array[1]=new ArrayList<Orf>();
 		final float[] scores=new float[bases.length];
-		final int[] kmersSeen=(pgm.lsuKmers==null && pgm.ssuKmers==null && pgm.trnaKmers==null) ? null : new int[bases.length];
+		final int[] kmersSeen=(lsuKmers==null && ssuKmers==null && trnaKmers==null && r5SKmers==null) ? null : new int[bases.length];
 		for(int strand=0; strand<2; strand++){
 			for(StatsContainer sc : pgm.rnaContainers){
-				if(sc.enabled){
-					ArrayList<Orf> list=makeRnasForStrand(name, bases, strand, sc, scores, (sc.kmerSet==null ? null : kmersSeen));//TODO: Make this loop through all RNA types
+				if(ProkObject.callType(sc.type)){
+					ArrayList<Orf> list=makeRnasForStrand(name, bases, strand, sc, scores, (sc.kmerSet()==null ? null : kmersSeen), false, -1);//TODO: Make this loop through all RNA types
 					if(strand==1 && list!=null){
 						for(Orf orf : list){
 							assert(orf.strand==strand);
@@ -148,6 +144,45 @@ public class GeneCaller {
 		return array;
 	}
 	
+	/** Designed for quickly calling a single SSU */
+	public Orf makeRna(String name, byte[] bases, int type){
+		final float[] scores=new float[bases.length];//TODO: Big and slow; make a FloatList?
+		StatsContainer sc=pgm.allContainers[type];
+		final int[] kmersSeen=(sc.kmerSet()==null ? null : new int[bases.length]);//TODO: IntList?
+		
+		int strand=0;
+		ArrayList<Orf> list=makeRnasForStrand(name, bases, strand, sc, scores, kmersSeen, true, -1);
+		final Orf best1=pickBest(list);
+		assert(best1==null || best1.start>=0 && best1.stop<bases.length) : bases.length+"\n"+best1;
+		if(best1!=null && best1.orfScore>-999){return best1;}
+		
+		strand++;
+		AminoAcid.reverseComplementBasesInPlace(bases);
+		list=makeRnasForStrand(name, bases, strand, sc, scores, kmersSeen, true, -1);
+		AminoAcid.reverseComplementBasesInPlace(bases);
+		if(strand==1 && list!=null){
+			for(Orf orf : list){
+				assert(orf.strand==strand);
+				orf.flip();
+			}
+		}
+		final Orf best2=pickBest(list);
+		assert(best2==null || best2.start>=0 && best2.stop<bases.length) : bases.length+"\n"+best2;
+		if(best2!=null && best2.orfScore>-999){return best2;}
+		return best1!=null ? best1 : best2;
+	}
+	
+	final Orf pickBest(ArrayList<Orf> list){
+		if(list==null){return null;}
+		Orf best=null;
+		for(Orf orf : list){
+			if(best==null || orf.orfScore>best.orfScore){
+				best=orf;
+			}
+		}
+		return best;
+	}
+	
 	/** 
 	 * Generates lists of all max-length non-overlapping Orfs per frame.
 	 * There IS overlap between frames.
@@ -231,6 +266,12 @@ public class GeneCaller {
 		ArrayList<Orf> bestPath=new ArrayList<Orf>();
 		for(Orf orf=best; orf!=null; orf=orf.prev()){
 			bestPath.add(orf);
+			if(orf.type==CDS){geneStartsOut++;}
+			else if(orf.type==tRNA){tRNAOut++;}
+			else if(orf.type==r16S){r16SOut++;}
+			else if(orf.type==r23S){r23SOut++;}
+			else if(orf.type==r5S){r5SOut++;}
+			else if(orf.type==r18S){r18SOut++;}
 		}
 		Collections.sort(bestPath);
 		return bestPath;
@@ -383,7 +424,6 @@ public class GeneCaller {
 					pathScore=prev.pathScore()+orfScore;
 					pathScore+=q1+Tools.mid(q2*prevLength, q3+q4*prevLength, q5);
 				}
-				
 				if(overlap<1 && prevScore>0){found=true;}
 				if(pathScore>=orf.pathScoreMinus){
 					orf.pathScoreMinus=pathScore;
@@ -408,7 +448,7 @@ public class GeneCaller {
 		assert(minlen>=3);
 		if(bases==null || bases.length<minlen){return null;}
 		ArrayList<Orf> orfs=new ArrayList<Orf>();
-		if(!callCDS){return orfs;}
+		if(!ProkObject.callCDS){return orfs;}
 //		int mask=63;
 		int code=0;
 		int start=-2;
@@ -428,7 +468,7 @@ public class GeneCaller {
 					if(GeneModel.isStopCodon(code) || code<0){//NOTE: This adds a stop codon wherever there are Ns.
 						int len=pos-start+1;
 						if(len>=minlen){
-							Orf f=new Orf(name, start, pos, strand, startFrame, bases, true, Orf.CDS);
+							Orf f=new Orf(name, start, pos, strand, startFrame, bases, true, CDS);
 							orfs.add(f);
 						}
 						start=-1;
@@ -452,7 +492,7 @@ public class GeneCaller {
 			int len=pos-start+1;
 			if(len>=minlen){
 				assert(pos<bases.length) : start+", "+pos+", "+bases.length;
-				Orf f=new Orf(name, start, pos, strand, startFrame, bases, true, Orf.CDS);
+				Orf f=new Orf(name, start, pos, strand, startFrame, bases, true, CDS);
 				orfs.add(f);
 			}
 		}
@@ -461,13 +501,13 @@ public class GeneCaller {
 	}
 	
 	/** 
-	 * Generates a list of maximal-length Orfs only (non-overlapping).
-	 * All Orfs come out in native orientation (unflipped). 
+	 * Generates a list of maximal-length RNAs (non-overlapping).
+	 * All RNAs come out in native orientation (unflipped). 
 	 * */
-	ArrayList<Orf> makeRnasForStrand(String name, byte[] bases, int strand, StatsContainer sc, float[] scores, int[] kmersSeen){
+	ArrayList<Orf> makeRnasForStrand(String name, byte[] bases, int strand, StatsContainer sc, float[] scores, int[] kmersSeen, boolean quitEarly, float bias){
 		final int window=sc.lengthAvg;
-		if(bases==null || bases.length<window/2){return null;}
-		ArrayList<Orf> orfs=new ArrayList<Orf>(sc.type==Orf.tRNA ? 32 : 8);
+		if(bases==null || bases.length*2<window){return null;}
+		ArrayList<Orf> orfs=new ArrayList<Orf>(sc.type==tRNA ? 32 : 8);
 		
 		final FrameStats inner=sc.inner;
 //		final FrameStats start=sc.start;
@@ -477,12 +517,15 @@ public class GeneCaller {
 		final int mask=inner.mask;
 //		final float invLen=sc.invLengthAvg;
 		final int halfWindow=window/2;
+		final int maxWindow=(int)(window*1.5f);
+		final int maxWindow2=(int)(window*2.5f);
 //		final int slop=Tools.max(50, window/8);
 		int len=0;
 		int kmer=0;
 		float currentScore=0;
 		float currentScoreAbs=0;
-		float bias=biases[sc.type];
+		bias=(bias>-1 ? bias : biases[sc.type]);
+		final float maxBias=biases[sc.type]*1.45f;
 		
 		float thresh=cutoff1[sc.type];
 		float prevScore=0;
@@ -516,13 +559,22 @@ public class GeneCaller {
 				}else{
 					int rnaLen=maxPos-lastStart;
 					if(max>thresh && rnaLen>=halfWindow){
-						Orf orf=new Orf(name, lastStart, maxPos, strand, 0, bases, false, sc.type);
-						orfs.add(orf);
-						orf.orfScore=max;
-						if(verbose){
-							final int start2=(strand==0 ? lastStart : bases.length-maxPos-1);
-							final int stop2=(strand==0 ? maxPos : bases.length-lastStart-1);
-							System.err.println(start2+"\t"+stop2+"\t"+max);
+						if(rnaLen>maxWindow){
+							if(bias<=maxBias){
+								orfs=null;
+								float biasMult=(rnaLen>8*window ? 1.2f : rnaLen>4*window ? 1.1f : 1.05f);
+								return makeRnasForStrand(name, bases, strand, sc, scores, kmersSeen, quitEarly, bias*biasMult);
+							}
+						}
+						if(rnaLen<=maxWindow2){
+							Orf orf=new Orf(name, lastStart, maxPos, strand, 0, bases, false, sc.type);
+							orfs.add(orf);
+							orf.orfScore=max;
+							if(verbose){
+								final int start2=(strand==0 ? lastStart : bases.length-maxPos-1);
+								final int stop2=(strand==0 ? maxPos : bases.length-lastStart-1);
+								System.err.println("Made Orf "+start2+"\t"+stop2+"\t"+max);
+							}
 						}
 					}
 					max=0;
@@ -549,31 +601,46 @@ public class GeneCaller {
 		{
 			int rnaLen=maxPos-lastStart;
 			if(max>thresh && rnaLen>=halfWindow){
-				Orf orf=new Orf(name, lastStart, maxPos, strand, 0, bases, false, sc.type);
-				orfs.add(orf);
-				orf.orfScore=max;
-				if(verbose){
-					final int start2=(strand==0 ? lastStart : bases.length-maxPos-1);
-					final int stop2=(strand==0 ? maxPos : bases.length-lastStart-1);
-					System.err.println(start2+"\t"+stop2+"\t"+max);
+				if(rnaLen>maxWindow){
+					if(bias<=maxBias){
+						orfs=null;
+						float biasMult=(rnaLen>8*window ? 1.2f : rnaLen>4*window ? 1.1f : 1.05f);
+						return makeRnasForStrand(name, bases, strand, sc, scores, kmersSeen, quitEarly, bias*biasMult);
+					}
+				}
+				if(rnaLen<=maxWindow2){
+					Orf orf=new Orf(name, lastStart, maxPos, strand, 0, bases, false, sc.type);
+					orfs.add(orf);
+					orf.orfScore=max;
+					if(verbose){
+						final int start2=(strand==0 ? lastStart : bases.length-maxPos-1);
+						final int stop2=(strand==0 ? maxPos : bases.length-lastStart-1);
+						System.err.println(start2+"\t"+stop2+"\t"+max);
+					}
 				}
 			}
 		}
 		
-		if(kmersSeen!=null && orfs.size()>0 && sc.kmerSet!=null){
-			fillKmersSeen(bases, kmersSeen, sc.kmerSet, sc.kLongLen);
+		if(kmersSeen!=null && orfs.size()>0 && sc.kmerSet()!=null){
+			fillKmersSeen(bases, kmersSeen, sc.kmerSet(), sc.kLongLen());
 		}
 		
 		float cutoff=cutoff2[sc.type];
+		
 		for(int i=0; i<orfs.size(); i++){
 			Orf orf=orfs.get(i);
 //			System.err.println(orf.orfScore);
-			refineRna(orf, bases, strand, sc, scores, kmersSeen);
-			if(orf.orfScore<cutoff){
+			boolean good=refineRna(orf, bases, strand, sc, scores, kmersSeen);
+			if(orf.orfScore<cutoff || !good){
 				if(verbose){System.err.println("REJECT: "+orf.toStringFlipped());}
 				orfs.set(i, null);
 			}else{
 				if(verbose){System.err.println("ACCEPT: "+orf.toStringFlipped());}
+				if(quitEarly){
+					orfs.clear();
+					orfs.add(orf);
+					return orfs;
+				}
 			}
 		}
 		Tools.condenseStrict(orfs);
@@ -615,9 +682,9 @@ public class GeneCaller {
 		return orfs;
 	}
 	
-	void fillKmersSeen(byte[] bases, int[] kmersSeen, IntHashSet set, int k){
-		final int mask=~((-1)<<(2*k));
-		int kmer=0;
+	void fillKmersSeen(byte[] bases, int[] kmersSeen, LongHashSet set, int k){
+		final long mask=~((-1L)<<(2*k));
+		long kmer=0;
 		int len=0;
 		int seen=0;
 		for(int i=0; i<bases.length; i++){
@@ -634,11 +701,12 @@ public class GeneCaller {
 		}
 	}
 	
-	void refineRna(Orf orf, byte[] bases, int strand, StatsContainer sc, float[] scores, int[] kmersSeen){
-		if(orf==null){return;}
+	boolean refineRna(Orf orf, byte[] bases, int strand, StatsContainer sc, float[] scores, int[] kmersSeen){
+		if(orf==null){return false;}
 		if(verbose){System.err.println("REFINE: "+orf.toStringFlipped());}
 		final int window=sc.lengthAvg;
-		final int halfWindow=window/2;
+//		final int halfWindow=window/2;
+//		final int maxWindow=(int)(window*1.5f);
 		final int slop=Tools.max(60, 10+window/8);
 		final float invWindow=sc.invLengthAvg;
 		final float oldScore=orf.orfScore;
@@ -653,11 +721,12 @@ public class GeneCaller {
 			if(kmersSeen[leftmost]>=kmersSeen[rightmost]){
 //				System.err.println("Bad: "+oldScore);
 				orf.orfScore=-999;
-				return;
+				return false;
 			}else{
 //				System.err.println("Good: "+oldScore);
 			}
 		}
+		if(verbose){System.err.println("REFINE2");}
 		
 		{//starts
 			final int left=leftmost;
@@ -681,7 +750,7 @@ public class GeneCaller {
 
 		final float innerThresh=cutoff5[sc.type];
 		
-		final int minlen=window-slop;
+		final int minlen=Tools.max(window/2, window-slop);
 		final int maxlen=window+slop;
 		
 		orf.orfScore=0;
@@ -693,10 +762,11 @@ public class GeneCaller {
 //			System.err.println("start="+start);
 			for(int j=lastStop; j<stops.size; j++){
 				final int stop=stops.get(j);
+				final int rnalen=stop-start+1;
 //				System.err.println("stop="+stop);
-				if(stop<start+minlen){
+				if(rnalen<minlen){
 					lastStop=j+1;
-				}else if(stop>start+window+maxlen){
+				}else if(rnalen>maxlen){
 //					System.err.println("broke");
 					break;
 				}else if(kmersSeen!=null && kmersSeen[stop]<=startSeen){//TODO: Test this
@@ -706,13 +776,14 @@ public class GeneCaller {
 					final float stopScore=stopScores.get(j);
 					final float innerScore=(scores[stop]-scores[start])/len;
 //					System.err.println("innerScore="+innerScore);
+					assert(rnalen<=maxlen) : "start="+start+", stop="+stop+", rnalen="+rnalen+", minlen="+minlen+", maxlen="+maxlen;
 					if(innerScore>=innerThresh){
 						final float a=Tools.max(startScore+0.25f, 0.25f);
 						final float b=Tools.max(stopScore+0.25f, 0.25f);
 						final float c=innerScore*innerScore;
 						final float d=(window-(2.4f*Tools.absdif(len, window)))*invWindow;
 						final float score=c*d*(float)Math.sqrt(a*b);
-						if(verbose && score>=0.01f*orf.orfScore){
+						if(verbose && score>=0.2f*orf.orfScore){
 							final int start2=(strand==0 ? start : bases.length-stop-1);
 							final int stop2=(strand==0 ? stop : bases.length-start-1);
 							System.err.println(start2+"-"+stop2+", "+startScore+", "+stopScore+", "+innerScore+", "+(score*scoreMult[sc.type])+", "+oldScore);
@@ -737,6 +808,110 @@ public class GeneCaller {
 			}
 		}
 		orf.orfScore*=scoreMult[sc.type];
+		
+		if(starts.isEmpty() || stops.isEmpty()){
+			if(verbose){System.err.println("No starts or stops.");}
+			orf.orfScore=Tools.min(-999, orf.orfScore-9999);
+			return false;
+		}
+		return refineByAlignment(orf, bases, strand, sc);//Returns true if no consensus is present
+	}
+	
+	boolean refineByAlignment(Orf orf, byte[] bases, int strand, StatsContainer sc){
+		if(verbose){System.err.println("ALIGN");}
+		Read[] consensus=ProkObject.consensusReads(sc.type);
+		if(consensus==null || consensus.length==0){return true;}
+		boolean refined=false;
+//		System.err.println("Initial: "+orf.start+", "+orf.stop);
+		for(Read r : consensus){
+//			refined=refineByAlignment(orf, bases, strand, sc, r.bases, 15, 15, 2);
+			refined=refineByAlignment(orf, bases, strand, sc, r.bases, sc.startSlop(), sc.stopSlop(), 2);
+			if(refined || sc.type==r18S || true){break;}
+		}
+		if(refined){
+			if(verbose){System.err.println("Aligned to: "+orf.start+", "+orf.stop);}
+		}else{
+			if(verbose){System.err.println("Alignment failed.");}
+			orf.orfScore=Tools.min(-999, orf.orfScore-9999);
+		}
+		return refined;
+	}
+	
+	boolean refineByAlignment(Orf orf, byte[] bases, int strand, StatsContainer sc, byte[] consensus, 
+			final int startSlop, final int stopSlop, int recurLimit){
+		final int start0=orf.start;
+		final int stop0=orf.stop;
+		
+		assert(start0>=0 && start0<bases.length) : start0+", "+stop0;
+		assert(stop0>=0 && stop0<bases.length) : start0+", "+stop0;
+		
+		final float minID=sc.minIdentity();
+		final int padding=Tools.min(alignmentPadding, 30+sc.lengthAvg/4);
+		final int a=Tools.max(0, orf.start-padding);
+		final int b=Tools.min(bases.length-1, orf.stop+padding);
+		final int reflen=b-a+1;
+		if(reflen>10*sc.lengthAvg && reflen>20000){
+			System.err.println("Skipped reflen "+reflen+"/"+sc.lengthAvg+" for "
+					+ "seqlen="+bases.length+", orf="+orf.toString());
+			assert(false);
+			//TODO: Possibly change return to -1, 0, 1 ("can't align")
+			//Should be a limit on window size...
+			//Also consider shrinking matrix after jumbo alignments
+			return false;
+		}
+		assert(a>=0 && b<bases.length) : a+", "+b;
+		SingleStateAlignerFlat2 ssa=getSSA();
+		final int minScore=ssa.minScoreByIdentity(consensus.length, minID);
+		int[] max=ssa.fillUnlimited(consensus, bases, a, b, minScore);
+		if(max==null){return false;}
+		
+		final int rows=max[0];
+		final int maxCol=max[1];
+		final int maxState=max[2];
+//		final int maxScore=max[3];
+//		final int maxStart=max[4];
+		
+		//returns {score, bestRefStart, bestRefStop} 
+		//padded: {score, bestRefStart, bestRefStop, padLeft, padRight};
+		final int[] score=ssa.score(consensus, bases, a, b, rows, maxCol, maxState);
+		final int rstart=Tools.max(score[1], 0);
+		final int rstop=Tools.min(score[2], bases.length-1);
+		final float id=ssa.tracebackIdentity(consensus, bases, a, b, rows, maxCol, maxState, null);
+		
+		assert(rstart>=0 && rstart<bases.length) : "bases="+bases.length+
+			", rstart="+rstart+", rstop="+rstop+", a="+a+", b="+b+"\n"+"score="+Arrays.toString(score)+", id="+id;
+		assert(rstop>=0 && rstop<bases.length) : rstart+", "+rstop;
+		
+		if(score.length>3 && recurLimit>0){
+			final int padLeft=score[3];
+			final int padRight=score[4];
+			//TODO:  This takes extra memory.  It may be better to cap the width or ignore start0/stop0 here.
+			int rstart2=Tools.max(0, Tools.min(start0, rstart)-10);
+			int rstop2=Tools.min(bases.length-1, Tools.max(stop0, rstop)+10);
+			assert(rstart2>=0 && rstart2<bases.length) : rstart2+", "+rstop2;
+			assert(rstop2>=0 && rstop2<bases.length) : rstart2+", "+rstop2;
+			orf.start=rstart2;
+			orf.stop=rstop2;
+			if(orf.start<a || orf.stop>b){
+				boolean ret=refineByAlignment(orf, bases, strand, sc, consensus, 0, 0, recurLimit-1);
+				if(ret){return true;}
+			}
+			orf.start=start0;
+			orf.stop=stop0;
+//			assert(false) : "Don't traceback after recur.";
+		}
+		if(verbose){
+			System.err.println("Identity: "+id);
+		}
+//		assert(score.length==3) : "TODO: Handle padding requests.";
+		
+//		System.err.println("Identity: "+String.format("%.2f", 100*id)+"; location: "+rstart+"-"+rstop);
+		if(id<minID){return false;}
+		
+		
+		if(Tools.absdif(rstart, start0)>startSlop){orf.start=rstart;}
+		if(Tools.absdif(rstop, stop0)>stopSlop){orf.stop=rstop;}
+		return true;
 	}
 	
 	void fillPoints(final int left, final int right, final byte[] bases, final FrameStats fs, float thresh, final IntList points, final FloatList scores){
@@ -1002,22 +1177,50 @@ public class GeneCaller {
 
 	//for k=6,3,3 - this has low scores for correct 23s stops; may try k=2 or k=4 for that end 
 	//Also, 5S seems to score low very for archaea
-	//Order is: CDS=0, tRNA=1, r16S=2, r23S=3, r5S=4, RNA=5
-	static float[] cutoff1=new float[] {0, 100f, 600f, 450f, 300f};//sum score
-	static float[] cutoff2=new float[] {0, 48f, 300f, 400f, 32f};//orf score
-	static float[] cutoff3=new float[] {0, 3.0f, 1.8f, 1.8f, 4.0f};//start score
-	static float[] cutoff4=new float[] {0, 2.8f, 2.0f, 1.25f, 1.9f};//stop score
-	static float[] cutoff5=new float[] {0, 2.8f, 1.1f, 1.15f, 1.4f};//inner score
-	static float[] scoreMult=new float[] {1f, 1f, 35f, 80f, 1.0f};//score mult
-	static float[] biases=new float[] {1f, 1.25f, 1.2f, 1.2f, 1.22f};//bias for sum score
-
-//	static float[] cutoff1=new float[] {0, 50f, 600f, 450f, 300f};//sum score
-//	static float[] cutoff2=new float[] {0, 24f, 300f, 400f, 32f};//orf score
-//	static float[] cutoff3=new float[] {0, 1.5f, 1.8f, 1.8f, 4.0f};//start score
-//	static float[] cutoff4=new float[] {0, 1.4f, 2.0f, 1.25f, 1.9f};//stop score
-//	static float[] cutoff5=new float[] {0, 1.4f, 1.1f, 1.15f, 1.4f};//inner score
+//	public static int CDS=0, tRNA=1, /*r16S=2,*/ r23S=3, r5S=4, r18S=5, r28S=6, RNA=7;
+//	static float[] cutoff1=new float[] {0, 100f, 600f, 400f, 300f};//sum score
+//	static float[] cutoff2=new float[] {0, 48f, 300f, 300f, 32f};//orf score
+//	static float[] cutoff3=new float[] {0, 3.0f, 1.8f, 1.6f, 4.0f};//start score
+//	static float[] cutoff4=new float[] {0, 2.8f, 2.0f, 0.90f, 1.9f};//stop score
+//	static float[] cutoff5=new float[] {0, 2.8f, 1.1f, 1.55f, 1.4f};//inner score
 //	static float[] scoreMult=new float[] {1f, 1f, 35f, 80f, 1.0f};//score mult
-//	static float[] biases=new float[] {1f, 1.25f, 1.2f, 1.2f, 1.22f};//bias for sum score
+//	static float[] biases=new float[] {1f, 1.25f, 1.30f, 1.30f, 1.22f};//16S bias for sum score: 1.25 best for archs, 1.4 best for bacts
+
+////	{"CDS", "tRNA", "16S", "23S", "5S", "18S", "28S", "RNA"};
+//	static float[] cutoff1=new float[] {0, 90f, 300f, 400f, 270f, 300f};//sum score
+//	static float[] cutoff2=new float[] {0, 48f, 300f, 300f, 32f, 300f};//orf score
+//	static float[] cutoff3=new float[] {0, 2.8f, 1.65f, 1.6f, 3.8f, 1.65f};//start score
+//	static float[] cutoff4=new float[] {0, 2.6f, 1.70f, 0.90f, 1.8f, 1.70f};//stop score
+//	static float[] cutoff5=new float[] {0, 2.8f, 1.70f, 1.55f, 1.4f, 1.70f};//inner score
+//	static float[] scoreMult=new float[] {1f, 1f, 35f, 80f, 1.0f, 35f};//score mult
+//	static float[] biases=new float[] {1f, 1.50f, 1.30f, 1.30f, 1.30f, 1.30f};
+
+////	{"CDS", "tRNA", "16S", "23S", "5S", "18S", "28S", "RNA"};
+//	static float[] cutoff1=new float[] {0, 90f, 300f, 400f, 90f, 300f};//sum score
+//	static float[] cutoff2=new float[] {0, 48f, 300f, 300f, 24f, 300f};//orf score
+//	static float[] cutoff3=new float[] {0, 2.8f, 1.65f, 1.6f, 2.0f, 1.65f};//start score
+//	static float[] cutoff4=new float[] {0, 2.6f, 1.70f, 0.90f, 1.0f, 1.70f};//stop score
+//	static float[] cutoff5=new float[] {0, 2.8f, 1.70f, 1.55f, 2.6f, 1.70f};//inner score
+//	static float[] scoreMult=new float[] {1f, 1f, 35f, 80f, 1.25f, 35f};//score mult
+//	static float[] biases=new float[] {1f, 1.50f, 1.30f, 1.30f, 1.50f, 1.30f};
+
+////	{"CDS", "tRNA", "16S", "23S", "5S", "18S", "28S", "RNA"};
+//	static float[] cutoff1=new float[] {0, 90f, 300f, 400f, 100f, 300f};//sum score
+//	static float[] cutoff2=new float[] {0, 48f, 300f, 300f, 32f, 300f};//orf score
+//	static float[] cutoff3=new float[] {0, 2.8f, 1.65f, 1.6f, 1.8f, 1.65f};//start score
+//	static float[] cutoff4=new float[] {0, 2.6f, 1.70f, 0.90f, 1.0f, 1.70f};//stop score
+//	static float[] cutoff5=new float[] {0, 2.8f, 1.70f, 1.55f, 2.6f, 1.70f};//inner score
+//	static float[] scoreMult=new float[] {1f, 1f, 35f, 80f, 1.25f, 35f};//score mult
+//	static float[] biases=new float[] {1f, 1.50f, 1.30f, 1.30f, 1.55f, 1.30f};
+
+//	{"CDS", "tRNA", "16S", "23S", "5S", "18S", "28S", "RNA"};
+	static float[] cutoff1=new float[] {0, 20f, 300f, 400f, 100f, 400f};//sum score
+	static float[] cutoff2=new float[] {0, 36f, 300f, 300f, 32f, 300f};//orf score //tRNA is very sensitive here
+	static float[] cutoff3=new float[] {0, 2.4f, 1.65f, 1.6f, 1.8f, 1.5f};//start score
+	static float[] cutoff4=new float[] {0, 1.5f, 1.70f, 0.90f, 1.0f, 1.1f};//stop score
+	static float[] cutoff5=new float[] {0, 2.2f, 1.70f, 1.55f, 2.6f, 1.5f};//inner score
+	static float[] scoreMult=new float[] {1f, 1.0f, 35f, 80f, 1.25f, 35f};//score mult
+	static float[] biases=new float[] {1f, 1.45f, 1.30f, 1.30f, 1.55f, 1.50f};
 	
 	long geneStopsMade=0;
 	long geneStartsMade=0;
@@ -1025,39 +1228,94 @@ public class GeneCaller {
 	long geneStopsRetained=0;
 	long geneStartsOut=0;
 
-	ScoreTracker stCds=new ScoreTracker(Orf.CDS);
-	ScoreTracker stCds2=new ScoreTracker(Orf.CDS);
-	ScoreTracker stCdsPass=new ScoreTracker(Orf.CDS);
+	long tRNAOut=0;
+	long r16SOut=0;
+	long r23SOut=0;
+	long r5SOut=0;
+	long r18SOut=0;	
+	
+	ScoreTracker stCds=new ScoreTracker(CDS);
+	ScoreTracker stCds2=new ScoreTracker(CDS);
+	ScoreTracker stCdsPass=new ScoreTracker(CDS);
+	ScoreTracker sttRNA=new ScoreTracker(tRNA);
+	ScoreTracker st16s=new ScoreTracker(r16S);
+	ScoreTracker st23s=new ScoreTracker(r23S);
+	ScoreTracker st5s=new ScoreTracker(r5S);
+	ScoreTracker st18s=new ScoreTracker(r18S);
+	
+	ScoreTracker[] trackers=new ScoreTracker[] {stCdsPass, sttRNA, st16s, st23s, st5s, st18s};
+	
+	public static SingleStateAlignerFlat2 getSSA(){
+		SingleStateAlignerFlat2 ssa=localSSA.get();
+		if(ssa==null){
+			synchronized(localSSA){
+				ssa=localSSA.get();
+				if(ssa==null){
+					ssa=new SingleStateAlignerFlat2();
+					localSSA.set(ssa);
+				}
+			}
+		}
+		return ssa;
+	}
+	
+	public static SingleStateAlignerFlat3 getSSA3(){
+		SingleStateAlignerFlat3 ssa=localSSA3.get();
+		if(ssa==null){
+			synchronized(localSSA3){
+				ssa=localSSA3.get();
+				if(ssa==null){
+					ssa=new SingleStateAlignerFlat3();
+					localSSA3.set(ssa);
+				}
+			}
+		}
+		return ssa;
+	}
+	
+	public static SingleStateAlignerFlatFloat getSSAF(){
+		SingleStateAlignerFlatFloat ssa=localSSAF.get();
+		if(ssa==null){
+			synchronized(localSSAF){
+				ssa=localSSAF.get();
+				if(ssa==null){
+					ssa=new SingleStateAlignerFlatFloat();
+					localSSAF.set(ssa);
+				}
+			}
+		}
+		return ssa;
+	}
+
+	private static ThreadLocal<SingleStateAlignerFlat2> localSSA=new ThreadLocal<SingleStateAlignerFlat2>();
+	private static ThreadLocal<SingleStateAlignerFlat3> localSSA3=new ThreadLocal<SingleStateAlignerFlat3>();
+	private static ThreadLocal<SingleStateAlignerFlatFloat> localSSAF=new ThreadLocal<SingleStateAlignerFlatFloat>();
+//	public static int maxAlignmentEndpointDifference=15;
+	public static int alignmentPadding=300;
 	
 	public static int breakLimit=12;
 	public static int lookbackPlus=70;
 	public static int lookbackMinus=25;
-
-	public static boolean callCDS=true;
-	public static boolean calltRNA=true;
-	public static boolean call16S=true;
-	public static boolean call23S=true;
-	public static boolean call5S=true;
 	
 //	pathScore+=p0+p1*(Tools.mid(p5*(p2+pathLength), p6*(p3-pathLength), p4));
 	
 	//Operon length modifiers for same strand
-	public static float p0=0f;
-	public static float p1=0.6f; //Sensitive - higher decreases FP and TP
-	public static float p2=1.0f;//Insensitive - higher positive decreases FP and TP
+	public static float p0=-30f;
+	public static float p1=-0.35f; //Sensitive - higher decreases FP and TP
+	public static float p2=4.0f;//Insensitive - higher positive decreases FP and TP
 	public static float p3=12f; //Higher decreases FP (substantially) and TP (slightly)
-	public static float p4=-50f; //Lower decreases FP (weakly) and TP (greatly)
+	public static float p4=-10f; //Lower decreases FP (weakly) and TP (greatly)
 	public static float p5=2.0f; //Insensitive - lower increases FP and TP
 	public static float p6=2f; //Greater increases FP and TP
 	
 //	pathScore+=q1+Tools.mid(q2*prevLength, q3+q4*prevLength, q5);
 	
 	//Operon length modifiers for opposite strand
-	public static float q1=-35f;
-	public static float q2=-1.5f; //q2 and q4 must have opposite signs
-	public static float q3=-10f;
-	public static float q4=1f; //(Lower [even negative] decreases FP and TP)
-	public static float q5=-60f;
+	public static float q1=-36f;
+	public static float q2=-1.6f; //q2 and q4 must have opposite signs
+	public static float q3=-12f;
+	public static float q4=3.0f; //(Lower [even negative] decreases FP and TP)
+	public static float q5=-40f;
 	
 	private static PrintStream outstream=System.err;
 	static boolean verbose;
