@@ -1,25 +1,20 @@
 package prok;
 
-import java.io.File;
 import java.io.PrintStream;
 import java.util.ArrayList;
 import java.util.BitSet;
 import java.util.HashMap;
 
+import aligner.SingleStateAlignerFlat2;
 import dna.AminoAcid;
-import dna.Data;
 import fileIO.FileFormat;
-import fileIO.ReadWrite;
 import gff.GffLine;
 import shared.Shared;
 import shared.Tools;
-import stream.ConcurrentReadInputStream;
 import stream.Read;
 import stream.ReadInputStream;
 import structures.ByteBuilder;
-import structures.IntHashSet;
 import structures.IntList;
-import structures.ListNum;
 
 /**
  * This class is designed to store kmer frequencies related to gene
@@ -31,7 +26,7 @@ import structures.ListNum;
  * @date Sep 24, 2018
  *
  */
-public class GeneModel {
+public class GeneModel extends ProkObject {
 	
 	/*--------------------------------------------------------------*/
 	/*----------------        Initialization        ----------------*/
@@ -40,10 +35,9 @@ public class GeneModel {
 	public GeneModel(boolean fill){
 		if(fill){
 			fillContainers();
-			fillContainerArrays();
 		}
 	}
-
+	
 	void fillContainers(){
 		statsCDS.setInner(kInnerCDS, 3);
 		statsCDS.setStart(kStartCDS, startFrames, startLeftOffset);
@@ -65,11 +59,9 @@ public class GeneModel {
 
 		stats5S.setStart(kStartRNA, 20, 5);
 		stats5S.setStop(kStopRNA, 15, 5);
-	}
 
-	void fillContainerArrays(){
-		rnaContainers=new StatsContainer[] {statstRNA, stats16S, stats23S, stats5S};
-		allContainers=new StatsContainer[] {statsCDS, statstRNA, stats16S, stats23S, stats5S};
+		stats18S.setStart(kStartRNA, 20, 7);//TODO: 18S bounds are untested and should be empirically determined
+		stats18S.setStop(kStopRNA, 12, 16);
 	}
 	
 	/*--------------------------------------------------------------*/
@@ -99,11 +91,14 @@ public class GeneModel {
 			}
 		}
 		{//Scoped to save memory
-			ArrayList<GffLine>[] allGffLines=GffLine.loadGffFileByType(gffFile, "CDS,rRNA,tRNA");
+			ArrayList<GffLine>[] allGffLines=GffLine.loadGffFileByType(gffFile, "CDS,rRNA,tRNA", true);
 			ArrayList<GffLine> cds=allGffLines[0];
 			ArrayList<GffLine> rrna=allGffLines[1];
 			ArrayList<GffLine> trna=allGffLines[2];
 			genesProcessed+=cds.size();
+			genesProcessed+=(rrna==null ? 0 : rrna.size());
+			genesProcessed+=(trna==null ? 0 : trna.size());
+			
 			HashMap<String, ScafData> scafMap=makeScafMap(scafList);
 			fillScafDataCDS(cds, scafMap);
 			fillScafDataRNA(rrna, scafMap);
@@ -130,6 +125,7 @@ public class GeneModel {
 	
 	public void add(GeneModel pgm){
 		for(int i=0; i<allContainers.length; i++){
+//			System.err.println("merging "+allContainers[i].name);
 			allContainers[i].add(pgm.allContainers[i]);
 		}
 		
@@ -137,6 +133,13 @@ public class GeneModel {
 		basesProcessed+=pgm.basesProcessed;
 		genesProcessed+=pgm.genesProcessed;
 		filesProcessed+=pgm.filesProcessed;
+
+//		geneStartsProcessed+=pgm.geneStartsProcessed;
+//		tRNAProcessed+=pgm.tRNAProcessed;
+//		r16SProcessed+=pgm.r16SProcessed;
+//		r23SProcessed+=pgm.r23SProcessed;
+//		r5SProcessed+=pgm.r5SProcessed;
+//		r18SProcessed+=pgm.r18SProcessed;
 		
 //		fnames.addAll(pgm.fnames);
 		numFiles+=pgm.numFiles;
@@ -144,6 +147,21 @@ public class GeneModel {
 		Tools.add(baseCounts, pgm.baseCounts);
 	}
 	
+	public void multiplyBy(double mult) {
+		for(int i=0; i<allContainers.length; i++){
+			allContainers[i].multiplyBy(mult);
+		}
+		
+		readsProcessed=Math.round(readsProcessed*mult);
+		basesProcessed=Math.round(basesProcessed*mult);
+		genesProcessed=Math.round(genesProcessed*mult);
+		filesProcessed=Math.round(filesProcessed*mult);
+		
+		for(int i=0; i<baseCounts.length; i++){
+			baseCounts[i]=Math.round(baseCounts[i]*mult);
+		}
+	}
+	
 	/*--------------------------------------------------------------*/
 	/*----------------        Outer Methods         ----------------*/
 	/*--------------------------------------------------------------*/
@@ -175,6 +193,7 @@ public class GeneModel {
 	}
 	
 	public void fillScafDataCDS(ArrayList<GffLine> cdsLines, HashMap<String, ScafData> scafMap){
+		if(!callCDS){return;}
 		for(GffLine gline : cdsLines){
 			ScafData sd=scafMap.get(gline.seqid);
 			assert(sd!=null) : "Can't find scaffold for GffLine "+gline.seqid;
@@ -186,7 +205,9 @@ public class GeneModel {
 		for(GffLine gline : rnaLines){
 			ScafData sd=scafMap.get(gline.seqid);
 			assert(sd!=null) : "Can't find scaffold for GffLine "+gline.seqid;
-			sd.addRNA(gline);
+			if(processType(gline.prokType())){
+				sd.addRNA(gline);
+			}
 		}
 	}
 	
@@ -281,12 +302,13 @@ public class GeneModel {
 	/*--------------------------------------------------------------*/
 	
 	private static void processGene(GffLine gline, ScafData sd){
+		if(gline.length()<2){return;}
 		final int strand=gline.strand;
 		assert(strand==sd.strand());
 		final byte[] frames=sd.frames;
 		int start=gline.start-1, stop=gline.stop-1;
 		if(start<0 || stop>=sd.length()){return;}
-		assert(start<stop);
+//		assert(start<stop) : gline; //Not always true for euks...
 		if(strand==Shared.MINUS){
 			int x=sd.length()-start-1;
 			int y=sd.length()-stop-1;
@@ -305,13 +327,13 @@ public class GeneModel {
 //		assert(gline.start!=337) : gline+"\n"+start+", "+stop;
 	}
 	
-	private void processRnaLine(GffLine gline, ScafData sd, byte flag){
+	private boolean processRnaLine(final GffLine gline, final ScafData sd, final int type){
 		final int strand=gline.strand;
 		assert(strand==sd.strand());
 		final byte[] frames=sd.frames;
 		int start=gline.start-1, stop=gline.stop-1;
-		if(start<0 || stop>=sd.length()){return;}
-		assert(start<stop);
+		if(start<0 || stop>=sd.length()){return false;}
+		assert(start<=stop);
 		if(strand==Shared.MINUS){
 			int x=sd.length()-start-1;
 			int y=sd.length()-stop-1;
@@ -319,28 +341,76 @@ public class GeneModel {
 			stop=x;
 		}
 		
-		if(flag==1){
-			//TODO
-			statstRNA.start.processPoint(sd.bases, start, 1);
-			statstRNA.stop.processPoint(sd.bases, stop, 1);
-		}else if(flag==2){
-			//TODO
-			stats16S.start.processPoint(sd.bases, start, 1);
-			stats16S.stop.processPoint(sd.bases, stop, 1);
-		}else if(flag==4){
-			//TODO
-			stats23S.start.processPoint(sd.bases, start, 1);
-			stats23S.stop.processPoint(sd.bases, stop, 1);
-		}else if(flag==8){
-			//TODO
-			stats5S.start.processPoint(sd.bases, start, 1);
-			stats5S.stop.processPoint(sd.bases, stop, 1);
+		if(AnalyzeGenes.alignRibo){
+//			byte[] seq=sd.fetch(start, stop);
+			Read[] consensusReads=ProkObject.consensusReads(type);
+			byte[] universal=(consensusReads!=null && consensusReads.length>0 ? consensusReads[0].bases : null);
+			float minIdentity=ProkObject.minID(type);
+			if(universal!=null){
+				int[] coords=new int[2];
+				final int a=Tools.max(0, start-(AnalyzeGenes.adjustEndpoints ? 200 : 50));
+				final int b=Tools.min(sd.bases.length-1, stop+(AnalyzeGenes.adjustEndpoints ? 200 : 50));
+				float id1=align(universal, sd.bases, a, b, minIdentity, coords);
+				final int rstart=coords[0], rstop=coords[1];
+//				assert(false) : rstart+", "+rstop+", "+(rstop-rstart+1)+", "+start+", "+stop;
+				if(id1<minIdentity){
+//					System.err.println("Low identity: "+String.format("%.2s", 100*id1));
+					return false;
+				}else{
+//					System.err.println("Good identity: "+String.format("%.2s", 100*id1));
+				}
+				if(AnalyzeGenes.adjustEndpoints){
+					int startSlop=startSlop(type);
+					int stopSlop=stopSlop(type);
+					if(Tools.absdif(start, rstart)>startSlop){
+//						System.err.println("rstart:\t"+start+" -> "+rstart);
+						start=rstart;
+					}
+					if(Tools.absdif(stop, rstop)>stopSlop){
+//						System.err.println("rstop: \t"+stop+" -> "+rstop);
+						stop=rstop;
+					}
+				}
+			}
 		}
 		
+		StatsContainer sc=allContainers[type];
+		sc.start.processPoint(sd.bases, start, 1);
+		sc.stop.processPoint(sd.bases, stop, 1);
+		assert(sc!=statsCDS);
+		
 		assert(start>=0) : gline.toString()+"\n"+sd.length()+"\n"+sd.name;
-		for(int i=start+kInnerRNA-1; i<stop; i++){
+		final byte flag=typeToFlag(type);
+		for(int i=start+kInnerRNA-1; i<=stop; i++){
 			frames[i]|=flag;
 		}
+		return true;
+	}
+	
+	private float align(byte[] query, byte[] ref, int start, int stop, float minIdentity, int[] coords){
+//		final int a=0, b=ref.length-1;
+		SingleStateAlignerFlat2 ssa=GeneCaller.getSSA();
+		final int minScore=ssa.minScoreByIdentity(query.length, minIdentity);
+		int[] max=ssa.fillUnlimited(query, ref, start, stop, minScore);
+		if(max==null){return 0;}
+		
+		final int rows=max[0];
+		final int maxCol=max[1];
+		final int maxState=max[2];
+//		final int maxScore=max[3];
+//		final int maxStart=max[4];
+		
+		//returns {score, bestRefStart, bestRefStop} 
+		//padded: {score, bestRefStart, bestRefStop, padLeft, padRight};
+		final int[] score=ssa.score(query, ref, start, stop, rows, maxCol, maxState);
+		final int rstart=Tools.max(score[1], 0);
+		final int rstop=Tools.min(score[2], ref.length-1);
+		if(coords!=null){
+			coords[0]=rstart;
+			coords[1]=rstop;
+		}
+		final float id=ssa.tracebackIdentity(query, ref, start, stop, rows, maxCol, maxState, null);
+		return id;
 	}
 	
 	/** 
@@ -353,7 +423,7 @@ public class GeneModel {
 	 * @param k
 	 */
 	private static void markFrames(int start, int stop, byte[] frames, int k){
-		assert(start<stop) : start+", "+stop;
+		assert(start<=stop) : start+", "+stop;
 		for(int i=start+k-1, frameBit=(1<<((k-1)%3)), max=Tools.min(stop-3, frames.length-1); i<=max; i++){
 			frames[i]=(byte)(frames[i]|frameBit);
 			frameBit<<=1;
@@ -367,6 +437,7 @@ public class GeneModel {
 	/*--------------------------------------------------------------*/
 	
 	private void processCDS(ScafData sd, int strand){
+		if(!callCDS){return;}
 		ArrayList<GffLine> glines=sd.cdsLines[strand];
 		for(GffLine gline : glines){
 			assert(gline.strand==strand);
@@ -386,40 +457,39 @@ public class GeneModel {
 		processEnds(sd.bases, statsCDS.stop, sd.stops, 0);
 	}
 	
+	private static int getGlineType(GffLine gline, ScafData sd){
+		if(!gline.inbounds(sd.length()) || gline.partial()){return -1;}
+
+		final int length=gline.length();
+		final int type=gline.prokType();
+		if(type<0){
+			return type;
+		}else if(type==CDS){
+			return type;
+		}else if(type==tRNA && length>=40 && length<=120){
+			return type;
+		}else if(type==r16S && length>=1440 && length<=1620){
+			return type;
+		}else if(type==r23S && length>=2720 && length<=3170){
+			return type;
+		}else if(type==r5S && length>=90 && length<=150){
+			return type;
+		}else if(type==r18S && length>=1400 && length<=2000){ //TODO: Check length range
+			return type;
+		}
+		return -1;
+	}
+	
 	private void processRNA(ScafData sd, int strand){
 		sd.clear();
 		ArrayList<GffLine> lines=sd.rnaLines[strand];
 		for(GffLine gline : lines){
 			assert(gline.strand==strand);
-			final byte flag;
-			if(!gline.attributes.contains("partial=true") && gline.start>1 && gline.stop<sd.length()){
-				if(gline.type.equals("tRNA")){
-					flag=1;
-					statstRNA.addLength(gline.length());
-				}else{
-					assert(gline.type.equals("rRNA"));
-					if(gline.attributes.contains("16S")){
-						flag=2;
-						stats16S.addLength(gline.length());
-					}else if(gline.attributes.contains("23S")){
-						flag=4;
-						stats23S.addLength(gline.length());
-					}else if(gline.attributes.contains("5S")){
-						flag=8;
-						stats5S.addLength(gline.length());
-//					}else if(gline.attributes.contains("SSU") || gline.attributes.contains("mall subunit")){
-//						flag=2;
-//						stats16S.addLength(gline.length());
-//					}else if(gline.attributes.contains("LSU") || gline.attributes.contains("arge subunit")){
-//						flag=4;
-//						stats23S.addLength(gline.length());
-					}else{
-						flag=0;
-//						assert(false) : gline;
-//						System.err.println("Can't determine rRNA type for "+gline);
-					}
-				}
-				processRnaLine(gline, sd, flag);
+			final int type=getGlineType(gline, sd);
+			if(type>0){
+				StatsContainer sc=allContainers[type];
+				sc.addLength(gline.length());
+				processRnaLine(gline, sd, type);
 			}
 		}
 		processRnaInner(sd);
@@ -442,7 +512,7 @@ public class GeneModel {
 				len++;
 				if(len>=k){
 					int vf=frames[i];
-					for(int type=0; type<4; type++){
+					for(int type=0; type<5; type++){
 						int valid=vf&1;
 						rnaContainers[type].inner.add(kmer, 0, valid);
 						vf=(vf>>1);
@@ -458,7 +528,7 @@ public class GeneModel {
 		final int k=stats16S.start.k;
 		final int kMax=stats16S.start.kMax;
 		final int mask=stats16S.start.mask;
-		final long[] counts=new long[kMax];//Slow
+		final long[] counts=new long[kMax];//TODO: Slow
 		
 		int kmer=0;
 		int len=0;
@@ -579,73 +649,6 @@ public class GeneModel {
 //		return right-left; //High numbers are likely to be starts; non-starts should be near 0.
 //	}
 	
-	/*--------------------------------------------------------------*/
-	/*----------------          Long Kmers          ----------------*/
-	/*--------------------------------------------------------------*/
-	
-	public synchronized void loadLongKmers(){
-		assert(ssuKmers==null);
-		if(load16Skmers){ssuKmers=loadLongKmers(stats16S, kLong16s, "ssu");}
-		if(load23Skmers){lsuKmers=loadLongKmers(stats23S, kLong23s, "lsu");}
-		if(load5Skmers){r5SKmers=loadLongKmers(stats5S, kLong5s, "5S");}
-		if(loadtRNAkmers){trnaKmers=loadLongKmers(statstRNA, kLongTRna, "tRNA");}
-	}
-	
-	private IntHashSet loadLongKmers(StatsContainer sc, int k, String prefix){
-		String fname=Data.findPath("?"+prefix+"_"+k+"mers.fa");
-		if(!new File(fname).exists()){
-			fname=fname+".gz";
-			if(!new File(fname).exists()){
-				System.err.println("Can't find "+fname);
-				return null;
-			}
-		}
-		IntHashSet set=loadLongKmers(fname, k);
-		sc.kmerSet=set;
-		sc.kLongLen=k;
-		return set;
-	}
-	
-	private static IntHashSet loadLongKmers(String fname, int k){//TODO: Consider making this a LongHashSet.  No reason not to...
-		FileFormat ff=FileFormat.testInput(fname, FileFormat.FA, null, false, false);
-		ConcurrentReadInputStream cris=ConcurrentReadInputStream.getReadInputStream(-1, false, ff, null);
-		cris.start(); //Start the stream
-		if(verbose){outstream.println("Started cris");}
-		
-		IntHashSet set=new IntHashSet(1000);
-		ListNum<Read> ln=cris.nextList();
-		while(ln!=null && ln.size()>0){
-			processList(ln, set, k);
-			cris.returnList(ln);
-			ln=cris.nextList();
-		}
-		if(ln!=null){cris.returnList(ln.id, ln.list==null || ln.list.isEmpty());}
-		ReadWrite.closeStream(cris);
-		return set;
-	}
-	
-	private static IntHashSet processList(ListNum<Read> ln, IntHashSet set, int k){
-		final int mask=~((-1)<<(2*k));
-		for(Read r : ln){
-			final byte[] bases=r.bases;
-			int kmer=0;
-			int len=0;
-			for(byte b : bases){
-				final int num=AminoAcid.baseToNumber[b];
-				if(num>=0){
-					len++;
-					kmer=((kmer<<2)|num)&mask;
-					if(len>=k){
-						set.add(kmer);
-					}
-				}else{
-					len=0;
-				}
-			}
-		}
-		return set;
-	}
-	
 	/*--------------------------------------------------------------*/
 	/*----------------           toString           ----------------*/
 	/*--------------------------------------------------------------*/
@@ -692,7 +695,10 @@ public class GeneModel {
 		}
 		bb.nl();
 
-		for(StatsContainer sc : allContainers){sc.appendTo(bb);}
+		for(StatsContainer sc : allContainers){
+			sc.appendTo(bb);
+		}
+		assert(allContainers.length>5) : allContainers.length;
 		
 		return bb;
 	}
@@ -701,19 +707,16 @@ public class GeneModel {
 	/*----------------            Stats             ----------------*/
 	/*--------------------------------------------------------------*/
 
-	public StatsContainer statsCDS=new StatsContainer("CDS", Orf.CDS);
-	public StatsContainer statstRNA=new StatsContainer("tRNA", Orf.tRNA);
-	public StatsContainer stats16S=new StatsContainer("16S", Orf.r16S);
-	public StatsContainer stats23S=new StatsContainer("23S", Orf.r23S);
-	public StatsContainer stats5S=new StatsContainer("5S", Orf.r5S);
-
-	public IntHashSet ssuKmers=null;
-	public IntHashSet lsuKmers=null;
-	public IntHashSet r5SKmers=null;
-	public IntHashSet trnaKmers=null;
+	public final StatsContainer statsCDS=new StatsContainer(CDS);
+	public final StatsContainer statstRNA=new StatsContainer(tRNA);
+	public final StatsContainer stats16S=new StatsContainer(r16S);
+	public final StatsContainer stats23S=new StatsContainer(r23S);
+	public final StatsContainer stats5S=new StatsContainer(r5S);
+	public final StatsContainer stats18S=new StatsContainer(r18S);
 	
-	StatsContainer[] rnaContainers=new StatsContainer[] {statstRNA, stats16S, stats23S, stats5S};
-	StatsContainer[] allContainers=new StatsContainer[] {statsCDS, statstRNA, stats16S, stats23S, stats5S};
+	final StatsContainer[] rnaContainers=new StatsContainer[] {statstRNA, stats16S, stats23S, stats5S, stats18S};
+	final StatsContainer[] allContainers=new StatsContainer[] {statsCDS, statstRNA, stats16S, stats23S, stats5S, stats18S};
+	//public static int CDS=0, tRNA=1, r16S=2, r23S=3, r5S=4, r18S=5, r28S=6, RNA=7;
 	
 //	public final FrameStats innerKmerStats=new FrameStats("innerKmerStats", innerKmerLength, 3, 0);
 //	public final FrameStats startStats=new FrameStats("startStats", endKmerLength, startFrames, startLeftOffset);
@@ -751,7 +754,8 @@ public class GeneModel {
 	/*----------------        Static Setters        ----------------*/
 	/*--------------------------------------------------------------*/
 
-	public void setStatics(){
+	public synchronized void setStatics(){
+//		assert(!setStatics);
 		kInnerCDS=statsCDS.inner.k;
 		kStartCDS=statsCDS.start.k;
 		kStopCDS=statsCDS.stop.k;
@@ -762,9 +766,10 @@ public class GeneModel {
 		setStopLeftOffset(statsCDS.stop.leftOffset);
 		setStopRightOffset(statsCDS.stop.rightOffset());
 		
-		kInnerRNA=stats16S.inner.k;
+		kInnerRNA=stats16S.inner.k;//TODO: Why is 16S used here?
 		kStartRNA=stats16S.start.k;
 		kStopRNA=stats16S.stop.k;
+		setStatics=true;
 	}
 	
 	public static void setInnerK(int k){
@@ -828,15 +833,6 @@ public class GeneModel {
 	public static int kInnerCDS=6;
 	public static int kStartCDS=3;
 	public static int kStopCDS=3;
-
-	public static int kInnerRNA=6;
-	public static int kStartRNA=3;
-	public static int kStopRNA=3;
-	
-	public static int kLong16s=15;
-	public static int kLong23s=15;
-	public static int kLong5s=10;
-	public static int kLongTRna=10;
 	
 	static int startLeftOffset(){return startLeftOffset;}
 	static int startRightOffset(){return startRightOffset;}
@@ -849,14 +845,8 @@ public class GeneModel {
 	private static int stopLeftOffset=9;
 	private static int stopRightOffset=12;
 	private static int stopFrames=stopLeftOffset+stopRightOffset+1;
-
-	public static boolean PROCESS_PLUS_STRAND=true;
-	public static boolean PROCESS_MINUS_STRAND=true;
 	
-	public static boolean load16Skmers=true;
-	public static boolean load23Skmers=true;
-	public static boolean load5Skmers=true;
-	public static boolean loadtRNAkmers=true;
+	private static boolean setStatics=false;
 	
 	/*--------------------------------------------------------------*/
 	/*----------------         More Statics         ----------------*/
