@@ -409,10 +409,14 @@ public class Parser {
 	}
 	
 	public boolean parseMapping(String arg, String a, String b){
-		if(a.equals("idfilter") || a.equals("identityfilter")){
-			idFilter=Float.parseFloat(b);
-			if(idFilter>1f){idFilter/=100;}
-			assert(idFilter<=1f) : "idfilter should be between 0 and 1.";
+		if(a.equals("idfilter") || a.equals("identityfilter") || a.equals("minidfilter") || a.equals("minidentityfilter") || a.equals("minid")){
+			minIdFilter=Float.parseFloat(b);
+			if(minIdFilter>1f){minIdFilter/=100;}
+			assert(minIdFilter<=1f) : "idfilter should be between 0 and 1.";
+		}else if(a.equals("maxidfilter") || a.equals("maxidentityfilter") || a.equals("maxid")){
+			maxIdFilter=Float.parseFloat(b);
+			if(maxIdFilter>1f){maxIdFilter/=100;}
+			assert(maxIdFilter<=1f) : "idfilter should be between 0 and 1.";
 		}else if(a.equals("subfilter")){
 			subfilter=Integer.parseInt(b);
 		}else if(a.equals("clipfilter")){
@@ -528,7 +532,10 @@ public class Parser {
 			else if("crash".equalsIgnoreCase(b) || "fail".equalsIgnoreCase(b)){Read.JUNK_MODE=Read.CRASH_JUNK;}
 			else if("fix".equalsIgnoreCase(b)){Read.JUNK_MODE=Read.FIX_JUNK;}
 			else if("flag".equalsIgnoreCase(b) || "discard".equalsIgnoreCase(b)){Read.JUNK_MODE=Read.FLAG_JUNK;}
+			else if("iupacton".equalsIgnoreCase(b)){Read.JUNK_MODE=Read.FIX_JUNK_AND_IUPAC;}
 			else{assert(false) : "Bad junk mode: "+arg;}
+		}else if(a.equals("undefinedton") || a.equals("iupacton") || a.equals("itn")){
+			Read.IUPAC_TO_N=Tools.parseBoolean(b);
 		}else if(a.equals("ignorejunk")){
 			boolean x=Tools.parseBoolean(b);
 			if(x){Read.JUNK_MODE=Read.IGNORE_JUNK;}
@@ -620,12 +627,12 @@ public class Parser {
 			Read.setMinCalledQuality((byte)x);
 		}else if(a.equals("t") || a.equals("threads")){
 			Shared.setThreads(b);
-			System.err.println("Set threads to "+Shared.threads());
+			if(!silent){System.err.println("Set threads to "+Shared.threads());}
 		}else if(a.equals("recalpairnum") || a.equals("recalibratepairnum")){
 			CalcTrueQuality.USE_PAIRNUM=Tools.parseBoolean(b);
 		}else if(a.equals("taxpath")){
 			if("auto".equalsIgnoreCase(b)){
-				TaxTree.TAX_PATH=TaxTree.defaultTaxPath;
+				TaxTree.TAX_PATH=TaxTree.defaultTaxPath();
 			}else{
 				TaxTree.TAX_PATH=b.replaceAll("\\\\", "/");
 			}
@@ -640,6 +647,8 @@ public class Parser {
 			FastaReadInputStream.WARN_FIRST_TIME_ONLY=Tools.parseBoolean(b);
 		}else if(a.equals("silva")){
 			TaxTree.SILVA_MODE=Tools.parseBoolean(b);
+		}else if(a.equals("unite")){
+			TaxTree.UNITE_MODE=Tools.parseBoolean(b);
 		}else if(a.equals("parallelsort") || a.equals("paralellsort")){
 			Shared.parallelSort=Tools.parseBoolean(b);
 		}else if(a.equals("imghq")){
@@ -682,6 +691,12 @@ public class Parser {
 			Tools.FORCE_JAVA_PARSE_DOUBLE=Tools.parseBoolean(b);
 		}
 		
+		else if(a.equalsIgnoreCase("awsServers") || a.equalsIgnoreCase("aws")){
+			Shared.awsServers=Tools.parseBoolean(b);
+		}else if(a.equalsIgnoreCase("nerscServers") || a.equalsIgnoreCase("nersc")){
+			Shared.awsServers=!Tools.parseBoolean(b);
+		}
+		
 		else{
 			return false;
 		}
@@ -990,6 +1005,8 @@ public class Parser {
 				}
 			}
 			SamLine.setxs=true;
+		}else if(a.equals("flipsam")){
+			SamLine.FLIP_ON_LOAD=Tools.parseBoolean(b);
 		}else if(parseReadgroup(arg, a, b)){
 			//do nothing
 		}else{
@@ -1006,7 +1023,7 @@ public class Parser {
 			FastaReadInputStream.MIN_READ_LEN=Integer.parseInt(b);
 		}else if(a.equals("forcesectionname")){
 			FastaReadInputStream.FORCE_SECTION_NAME=Tools.parseBoolean(b);
-		}else if(a.equals("fastawrap")){
+		}else if(a.equals("fastawrap") || a.equals("wrap")){
 			Shared.FASTA_WRAP=Tools.parseIntKMG(b);
 		}else if(a.equals("fastadump")){
 			AbstractKmerTable.FASTA_DUMP=Tools.parseBoolean(b);
@@ -1175,6 +1192,49 @@ public class Parser {
 		return true;
 	}
 	
+	/** Fix Readgroup Strings */
+	public static void postparseReadgroup(String fname){
+		if(fname==null){return;}
+		fname=ReadWrite.stripToCore(fname);
+		if("filename".equalsIgnoreCase(SamLine.READGROUP_ID)){
+			SamLine.READGROUP_ID=fname;
+			if(fname!=null){SamLine.READGROUP_TAG="RG:Z:"+fname;}
+		}
+		if("filename".equalsIgnoreCase(SamLine.READGROUP_CN)){
+			SamLine.READGROUP_CN=fname;
+		}
+		if("filename".equalsIgnoreCase(SamLine.READGROUP_DS)){
+			SamLine.READGROUP_DS=fname;
+		}
+		if("filename".equalsIgnoreCase(SamLine.READGROUP_DT)){
+			SamLine.READGROUP_DT=fname;
+		}
+		if("filename".equalsIgnoreCase(SamLine.READGROUP_FO)){
+			SamLine.READGROUP_FO=fname;
+		}
+		if("filename".equalsIgnoreCase(SamLine.READGROUP_KS)){
+			SamLine.READGROUP_KS=fname;
+		}
+		if("filename".equalsIgnoreCase(SamLine.READGROUP_LB)){
+			SamLine.READGROUP_LB=fname;
+		}
+		if("filename".equalsIgnoreCase(SamLine.READGROUP_PG)){
+			SamLine.READGROUP_PG=fname;
+		}
+		if("filename".equalsIgnoreCase(SamLine.READGROUP_PI)){
+			SamLine.READGROUP_PI=fname;
+		}
+		if("filename".equalsIgnoreCase(SamLine.READGROUP_PL)){
+			SamLine.READGROUP_PL=fname;
+		}
+		if("filename".equalsIgnoreCase(SamLine.READGROUP_PU)){
+			SamLine.READGROUP_PU=fname;
+		}
+		if("filename".equalsIgnoreCase(SamLine.READGROUP_SM)){
+			SamLine.READGROUP_SM=fname;
+		}
+	}
+	
 	
 	/*--------------------------------------------------------------*/
 	/*----------------            Fields            ----------------*/
@@ -1243,7 +1303,8 @@ public class Parser {
 	public boolean untrim=false;
 	public boolean tossJunk=false;
 
-	public float idFilter=-1;
+	public float minIdFilter=-1;
+	public float maxIdFilter=999999999;
 	public int subfilter=-1;
 	public int clipfilter=-1;
 	public int delfilter=-1;
@@ -1292,12 +1353,12 @@ public class Parser {
 	public String extin=null;
 	public String extout=null;
 	
-	public boolean silent=false;
-	
 	/*--------------------------------------------------------------*/
 	/*----------------        Static Fields         ----------------*/
 	/*--------------------------------------------------------------*/
 	
+	public static boolean silent=false;
+	
 	private static boolean setTrimRname=false;
 	private static byte qin=-1;
 	private static byte qout=-1;
