@@ -26,6 +26,7 @@ import stream.FastaReadInputStream;
 import stream.Read;
 import structures.EntropyTracker;
 import structures.IntList;
+import structures.IntListCompressor;
 import structures.ListNum;
 import structures.LongList;
 import tax.AccessionToTaxid;
@@ -82,6 +83,8 @@ public class BlacklistMaker extends SketchObject {
 		}
 		
 		//Set shared static variables
+		if(Shared.threads()>=32){Shared.capThreads(Shared.threads()/2);}//Deals with hyperthreading, which causes excessive memory usage
+		Shared.capBuffers(Tools.max(4, Shared.threads()+1));
 		ReadWrite.USE_PIGZ=ReadWrite.USE_UNPIGZ=true;
 		ReadWrite.MAX_ZIP_THREADS=Shared.threads();
 		KmerCountAbstract.SKETCH_MODE=true;
@@ -148,7 +151,7 @@ public class BlacklistMaker extends SketchObject {
 				outName=b;
 			}else if(a.equalsIgnoreCase("name0") || a.equalsIgnoreCase("nm0")){
 				sketchName=b;
-			}else if(a.equals("taxid")){
+			}else if(a.equals("taxid") || a.equals("tid")){
 				outTaxid=Integer.parseInt(b);
 			}else if(parseMode(arg, a, b)>-1){
 				mode_=parseMode(arg, a, b);
@@ -156,12 +159,7 @@ public class BlacklistMaker extends SketchObject {
 				outHist=b;
 			}
 			
-			
-			else if(a.equals("silva")){
-				TaxTree.SILVA_MODE=Tools.parseBoolean(b);
-			}
-			
-			else if(a.equals("taxlevel") || a.equals("level")){
+			else if(a.equals("taxlevel") || a.equals("tl") || a.equals("level") || a.equals("lv")){
 				if(b==null){taxLevel=-1;}
 				else if(Tools.isDigit(b.charAt(0))){
 					taxLevel=Integer.parseInt(b);
@@ -261,9 +259,9 @@ public class BlacklistMaker extends SketchObject {
 		ffin1=FileFormat.testInput(in1, FileFormat.FASTQ, extin, true, true);
 		ffin2=FileFormat.testInput(in2, FileFormat.FASTQ, extin, true, true);
 		
-		tool=new SketchTool(targetSketchSize, defaultParams.minKeyOccuranceCount, false, false);
+//		tool=new SketchTool(targetSketchSize, defaultParams.minKeyOccuranceCount, false, false);
 		
-		if(taxTreeFile!=null){setTaxtree(taxTreeFile);}
+		if(taxTreeFile!=null){setTaxtree(taxTreeFile, outstream);}
 		
 		if(giTableFile!=null){
 			loadGiToTaxid();
@@ -280,7 +278,7 @@ public class BlacklistMaker extends SketchObject {
 		
 		maps=new HashMap[ways];
 		for(int i=0; i<ways; i++){
-			maps[i]=new HashMap<Long, ListHolder>();
+			maps[i]=new HashMap<Long, IntListCompressor>();
 		}
 		
 		calcMemory();
@@ -404,7 +402,7 @@ public class BlacklistMaker extends SketchObject {
 		int max=1000000;
 		long[] counts=new long[max+1];
 		for(int i=0; i<ways; i++){
-			for(Entry<Long, ListHolder> entry : maps[i].entrySet()){
+			for(Entry<Long, IntListCompressor> entry : maps[i].entrySet()){
 				IntList value=entry.getValue().list;
 				value.sort();
 				value.shrinkToUnique();
@@ -435,14 +433,14 @@ public class BlacklistMaker extends SketchObject {
 		ArrayList<String> meta=new ArrayList<String>();
 		meta.add("minTaxCount:"+minTaxCount);
 		meta.add("taxLevel:"+taxLevel);
-		Sketch sk=new Sketch(array, null, null, outTaxid, -1, -1, -1, -1, -1, "blacklist", sketchName, ffin1.simpleName(), meta);
+		Sketch sk=new Sketch(array, null, null, null, null, outTaxid, -1, -1, -1, -1, -1, "blacklist", sketchName, ffin1.simpleName(), meta);
 		return sk;
 	}
 	
 	private long[] toArray(boolean destroy){
 		LongList list=new LongList();
 		for(int i=0; i<ways; i++){
-			for(Entry<Long, ListHolder> entry : maps[i].entrySet()){
+			for(Entry<Long, IntListCompressor> entry : maps[i].entrySet()){
 				Long key=entry.getKey();
 				IntList value=entry.getValue().list;
 				if(value.size()>=minTaxCount){
@@ -460,10 +458,7 @@ public class BlacklistMaker extends SketchObject {
 	/*--------------------------------------------------------------*/
 
 	private void calcMemory(){
-		long memory=Shared.memAvailable(); //Shared.memFree();
-		double xmsRatio=Shared.xmsRatio();
-		//			long tmemory=Runtime.getRuntime().totalMemory();
-		long usableMemory=(long)Tools.max(((memory-96000000)*(xmsRatio>0.97 ? 0.82 : 0.72)), memory*0.45);
+		long usableMemory=Shared.memAvailableAdvanced();
 		
 //		System.err.println(memory/1000000+", "+usableMemory/1000000+", "+usableMemory/1000000+", "+prepasses);
 		
@@ -810,15 +805,15 @@ public class BlacklistMaker extends SketchObject {
 		
 		
 		void addToMap(long key0, int value){
-//			Long key=Long.valueOf(key0);
-			Long key=new Long(key0);
-			HashMap<Long, ListHolder> map=maps[(int)(key%ways)];
-			ListHolder lh=map.get(key);
+			Long key=Long.valueOf(key0);
+//			Long key=new Long(key0);
+			HashMap<Long, IntListCompressor> map=maps[(int)(key%ways)];
+			IntListCompressor lh=map.get(key);
 			if(lh==null){
 				synchronized(map){
 					lh=map.get(key);
 					if(lh==null){
-						lh=new ListHolder();
+						lh=new IntListCompressor();
 						map.put(key, lh);
 					}
 				}
@@ -846,32 +841,12 @@ public class BlacklistMaker extends SketchObject {
 		private final EntropyTracker eTracker;
 	}
 	
-	/*--------------------------------------------------------------*/
-	
-	private class ListHolder {
-		
-		public void add(int value){
-			list.add(value);
-			if(list.freeSpace()==0 && lastCompression<0.75f*list.size()){
-				list.sort();
-				list.shrinkToUnique();
-				lastCompression=list.size();
-			}
-		}
-		
-		IntList list=new IntList(4);
-		int lastCompression=0;
-		
-	}
-	
 	/*--------------------------------------------------------------*/
 	/*----------------            Fields            ----------------*/
 	/*--------------------------------------------------------------*/
 	
 	private final int mode;
 	
-	private final SketchTool tool;
-	
 	private String giTableFile=null;
 	private String taxTreeFile=null;
 	private String accessionFile=null;
@@ -897,7 +872,7 @@ public class BlacklistMaker extends SketchObject {
 	long filterMemory0;
 	long filterMemory1;
 	
-	private HashMap<Long, ListHolder>[] maps;
+	private HashMap<Long, IntListCompressor>[] maps;
 	
 	public KCountArray prefilterArray=null;
 	
