@@ -12,7 +12,7 @@ import fileIO.ByteFile;
 import fileIO.ByteStreamWriter;
 import fileIO.FileFormat;
 import fileIO.ReadWrite;
-import fileIO.TextStreamWriter;
+import kmer.AbstractKmerTableSet;
 import shared.Parser;
 import shared.PreParser;
 import shared.ReadStats;
@@ -59,6 +59,8 @@ public class CompareSketch extends SketchObject {
 		
 		//Close the print stream if it was redirected
 		Shared.closeStream(x.outstream);
+		
+		alignerPool.poison();
 	}
 	
 	/**
@@ -72,6 +74,8 @@ public class CompareSketch extends SketchObject {
 //			PreParser pp=new PreParser(args, getClass(), false);
 			args=pp.args;
 			outstream=pp.outstream;
+			silent=PreParser.silent;
+			if(silent){AbstractKmerTableSet.DISPLAY_PROGRESS=false;}
 		}
 		
 		//Set shared static variables
@@ -79,6 +83,7 @@ public class CompareSketch extends SketchObject {
 		ReadWrite.MAX_ZIP_THREADS=Shared.threads();
 		KILL_OK=true;
 		TaxFilter.REQUIRE_PRESENT=false;
+		defaultParams.mode=PER_FILE;
 		
 		//Create a parser object
 		Parser parser=new Parser();
@@ -126,6 +131,8 @@ public class CompareSketch extends SketchObject {
 				outFname=b;
 			}else if(a.equals("outsketch") || a.equals("sketchout") || a.equals("outs") || a.equals("sketch")){
 				outSketch=b;
+			}else if(a.equals("files")){
+				sketchFiles=Integer.parseInt(b);
 			}else if(a.equals("taxid") || a.equals("tid")){
 				outTaxID=Integer.parseInt(b);
 			}else if(a.equals("spid")){
@@ -209,7 +216,7 @@ public class CompareSketch extends SketchObject {
 			ByteFile.FORCE_MODE_BF2=true;
 		}
 		
-		ffout=FileFormat.testOutput(out, FileFormat.TEXT, null, false, overwrite, append, false);
+		ffout=FileFormat.testOutput(out, FileFormat.TEXT, null, false, overwrite, append, ordered);
 		if(!ffout.stdio() && !defaultParams.setColors){defaultParams.printColors=false;}
 		
 		//Ensure input files can be read
@@ -228,8 +235,10 @@ public class CompareSketch extends SketchObject {
 		
 //		assert(makeIndex || defaultParams.printContam2==false) : "Contam2 requires the flag index=t";
 		
-		if(taxTreeFile!=null){setTaxtree(taxTreeFile);}
+		SSUMap.load(outstream);
+		if(taxTreeFile!=null){setTaxtree(taxTreeFile, silent ? null : outstream);}
 		defaultParams.postParse(true, true);
+		if(!defaultParams.printSSU){processSSU=false;}
 		allowMultithreadedFastq=in.size()<2 && !allToAll;
 		if(!allowMultithreadedFastq){Shared.capBufferLen(40);}
 //		assert(defaultParams.checkValid());
@@ -244,7 +253,7 @@ public class CompareSketch extends SketchObject {
 		
 		t.start();
 		
-		outstream.println("Loading sketches.");
+		if(!silent){outstream.println("Loading sketches.");}
 		searcher.makeTool(1, false, defaultParams.mergePairs);
 		SketchTool tool=new SketchTool(targetSketchSize, defaultParams);
 		
@@ -268,6 +277,7 @@ public class CompareSketch extends SketchObject {
 					if(outFname!=null){sk.setFname(outFname);}
 					if(outName0!=null){sk.setName0(outName0);}
 					sk.setMeta(outMeta);
+					if(defaultParams.printSSU()){sk.loadSSU();}//since taxID was just set
 				}
 				
 				if(outTaxID>0){
@@ -292,21 +302,12 @@ public class CompareSketch extends SketchObject {
 		}
 		
 		if(outSketch!=null){
-			ByteBuilder bb=new ByteBuilder();
-			ByteStreamWriter bsw=new ByteStreamWriter(outSketch, overwrite, append, true, FileFormat.SKETCH);
-			bsw.start();
-			for(Sketch sk : inSketches){
-				sk.toBytes(bb);
-				bsw.print(bb);
-				bb.clear();
-			}
-			bsw.poisonAndWait();
-			errorState|=bsw.errorState;
+			writeSketches(outSketch, sketchFiles);
 		}
 		
 		final int numLoaded=(inSketches.size()+searcher.refSketchCount())/(allToAll ? 2 : 1);
 		t.stop();
-		outstream.println("Loaded "+numLoaded+" sketch"+(numLoaded==1 ? "" : "es")+" in "+t.toString());
+		if(!silent){outstream.println("Loaded "+numLoaded+" sketch"+(numLoaded==1 ? "" : "es")+" in "+t.toString());}
 		if(printMemory){
 			System.gc();
 			Shared.printMemory();
@@ -320,19 +321,29 @@ public class CompareSketch extends SketchObject {
 		t.start();
 
 		
-		TextStreamWriter tsw=(ffout==null ? null : new TextStreamWriter(ffout));
+		ByteStreamWriter tsw=(ffout==null ? null : new ByteStreamWriter(ffout));
 		if(tsw!=null){
 			tsw.start();
-			if(defaultParams.format==DisplayParams.FORMAT_QUERY_REF_ANI || defaultParams.format==DisplayParams.FORMAT_CONSTELLATION){tsw.println(defaultParams.header());}
+			if(defaultParams.format==DisplayParams.FORMAT_QUERY_REF_ANI || defaultParams.format==DisplayParams.FORMAT_CONSTELLATION){
+				String s=defaultParams.header()+"\n";
+				tsw.forcePrint(s.getBytes());
+			}
 		}
 
 		boolean success=true;
 		final int inSize=inSketches.size();
 		if(inSize==1 || Shared.threads()<2 || inSize<4){
-			StringBuilder sb=new StringBuilder();
+			ByteBuilder sb=new ByteBuilder();
 			success=searcher.compare(inSketches, sb, defaultParams, Shared.threads());
 			success&=(!searcher.errorState);
-			if(tsw!=null){tsw.println(sb);}
+			if(tsw!=null){
+				sb.append('\n');
+				if(ordered){
+					tsw.addJob(sb);
+				}else{
+					tsw.println(sb);
+				}
+			}
 		}else{//More sketches than threads, and more than one thread
 			final int threads=Tools.min(Shared.threads(), inSize);
 			
@@ -370,9 +381,54 @@ public class CompareSketch extends SketchObject {
 //			allToAll ? (inSketches.size()*(long)(inSketches.size()-(compareSelf ? 0 : 1)))
 //					: inSketches.size()*(long)searcher.refSketchCount());
 		long comparisons=searcher.comparisons.get();
-		if(!skipCompare) {outstream.println("\nRan "+comparisons+" comparison"+(comparisons==1 ? "" : "s")+" in "+t);}
+		if(!skipCompare && !silent) {outstream.println("\nRan "+comparisons+" comparison"+(comparisons==1 ? "" : "s")+" in "+t);}
 		ttotal.stop();
-		outstream.println("Total Time: \t"+ttotal);
+		if(!silent){outstream.println("Total Time: \t"+ttotal);}
+	}
+	
+	void writeSketches(String fname, int files){
+		if(fname==null){return;}
+		if(files==1 || fname.indexOf('#')<0){
+			writeOneSketchFile(fname);
+		}else{
+			writeManySketchFiles(fname, files);
+		}
+	}
+	
+	void writeOneSketchFile(String fname){
+		if(fname==null){return;}
+		ByteBuilder bb=new ByteBuilder();
+		ByteStreamWriter bsw=new ByteStreamWriter(outSketch, overwrite, append, true, FileFormat.SKETCH);
+		bsw.start();
+		for(Sketch sk : inSketches){
+			sk.toBytes(bb);
+			bsw.print(bb);
+			bb.clear();
+		}
+		bsw.poisonAndWait();
+		errorState|=bsw.errorState;
+	}
+	
+	void writeManySketchFiles(String fname, int files){
+		if(fname==null){return;}
+		assert(fname.indexOf('#')>=0) : fname;
+		assert(files>0) : files;
+		
+		ByteStreamWriter[] bswa=new ByteStreamWriter[files];
+		for(int i=0; i<files; i++){
+			ByteStreamWriter bsw=new ByteStreamWriter(outSketch.replaceFirst("#", ""+i), overwrite, append, true, FileFormat.SKETCH);
+			bsw.start();
+			bswa[i]=bsw;
+		}
+		for(Sketch sk : inSketches){
+			ByteBuilder bb=new ByteBuilder(4096);
+			sk.toBytes(bb);
+			bswa[sk.sketchID%files].addJob(bb);
+		}
+		for(ByteStreamWriter bsw : bswa){
+			bsw.poisonAndWait();
+			errorState|=bsw.errorState;
+		}
 	}
 	
 	/*--------------------------------------------------------------*/
@@ -396,7 +452,7 @@ public class CompareSketch extends SketchObject {
 	
 	private class CompareThread extends Thread {
 		
-		CompareThread(final int tid_, final AtomicInteger nextSketch_, TextStreamWriter tsw_){
+		CompareThread(final int tid_, final AtomicInteger nextSketch_, ByteStreamWriter tsw_){
 			tid=tid_;
 			nextSketch=nextSketch_;
 			tsw=tsw_;
@@ -406,6 +462,7 @@ public class CompareSketch extends SketchObject {
 		public void run(){
 			success=false;
 			final int inLim=inSketches.size();
+			final boolean json=defaultParams.json();
 			
 			for(int inNum=nextSketch.getAndIncrement(); inNum<inLim; inNum=nextSketch.getAndIncrement()){
 				Sketch a=inSketches.get(inNum);
@@ -414,13 +471,31 @@ public class CompareSketch extends SketchObject {
 				a.clearRefHitCounts();
 				
 				if(tsw!=null){
+					ByteBuilder sb=sr.toText(defaultParams);
 					synchronized(tsw){
 						if(ordered){
-							ByteBuilder sb=sr.toText(defaultParams);
-							tsw.writeOrdered(sb, inNum);
+							if(json){
+								if(inNum==0){
+									sb.insert(0, (byte)'[');//Rare, slow case
+								}
+								if(inNum<inLim-1){
+									sb.append(',');
+								}else{
+									sb.append(']');
+								}
+							}
+							tsw.add(sb, inNum);
 						}else{
-							sr.writeResults(defaultParams, tsw);
+							if(json){
+								if(resultsPrinted==0){
+									tsw.print('[');
+								}else{
+									sb.insert(0, (byte)',');
+								}
+							}
+							tsw.print(sb);
 						}
+						resultsPrinted++;
 					}
 				}
 			}
@@ -433,7 +508,7 @@ public class CompareSketch extends SketchObject {
 		private final AtomicInteger nextSketch;
 		private final AtomicInteger fakeID=new AtomicInteger(minFakeID);
 		private ConcurrentHashMap<Integer, Comparison> map=new ConcurrentHashMap<Integer, Comparison>(101);
-		final TextStreamWriter tsw;
+		final ByteStreamWriter tsw;
 		
 		boolean success=false;
 		
@@ -454,16 +529,19 @@ public class CompareSketch extends SketchObject {
 	public final SketchSearcher searcher=new SketchSearcher();
 	
 	private boolean printMemory=false;
+	private boolean silent=false;
 	
 	/*Override metadata */
 	private String outTaxName=null;
 	private String outFname=null;
 	private String outName0=null;
 	private String outSketch=null;
+	private int sketchFiles=1;
 	private int outTaxID=-1;
 	private long outSpid=-1;
 	private long outImgID=-1;
 	private ArrayList<String> outMeta=null;
+	private long resultsPrinted=0;
 	
 	/*--------------------------------------------------------------*/
 	/*----------------         Final Fields         ----------------*/
