@@ -505,7 +505,7 @@ public class KmerLimit2 extends SketchObject {
 //		long[] array_, int[] counts_, int taxID_, long imgID_, long gSizeBases_, long gSizeKmers_, long gSequences_, double probCorrect_,
 //		String taxName_, String name0_, String fname_, ArrayList<String> meta_
 		
-		Sketch sk=new Sketch(keys, counts, null, -1, -1, 
+		Sketch sk=new Sketch(keys, counts, null, null, null, -1, -1, 
 				sketch0.genomeSizeBases, sketch0.genomeSizeKmers, sketch0.genomeSequences, sketch0.probCorrect,
 				null, null, null, null);
 		
