@@ -85,6 +85,7 @@ public class SendSketch extends SketchObject {
 		parser.out1="stdout.txt";
 		
 		defaultParams.inputVersion=Shared.BBMAP_VERSION_STRING;
+		defaultParams.mode=PER_FILE;
 		boolean setBlacklist=false;
 		boolean setLocal=false;
 		boolean setPrintDepth=false;
@@ -102,7 +103,7 @@ public class SendSketch extends SketchObject {
 				verbose=Tools.parseBoolean(b);
 			}else if(a.equals("in")){
 				addFiles(b, in);
-			}else if(a.equals("blacklist")){
+			}else if(a.equals("blacklist") || a.equalsIgnoreCase("bl")){
 				setBlacklist=true;
 				parseSketchFlags(arg, a, b);
 			}else if(parseSketchFlags(arg, a, b)){
@@ -120,36 +121,19 @@ public class SendSketch extends SketchObject {
 			}else if(a.equals("address")){
 				assert(b!=null) : "Bad parameter: "+arg;
 				address=b;
-			}else if(a.equals("alternateaddress") || a.equals("altaddress") || a.equalsIgnoreCase("vm")){
-				boolean alt=Tools.parseBoolean(b);
-				switchDefaultAddresses(alt);
-			}else if(a.equalsIgnoreCase("nt")){
-				address=toAddress(a);
-				if(blacklist==null){blacklist=Blacklist.toBlacklist(a);}
-			}else if(a.equalsIgnoreCase("silva") || a.equalsIgnoreCase("ribo")){
-				address=toAddress(a);
-				if(blacklist==null){blacklist=Blacklist.toBlacklist(a);}
-			}else if(a.equalsIgnoreCase("refseq")){
-				address=toAddress(a);
-				if(blacklist==null){blacklist=Blacklist.toBlacklist(a);}
-			}else if(a.equalsIgnoreCase("img")){
-				address=toAddress(a);
-				if(blacklist==null){blacklist=Blacklist.toBlacklist(a);}
-			}else if(a.equalsIgnoreCase("nr")){
-				address=toAddress(a);
-				if(blacklist==null){blacklist=Blacklist.toBlacklist(a);}
-				amino=true;
-			}else if(a.equalsIgnoreCase("refseqprot") || a.equalsIgnoreCase("prokprot") 
-					|| a.equalsIgnoreCase("protein") || a.equalsIgnoreCase("protien") || a.equalsIgnoreCase("prot")){
-				address=toAddress(a);
-				if(blacklist==null){blacklist=Blacklist.toBlacklist(a);}
-				translate=true;
-			}else if(a.equalsIgnoreCase("mito")){
-				address=toAddress(a);
-				if(blacklist==null){blacklist=Blacklist.toBlacklist(a);}
-			}else if(a.equalsIgnoreCase("fungi")){
-				address=toAddress(a);
-				if(blacklist==null){blacklist=Blacklist.toBlacklist(a);}
+			}
+//			else if(a.equals("alternateaddress") || a.equals("altaddress") || a.equalsIgnoreCase("vm")){
+//				boolean alt=Tools.parseBoolean(b);
+//				switchDefaultAddresses(alt);
+//			}
+			
+
+			else if(a.equalsIgnoreCase("nt") || a.equalsIgnoreCase("silva") || a.equalsIgnoreCase("ribo") || 
+					a.equalsIgnoreCase("refseq") || a.equalsIgnoreCase("img") || a.equalsIgnoreCase("nr") || 
+					a.equalsIgnoreCase("refseqprot") || a.equalsIgnoreCase("prokprot") || a.equalsIgnoreCase("protein") || 
+					a.equalsIgnoreCase("protien") || a.equalsIgnoreCase("prot") || a.equalsIgnoreCase("mito") || a.equalsIgnoreCase("fungi")){
+				address=a;
+				blacklist=a;
 			}
 			
 			else if(a.equals("taxtree") || a.equals("tree")){
@@ -197,9 +181,9 @@ public class SendSketch extends SketchObject {
 		outMeta=SketchObject.fixMeta(outMeta);
 		
 		if(address!=null && !SET_AUTOSIZE_FACTOR){
-			if(address.equals(refseqAddress)){
+			if(address.equals(refseqAddress())){
 				AUTOSIZE_FACTOR=2.0f;
-			}else if(address.equals(prokProtAddress)){
+			}else if(address.equals(prokProtAddress())){
 				AUTOSIZE_FACTOR=3.0f;
 			}
 		}
@@ -259,6 +243,7 @@ public class SendSketch extends SketchObject {
 			throw new RuntimeException("\nSome file names were specified multiple times.\n");
 		}
 		
+		SSUMap.load(outstream);
 		tool=new SketchTool(targetSketchSize, defaultParams);
 		
 //		assert(false) : defaultParams.toString()+"\n"+k+", "+amino+", "+HASH_VERSION;
@@ -268,43 +253,44 @@ public class SendSketch extends SketchObject {
 			if(blacklist!=null){outstream.println("Using a blacklist.");}
 		}
 		
-		if(taxTreeFile!=null){setTaxtree(taxTreeFile);}
+		if(taxTreeFile!=null){setTaxtree(taxTreeFile, silent ? null : outstream);}
 		defaultParams.postParse(false, false);
+		if(!defaultParams.printSSU){processSSU=false;}
 		allowMultithreadedFastq=(in.size()==1 && Shared.threads()>2);
 		if(!allowMultithreadedFastq){Shared.capBufferLen(40);}
 	}
 	
 	private void setFromAddress(String address, boolean setBlacklist){
-		if(address.equals(nrAddress)){
+		if(address.equals(nrAddress())){
 			amino=true;
 			if(!setK){k=defaultKAmino; k2=defaultK2Amino;}
 //			defaultParams.dbName="nr";
 			assert(false) : "Need to set K.";
-		}else if(address.equals(prokProtAddress)){
+		}else if(address.equals(prokProtAddress())){
 			translate=true;
 			if(!setK){k=defaultKAmino; k2=defaultK2Amino;}
 //			defaultParams.dbName="nr";
 			if(blacklist==null && !setBlacklist){blacklist=Blacklist.prokProtBlacklist();}
-		}else if(address.equals(ntAddress)){
+		}else if(address.equals(ntAddress())){
 			if(!setK){k=defaultK; k2=defaultK2;}
 //			defaultParams.dbName="nt";
 			if(blacklist==null && !setBlacklist){blacklist=Blacklist.ntBlacklist();}
-		}else if(address.equals(refseqAddress)){
+		}else if(address.equals(refseqAddress())){
 			if(!setK){k=defaultK; k2=defaultK2;}
 //			defaultParams.dbName="RefSeq";
 			if(blacklist==null && !setBlacklist){blacklist=Blacklist.refseqBlacklist();}
-		}else if(address.equals(silvaAddress)){
+		}else if(address.equals(silvaAddress())){
 			if(!setK){k=defaultK; k2=defaultK2;}
 //			defaultParams.dbName="Silva";
 			if(blacklist==null && !setBlacklist){blacklist=Blacklist.silvaBlacklist();}
-		}else if(address.equals(imgAddress)){
+		}else if(address.equals(imgAddress())){
 			if(!setK){k=defaultK; k2=defaultK2;}
 //			defaultParams.dbName="IMG";
 			if(blacklist==null && !setBlacklist){blacklist=Blacklist.imgBlacklist();}
-		}else if(address.equals(mitoAddress)){
+		}else if(address.equals(mitoAddress())){
 			if(!setK){k=defaultK; k2=defaultK2;}
 			if(blacklist==null && !setBlacklist){blacklist=Blacklist.mitoBlacklist();}
-		}else if(address.equals(fungiAddress)){
+		}else if(address.equals(fungiAddress())){
 			if(!setK){k=defaultK; k2=defaultK2;}
 			if(blacklist==null && !setBlacklist){blacklist=Blacklist.fungiBlacklist();}
 		}
@@ -326,6 +312,10 @@ public class SendSketch extends SketchObject {
 		t.start();
 		inSketches=tool.loadSketches_MT(defaultParams, in);
 		final int numLoaded=(inSketches.size());
+		if(numLoaded>1 && defaultParams.mode==PER_FILE){
+			inSketches.sort(SketchIdComparator.comparator);//Otherwise they come out disordered
+		}
+		
 		t.stop();
 		if(!silent){outstream.println("Loaded "+numLoaded+" sketch"+(numLoaded==1 ? "" : "es")+" in "+t);}
 		assert(numLoaded<=MAX_ALLOWED_SKETCHES) : "\nSendSketch is configured to send at most "+MAX_ALLOWED_SKETCHES+" to prevent overwhelming the server.\n"
@@ -354,6 +344,8 @@ public class SendSketch extends SketchObject {
 		for(Sketch sk : inSketches){
 			
 			if(sk.taxID<1 || sk.taxID>=minFakeID || outTaxID>0){sk.taxID=outTaxID;}
+			if(defaultParams.printSSU()){sk.loadSSU();}
+			
 			if(outSpid>0){sk.spid=outSpid;}
 			if(outImgID>0){sk.imgID=outImgID;}
 			if(outTaxName!=null){sk.setTaxName(outTaxName);}
@@ -627,26 +619,26 @@ public class SendSketch extends SketchObject {
 	public static final String toAddress(String b){
 		String address=b;
 		if(b==null){
-			address=refseqAddress;//default
+			address=refseqAddress();//default
 		}else if(b.equalsIgnoreCase("nt")){
-			address=ntAddress;
+			address=ntAddress();
 		}else if(b.equalsIgnoreCase("refseq")){
-			address=refseqAddress;
+			address=refseqAddress();
 		}else if(b.equalsIgnoreCase("silva") || b.equalsIgnoreCase("ribo")){
-			address=silvaAddress;
+			address=silvaAddress();
 		}else if(b.equalsIgnoreCase("img")){
-			address=imgAddress;
+			address=imgAddress();
 		}else if(b.equalsIgnoreCase("refseqprot") || b.equalsIgnoreCase("prokprot") 
 				|| b.equalsIgnoreCase("protein") || b.equalsIgnoreCase("protien") || b.equalsIgnoreCase("prot")){
-			address=prokProtAddress;
+			address=prokProtAddress();
 		}else if(b.equalsIgnoreCase("refseqmito") || b.equalsIgnoreCase("mito")){
-			address=mitoAddress;
+			address=mitoAddress();
 		}else if(b.equalsIgnoreCase("refseqfungi") || b.equalsIgnoreCase("fungi")){
-			address=fungiAddress;
+			address=fungiAddress();
 		}
 		return address;
 	}
-
+	
 	public int SEND_BUFFER_MAX_BYTES=8000000;
 	public int SEND_BUFFER_MAX_SKETCHES=400;
 	private static final int MAX_ALLOWED_SKETCHES=100000;
@@ -654,35 +646,13 @@ public class SendSketch extends SketchObject {
 	/** Don't print caught exceptions */
 	public static boolean suppressErrors=false;
 	
-	public static void switchDefaultAddresses(boolean useAlternate){
-		if(useAlternate){
-			ntAddress="https://nt-sketch-2.jgi.doe.gov/sketch";
-			refseqAddress="https://refseq-sketch-2.jgi.doe.gov/sketch";
-			silvaAddress="https://ribo-sketch-2.jgi.doe.gov/sketch";
-			imgAddress="https://img-sketch-2.jgi.doe.gov/sketch";
-			nrAddress="https://nr-sketch-2.jgi.doe.gov/sketch";
-			prokProtAddress="https://protein-sketch-2.jgi.doe.gov/sketch";
-			mitoAddress="https://mito-sketch-2.jgi.doe.gov/sketch";
-			fungiAddress="https://fungi-sketch-2.jgi.doe.gov/sketch";
-		}else{
-			ntAddress="https://nt-sketch.jgi.doe.gov/sketch";
-			refseqAddress="https://refseq-sketch.jgi.doe.gov/sketch";
-			silvaAddress="https://ribo-sketch.jgi.doe.gov/sketch";
-			imgAddress="https://img-sketch.jgi.doe.gov/sketch";
-			nrAddress="https://nr-sketch.jgi.doe.gov/sketch";
-			prokProtAddress="https://protein-sketch.jgi.doe.gov/sketch";
-			mitoAddress="https://mito-sketch.jgi.doe.gov/sketch";
-			fungiAddress="https://fungi-sketch.jgi.doe.gov/sketch";
-		}
-	}
-	
-	private static String ntAddress="https://nt-sketch.jgi.doe.gov/sketch"; //gpweb34 port 3071
-	private static String refseqAddress="https://refseq-sketch.jgi.doe.gov/sketch"; //gpweb34 port 3072
-	private static String silvaAddress="https://ribo-sketch.jgi.doe.gov/sketch"; //gpweb34 port 3073
-	private static String imgAddress="https://img-sketch.jgi.doe.gov/sketch";
-	private static String nrAddress="https://nr-sketch.jgi.doe.gov/sketch";
-	private static String prokProtAddress="https://protein-sketch.jgi.doe.gov/sketch";//http://gpweb35.nersc.gov:3074/sketch/
-	private static String mitoAddress="https://mito-sketch.jgi.doe.gov/sketch";
-	private static String fungiAddress="https://fungi-sketch.jgi.doe.gov/sketch";
+	private static String ntAddress(){return Shared.ntSketchServer()+"sketch";}
+	private static String refseqAddress(){return Shared.refseqSketchServer()+"sketch";}
+	private static String silvaAddress(){return Shared.riboSketchServer()+"sketch";}
+	private static String imgAddress(){return null;}//Shared.SketchServer()+"sketch";
+	private static String nrAddress(){return null;}//Shared.SketchServer()+"sketch";
+	private static String prokProtAddress(){return Shared.proteinSketchServer()+"sketch";}
+	private static String mitoAddress(){return null;}//Shared.SketchServer()+"sketch";
+	private static String fungiAddress(){return null;}//Shared.SketchServer()+"sketch";
 	
 }
