@@ -44,12 +44,17 @@ public class Sketch extends SketchObject implements Comparable<Sketch>, Cloneabl
 	}
 	
 	//Array should already be hashed, sorted, unique, subtracted from Long.MAX_VALUE, then reversed.
-	public Sketch(long[] keys_, int[] keyCounts_, long[] baseCounts_, ArrayList<String> meta_){
-		this(keys_, keyCounts_, baseCounts_, -1, -1, -1, -1, -1, -1, null, null, null, meta_);
+	public Sketch(long[] keys_, int[] keyCounts_, long[] baseCounts_, byte[] r16S_, byte[] r18S_, ArrayList<String> meta_){
+		this(keys_, keyCounts_, baseCounts_, r16S_, r18S_, -1, -1, -1, -1, -1, -1, null, null, null, meta_);
 	}
 	
 	public Sketch(SketchHeap heap, boolean clearFname, boolean keepCounts, ArrayList<String> meta_){
-		this(heap.toSketchArray(), null, heap.baseCounts(false), (int)heap.taxID, heap.imgID, heap.genomeSizeBases, heap.genomeSizeKmers, heap.genomeSequences,
+		this(heap, clearFname, keepCounts, meta_, -1);
+	}
+	
+	public Sketch(SketchHeap heap, boolean clearFname, boolean keepCounts, ArrayList<String> meta_, int minKeyOccuranceCount){
+		this(heap.toSketchArray_minCount(minKeyOccuranceCount), null, heap.baseCounts(false), heap.r16S(), heap.r18S(), (int)heap.taxID, heap.imgID,
+				heap.genomeSizeBases, heap.genomeSizeKmers, heap.genomeSequences,
 				heap.probCorrect(), heap.taxName(), heap.name0(), heap.fname(), meta_);
 		assert(keyCounts==null);
 		if(!heap.setMode && keepCounts){
@@ -68,13 +73,16 @@ public class Sketch extends SketchObject implements Comparable<Sketch>, Cloneabl
 //		assert(false) : (counts==null)+", "+heap.setMode+", "+keepCounts;
 	}
 
-	public Sketch(long[] keys_, int[] keyCounts_, long[] baseCounts_, int taxID_, long imgID_, long gSizeBases_, long gSizeKmers_, long gSequences_, double probCorrect_,
-			String taxName_, String name0_, String fname_, ArrayList<String> meta_){
+	public Sketch(long[] keys_, int[] keyCounts_, long[] baseCounts_, byte[] r16S_, byte[] r18S_, int taxID_, long imgID_, long gSizeBases_, 
+			long gSizeKmers_, long gSequences_, double probCorrect_, String taxName_, String name0_, String fname_, ArrayList<String> meta_){
 //		Exception e=new Exception(baseCounts_+", "+Arrays.toString(baseCounts_));
 //		e.printStackTrace();
 		keys=keys_;
 		keyCounts=keyCounts_;
+//		fixKeyCounts();
 		baseCounts=baseCounts_;
+		r16S=r16S_;
+		r18S=r18S_;
 		assert(keyCounts==null || keys==null || keyCounts.length==keys.length) : (keys==null ? "null" : keys.length)+", "+(keyCounts==null ? "null" : keyCounts.length);
 		taxID=taxID_;
 		imgID=imgID_;
@@ -88,6 +96,7 @@ public class Sketch extends SketchObject implements Comparable<Sketch>, Cloneabl
 		taxName=fix(taxName_);
 		name0=fix(name0_);
 		fname=fix(fname_);
+		if(fname!=null && (fname.startsWith("stdin.") || fname.endsWith(".sketch") || fname.endsWith(".sketch.gz"))){fname=null;}
 		
 //		if(k2>0){
 //			if(useToValue2) {
@@ -112,6 +121,19 @@ public class Sketch extends SketchObject implements Comparable<Sketch>, Cloneabl
 		}
 	}
 	
+	void loadSSU(){
+		if(taxID>0 && taxID<minFakeID){
+			if(r16S==null && SSUMap.r16SMap!=null){
+				r16S=SSUMap.r16SMap.get(taxID);
+			}
+			if(r18S==null && SSUMap.r18SMap!=null){
+				r18S=SSUMap.r18SMap.get(taxID);
+			}
+		}
+//		new Exception().printStackTrace();
+//		System.err.println(taxID+", "+ssu+", "+SSUMap.ssuMap.size());
+	}
+	
 	void addMeta(String s){
 		s=fixMeta(s);
 		if(s==null){return;}
@@ -134,6 +156,7 @@ public class Sketch extends SketchObject implements Comparable<Sketch>, Cloneabl
 	/*--------------------------------------------------------------*/
 	
 	public void add(Sketch other, int maxlen){
+		assert(keyCounts==null && other.keyCounts==null);
 		final long[] a=keys;
 		final long[] b=other.keys;
 		if(maxlen<1){
@@ -170,7 +193,51 @@ public class Sketch extends SketchObject implements Comparable<Sketch>, Cloneabl
 	public void resize(int newSize) {
 		if(newSize>=length()){return;}
 		keys=Arrays.copyOf(keys, newSize);
-		if(keyCounts!=null){keyCounts=Arrays.copyOf(keyCounts, newSize);}
+//		fixKeyCounts(newSize);
+		if(keyCounts!=null){
+			for(int i=0; i<newSize; i++)
+			keyCounts=Arrays.copyOf(keyCounts, newSize);
+		}
+	}
+	
+	int fixKeyCounts(){
+		return fixKeyCounts(keyCounts==null ? maxCount : keyCounts.length);
+	}
+	
+	int fixKeyCounts(int maxLen){
+		int max=0;
+		if(keyCounts!=null){
+			for(int i=0; i<maxLen && max<2; i++){
+				max=Tools.max(keyCounts[i], max);
+			}
+		}
+		if(max<2){keyCounts=null;}
+		maxCount=Tools.max((maxCount>0 ? 1 : 0), max);
+		return maxCount;
+	}
+
+	public int applyBlacklist() {
+		assert(blacklist!=null);
+		LongList keylist=new LongList(keys.length);
+		IntList countlist=(keyCounts==null ? null : new IntList(keys.length));
+		int removed=0;
+		for(int i=0; i<keys.length; i++){
+			long key=keys[i];
+			if(!Blacklist.contains(Long.MAX_VALUE-key)){
+				keylist.add(key);
+				if(keyCounts!=null){countlist.add(keyCounts[i]);}
+			}else{
+				removed++;
+			}
+		}
+		if(keylist.size()!=keys.length){
+			keys=keylist.toArray();
+			if(keyCounts!=null){keyCounts=countlist.toArray();}
+		}
+//		if(removed>0){
+//			fixKeyCounts();
+//		}
+		return removed;
 	}
 	
 	/*--------------------------------------------------------------*/
@@ -213,11 +280,11 @@ public class Sketch extends SketchObject implements Comparable<Sketch>, Cloneabl
 		}
 		if(matches<1){return null;}
 			
-		return new Sketch(ll.toArray(), il.size>0 ? il.toArray() : null, null, null);
+		return new Sketch(ll.toArray(), il.size>0 ? il.toArray() : null, null, null, null, null);
 	}
 	
 	public static final Sketch union(Sketch sa, Sketch sb){
-		Sketch shared=union(sa.keys, sb.keys, sa.keyCounts, sb.keyCounts, sa.baseCounts, sb.baseCounts);
+		Sketch shared=union(sa.keys, sb.keys, sa.keyCounts, sb.keyCounts, sa.baseCounts, sb.baseCounts, sa.r16S, sb.r16S, sa.r18S, sb.r18S);
 		if(shared!=null){
 			shared.taxID=sa.taxID;
 			shared.taxName=sa.taxName;
@@ -230,10 +297,13 @@ public class Sketch extends SketchObject implements Comparable<Sketch>, Cloneabl
 		return shared;
 	}
 	
-	public static final Sketch union(long[] a, long[] b, int[] aCounts, int[] bCounts, long[] aBaseCounts, long[] bBaseCounts){
+	public static final Sketch union(long[] a, long[] b, int[] aCounts, int[] bCounts, long[] aBaseCounts, long[] bBaseCounts, 
+			byte[] a16S, byte[] b16S, byte[] a18S, byte[] b18S){
 		int i=0, j=0, matches=0;
 		LongList ll=new LongList();
 		IntList il=(aCounts==null || bCounts==null ? null : new IntList());
+		byte[] r16S=(b16S==null ? a16S : a16S==null ? b16S : a16S.length>b16S.length ? a16S : b16S);
+		byte[] r18S=(b18S==null ? a18S : a18S==null ? b18S : a18S.length>b18S.length ? a18S : b18S);
 		long[] baseCounts=(aBaseCounts==null || bBaseCounts==null ? null : new long[aBaseCounts.length]);
 		if(baseCounts!=null){
 			for(int k=0; k<baseCounts.length; k++) {baseCounts[k]+=(aBaseCounts[k]+bBaseCounts[k]);}
@@ -264,7 +334,7 @@ public class Sketch extends SketchObject implements Comparable<Sketch>, Cloneabl
 		}
 		if(matches<1){return null;}
 			
-		return new Sketch(ll.toArray(), il.size>0 ? il.toArray() : null, baseCounts, null);
+		return new Sketch(ll.toArray(), il.size>0 ? il.toArray() : null, baseCounts, r16S, r18S, null);
 	}
 	
 	/*--------------------------------------------------------------*/
@@ -578,6 +648,7 @@ public class Sketch extends SketchObject implements Comparable<Sketch>, Cloneabl
 		bb.append("\tK:").append(k);
 		if(k2!=0){bb.append(",").append(k2);}
 		if(HASH_VERSION>1){bb.append("\tH:").append(HASH_VERSION);}
+//		if(maxCount>0){bb.append("\tMC:").append(maxCount);}
 		
 		if(genomeSizeBases>0){bb.append("\tGS:").append(genomeSizeBases);}
 		if(genomeSizeKmers>0){bb.append("\tGK:").append(genomeSizeKmers);}
@@ -600,6 +671,21 @@ public class Sketch extends SketchObject implements Comparable<Sketch>, Cloneabl
 				bb.append("\tMT_").append(s);
 			}
 		}
+		
+		if(r16S!=null){
+			bb.append("\t16S:").append(r16S.length);
+		}
+		if(r18S!=null){
+			bb.append("\t18S:").append(r18S.length);
+		}
+		if(r16S!=null){
+			bb.append('\n');
+			bb.append("#16S:").append(r16S);
+		}
+		if(r18S!=null){
+			bb.append('\n');
+			bb.append("#18S:").append(r18S);
+		}
 		return bb;
 	}
 	
@@ -944,7 +1030,7 @@ public class Sketch extends SketchObject implements Comparable<Sketch>, Cloneabl
 	public void setName0(String s){name0=s;}
 	public void setFname(String s){
 //		assert(!s.endsWith("sketch")) : s; //123
-		fname=s;
+		fname=(s==null ? s : ReadWrite.stripPath(s));
 	}
 	
 //	public float k1Fraction(){return k1Count/(float)array.length;}
@@ -972,6 +1058,35 @@ public class Sketch extends SketchObject implements Comparable<Sketch>, Cloneabl
 		return list;
 	}
 	
+//	static boolean warned=false;
+	public void addSSU(){
+		if(useSSUMapOnly){r16S=r18S=null;}
+		if(taxID<1 || taxID>=minFakeID){return;}
+		if(!SSUMap.hasMap()){return;}
+		if(taxtree==null/* && !warned*/){
+//			warned=true;
+//			System.err.println("*** Warning - no taxtree loaded. A taxtree is recommended when adding SSUs. ***");
+			assert(false) : "Please set a path to the taxtree when adding SSUs.";
+		}
+		final boolean euk=taxtree!=null && (preferSSUMapForEuks || useSSUMapOnlyForEuks || ban16SForEuks) ? taxtree.isEukaryote(taxID) : false;
+		final boolean prok=taxtree!=null && ban18SForProks ? taxtree.isProkaryote(taxID) : false;
+		
+		final boolean mapOnly=(useSSUMapOnly || (useSSUMapOnlyForEuks && euk));
+		final boolean preferMap=(mapOnly || preferSSUMap || (preferSSUMapForEuks && euk));
+		if(mapOnly){r16S=r18S=null;}
+		
+		if(SSUMap.r16SMap!=null && (r16S==null || preferMap)){
+			byte[] x=SSUMap.r16SMap.get(taxID);
+			if(x!=null){r16S=x;}
+		}
+		if(SSUMap.r18SMap!=null && (r18S==null || preferMap)){
+			byte[] x=SSUMap.r18SMap.get(taxID);
+			if(x!=null){r18S=x;}
+		}
+		if(ban18SForProks && prok){r18S=null;}
+		if(ban16SForEuks && euk){r16S=null;}
+	}
+	
 	/*--------------------------------------------------------------*/
 	/*----------------            BitSet            ----------------*/
 	/*--------------------------------------------------------------*/
@@ -989,7 +1104,6 @@ public class Sketch extends SketchObject implements Comparable<Sketch>, Cloneabl
 	
 	public AbstractBitSet compareBitSet(){return compareBitSet;}
 	
-	
 	public AbstractBitSet indexBitSet(){return indexBitSet;}
 	
 	public void mergeBitSets(){
@@ -1002,6 +1116,15 @@ public class Sketch extends SketchObject implements Comparable<Sketch>, Cloneabl
 	}
 	
 	public boolean merged(){return mergedBitSets;}
+
+	public int r16SLen(){return r16S==null ? 0 : r16S.length;}
+	public int r18SLen(){return r18S==null ? 0 : r18S.length;}
+	public byte[] r16S(){return r16S;}
+	public byte[] r18S(){return r18S;}
+	public boolean hasSSU(){return r16S!=null || r18S!=null;}
+	public boolean sharesSSU(Sketch b){
+		return (r16S!=null && b.r16S!=null) || (r18S!=null && b.r18S!=null);
+	}
 	
 	/*--------------------------------------------------------------*/
 	/*----------------            Fields            ----------------*/
@@ -1013,7 +1136,10 @@ public class Sketch extends SketchObject implements Comparable<Sketch>, Cloneabl
 	int[] keyCounts;
 	/** Stores base (ACGTN) counts */
 	final long[] baseCounts;
+	private byte[] r16S;
+	private byte[] r18S;
 	public int taxID;
+	public int maxCount;
 	int sketchID;//Based on loading order
 	public final long genomeSequences;
 	public final long genomeSizeBases;
