@@ -53,7 +53,8 @@ public class SketchMaker extends SketchObject {
 		//Start a timer immediately upon code entrance.
 		Timer t=new Timer();
 		
-		if(parseMode(args)==PER_FILE){
+		final int mode=parseMode(args);
+		if(mode==PER_FILE/* || mode==ONE_SKETCH || mode==PER_HEADER*/ || mode==PER_SEQUENCE){//ONE_SKETCH does not work for multiple input files.
 			recallCompareSketch(args);
 			return;
 		}
@@ -171,11 +172,11 @@ public class SketchMaker extends SketchObject {
 				tossJunk=Tools.parseBoolean(b);
 			}
 			
-			else if(a.equals("silva")){
-				TaxTree.SILVA_MODE=Tools.parseBoolean(b);
-			}
+//			else if(a.equals("silva")){//Handled by parser
+//				TaxTree.SILVA_MODE=Tools.parseBoolean(b);
+//			}
 			
-			else if(a.equals("taxlevel") || a.equals("level")){
+			else if(a.equals("taxlevel") || a.equals("tl") || a.equals("level") || a.equals("lv")){
 				taxLevel=TaxTree.parseLevel(b);
 			}
 			
@@ -267,12 +268,12 @@ public class SketchMaker extends SketchObject {
 //		}
 		
 		//Ensure input files can be read
-		if(!Tools.testInputFiles(false, true, in1, in2, taxTreeFile, giTableFile, imgFile)){
+		if(!Tools.testInputFiles(false, true, in1, in2, taxTreeFile, giTableFile, imgFile, SSUMap.r16SFile, SSUMap.r18SFile)){
 			throw new RuntimeException("\nCan't read some input files.\n");  
 		}
 		
 		//Ensure that no file was specified multiple times
-		if(!Tools.testForDuplicateFiles(true, in1, in2, out1, taxTreeFile, giTableFile, imgFile)){
+		if(!Tools.testForDuplicateFiles(true, in1, in2, out1, taxTreeFile, giTableFile, imgFile, SSUMap.r16SFile, SSUMap.r18SFile)){
 			throw new RuntimeException("\nSome file names were specified multiple times.\n");
 		}
 		
@@ -283,7 +284,7 @@ public class SketchMaker extends SketchObject {
 //		assert(false) :  defaultParams.trackCounts();
 		tool=new SketchTool(targetSketchSize, defaultParams);
 		
-		if(taxTreeFile!=null){setTaxtree(taxTreeFile);}
+		if(taxTreeFile!=null){setTaxtree(taxTreeFile, outstream);}
 		
 		if(giTableFile!=null){
 			loadGiToTaxid();
@@ -297,6 +298,7 @@ public class SketchMaker extends SketchObject {
 		if(imgFile!=null){
 			TaxTree.loadIMG(imgFile, true, outstream);
 		}
+		SSUMap.load(outstream);
 		
 		if(prefilter){
 			if(mode==PER_TAXA){sizeList=sizeList(); sizeMap=null;}
@@ -358,7 +360,10 @@ public class SketchMaker extends SketchObject {
 	//Makes a list of genome sizes (bases, not kmers) per taxa.
 	private LongList sizeList(){
 		Timer t=new Timer();
-		t.start("Making prefilter.");
+		Shared.GC_BEFORE_PRINT_MEMORY=true;
+		t.start("Making taxa prefilter.");
+		//For some reason this function is very slow, using only ~95% java and ~95% bgzip CPU.
+		//But that's about the same speed as reformat...  BBDuk is only a little faster.
 		
 //		assert(false) : ReadWrite.USE_PIGZ+", "+ReadWrite.USE_UNPIGZ+", "+ReadWrite.MAX_ZIP_THREADS+", "+Shared.threads();
 		
@@ -368,7 +373,7 @@ public class SketchMaker extends SketchObject {
 		final ConcurrentReadInputStream cris;
 		{
 			cris=ConcurrentReadInputStream.getReadInputStream(maxReads, true, ffin1, ffin2, null, null);
-			if(defaultParams.samplerate!=1){cris.setSampleRate(defaultParams.samplerate, sampleseed);}
+//			if(defaultParams.samplerate!=1){cris.setSampleRate(defaultParams.samplerate, sampleseed);} //Why would I ever want this here?
 			cris.start(); //Start the stream
 			if(verbose){outstream.println("Started cris");}
 		}
@@ -434,7 +439,7 @@ public class SketchMaker extends SketchObject {
 	//Makes a list of genome sizes (bases, not kmers) per img.
 	private HashMap<Long, Long> sizeMap(){
 		Timer t=new Timer();
-		t.start("Making prefilter.");
+		t.start("Making img prefilter.");
 		
 //		assert(false) : ReadWrite.USE_PIGZ+", "+ReadWrite.USE_UNPIGZ+", "+ReadWrite.MAX_ZIP_THREADS+", "+Shared.threads();
 		
@@ -580,6 +585,7 @@ public class SketchMaker extends SketchObject {
 		Shared.printMemory();
 
 		if(ffout!=null && ffout.length>0){
+			sketch.addSSU();
 			SketchTool.write(sketch, ffout[0]);
 			sketchesWritten+=1;
 		}
@@ -593,7 +599,7 @@ public class SketchMaker extends SketchObject {
 		Timer t=new Timer();
 		
 		//Determine how many threads may be used
-		final int threads=Tools.mid(1, Shared.threads(), 12);
+		final int threads=Tools.mid(1, Shared.threads(), 14);//Probably capped for memory reasons.  Rarely hits 1200% anyway (was 12, bumped to 14).
 		
 		//Fill a list with ProcessThreads
 		ArrayList<ProcessThread> alpt=new ArrayList<ProcessThread>(threads);
@@ -688,7 +694,10 @@ public class SketchMaker extends SketchObject {
 				if(outName0!=null){sketch.setName0(outName0);}
 				if(outSpid>=0){sketch.spid=outSpid;}
 				if(outImgID>=0){sketch.imgID=outImgID;}
-				if(ffout!=null && ffout.length>0){SketchTool.write(sketch, ffout[0]);}
+				if(ffout!=null && ffout.length>0){
+					sketch.addSSU();
+					SketchTool.write(sketch, ffout[0]);
+				}
 				sketchesMade++;
 				sketchesWritten++;
 			}
@@ -782,6 +791,7 @@ public class SketchMaker extends SketchObject {
 					if(outName0!=null && sketch.name0()==null){sketch.setName0(outName0);}
 					if(outSpid>=0 && sketch.spid<0){sketch.spid=outSpid;}
 					if(outImgID>=0 && sketch.imgID<0){sketch.imgID=outImgID;}
+					sketch.addSSU();
 					SketchTool.write(sketch, tsw[tnum], bb);
 					sketchesWrittenT++;
 				}
@@ -935,7 +945,7 @@ public class SketchMaker extends SketchObject {
 			{
 				int taxID=-1;
 				TaxNode tn=null;
-				if(taxtree!=null && (mode==PER_TAXA || mode==PER_IMG || mode==PER_SEQUENCE || ((mode==ONE_SKETCH || mode==PER_HEADER) && smm.heap.taxID<0))){
+				if(taxtree!=null && (mode==PER_TAXA || mode==PER_IMG || mode==PER_SEQUENCE || ((mode==ONE_SKETCH/* || mode==PER_HEADER*/) && smm.heap.taxID<0))){
 					if(mode==PER_IMG){
 						imgID=ImgRecord2.parseImgId(rid, true);
 						tn=taxtree.imgToTaxNode(imgID);
@@ -1072,7 +1082,7 @@ public class SketchMaker extends SketchObject {
 					//assert(!localMap.containsKey((long)taxID) || localMap.get((long)taxID)==smm.heap) : taxID; //123
 					manageHeap_perTaxa(taxID, imgID, unitSizeBases);
 					if(localMap.size()>20){dumpLocalMap();}
-				}else if(mode==ONE_SKETCH || mode==PER_HEADER){
+				}else if(mode==ONE_SKETCH/* || mode==PER_HEADER*/){
 					//do nothing
 				}else{
 					assert(false) : mode;
@@ -1104,8 +1114,14 @@ public class SketchMaker extends SketchObject {
 			final boolean hasSize=(known && (sizeList!=null || sizeMap!=null));
 			boolean finished=(unknown || (hasSize && smm.heap.genomeSizeBases>=unitSizeBases));
 			
+			//For some reason, this assertion fired for a single taxID (52271) halfway through sketching RefSeq.
+			//Likely a transient hardware error.
 			assert(!finished || smm.heap.genomeSizeBases==unitSizeBases) : finished+", "+unknown+", "+hasSize+", "+(sizeList==null)+"\n"
 					+taxID+", "+unitSizeBases+", "+smm.heap.genomeSizeBases+", "+smm.heap.genomeSizeKmers;
+//			if(finished && smm.heap.genomeSizeBases!=unitSizeBases){
+//				System.err.println("Warning: tid="+taxID+", finished="+finished+", known="+known+
+//						", smm.heap.genomeSizeBases="+smm.heap.genomeSizeBases+", unitSizeBases="+unitSizeBases);
+//			}
 			
 			smm.heap.taxID=taxID;
 			smm.heap.imgID=imgID;
@@ -1272,6 +1288,7 @@ public class SketchMaker extends SketchObject {
 				}
 				if(tsw!=null){
 					final int choice=(sketch.hashCode()&Integer.MAX_VALUE)%files;
+					sketch.addSSU();
 					synchronized(tsw[choice]){
 						SketchTool.write(sketch, tsw[choice], bb);
 						sketchesWrittenT++;
@@ -1386,6 +1403,7 @@ public class SketchMaker extends SketchObject {
 	private boolean prefilter=false;
 	private boolean tossJunk=true;
 	boolean bestEffort=true;
+//	private final HashMap<Integer, byte[]> ssuMap=null;
 	
 	private final AtomicInteger nextUnknown=new AtomicInteger(minFakeID);
 
