@@ -1,18 +1,23 @@
 package sketch;
 
+import java.io.PrintStream;
 import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.Collections;
 import java.util.Random;
 
+import aligner.SingleStateAlignerFlat2;
+import aligner.SingleStateAlignerFlatFloat;
 import dna.AminoAcid;
 import dna.Data;
 import prok.GeneCaller;
 import prok.GeneModel;
 import prok.GeneModelParser;
+import prok.ProkObject;
 import shared.Shared;
 import shared.Timer;
 import shared.Tools;
+import stream.Read;
 import structures.EntropyTracker;
 import structures.IntList3;
 import tax.TaxTree;
@@ -58,11 +63,19 @@ public class SketchObject {
 			minFakeID=Integer.parseInt(b);
 		}else if(a.equalsIgnoreCase("hashNames")){
 			hashNames=Tools.parseBoolean(b);
+		}else if(a.equalsIgnoreCase("preferSSUMap")){
+			preferSSUMap=Tools.parseBoolean(b);
+		}else if(a.equalsIgnoreCase("preferSSUMapForEuks") || a.equalsIgnoreCase("preferSSUMapEuks")){
+			preferSSUMapForEuks=Tools.parseBoolean(b);
+		}else if(a.equalsIgnoreCase("useSSUMapOnly")){
+			useSSUMapOnly=Tools.parseBoolean(b);
+		}else if(a.equalsIgnoreCase("useSSUMapOnlyForEuks") || a.equalsIgnoreCase("SSUMapOnlyEuks")){
+			useSSUMapOnlyForEuks=Tools.parseBoolean(b);
 		}else if(a.equalsIgnoreCase("skipNonCanonical")){
 			skipNonCanonical=Tools.parseBoolean(b);
 		}else if(a.equalsIgnoreCase("useSizeEstimateInScore") || a.equalsIgnoreCase("useSizeEstimate")){
 			useSizeEstimate=Tools.parseBoolean(b);
-		}else if(a.equalsIgnoreCase("blacklist")){
+		}else if(a.equalsIgnoreCase("blacklist") || a.equalsIgnoreCase("bl")){
 			blacklist=b;
 		}else if(a.equalsIgnoreCase("whitelist") || a.equalsIgnoreCase("usewhitelist")){
 			useWhitelist=Tools.parseBoolean(b);
@@ -76,13 +89,16 @@ public class SketchObject {
 			if("auto".equalsIgnoreCase(b)){
 				AUTOSIZE=true;
 				AUTOSIZE_LINEAR=false;
+				SET_AUTOSIZE=true;
 			}else if("linear".equalsIgnoreCase(b)){
 				AUTOSIZE=false;
 				AUTOSIZE_LINEAR=true;
+				SET_AUTOSIZE=true;
 			}else{
 				AUTOSIZE=false;
 				AUTOSIZE_LINEAR=false;
 				targetSketchSize=Tools.parseIntKMG(b);
+				SET_TARGET_SIZE=true;
 			}
 		}else if(a.equalsIgnoreCase("maxfraction") || a.equalsIgnoreCase("maxgenomefraction") || a.equalsIgnoreCase("mgf")){
 			maxGenomeFraction=Float.parseFloat(b);
@@ -93,18 +109,22 @@ public class SketchObject {
 		}else if(a.equalsIgnoreCase("autosize")){
 			AUTOSIZE=Tools.parseBoolean(b);
 			if(AUTOSIZE){AUTOSIZE_LINEAR=false;}
+			SET_AUTOSIZE=true;
 		}else if(a.equalsIgnoreCase("autosizefactor") || a.equalsIgnoreCase("autosizefraction") || a.equalsIgnoreCase("autosizemult") || a.equalsIgnoreCase("sizemult")){
 			AUTOSIZE_FACTOR=Float.parseFloat(b);
 			SET_AUTOSIZE_FACTOR=true;
 			AUTOSIZE_LINEAR=false;
+			SET_AUTOSIZE=true;
 		}else if(a.equalsIgnoreCase("linear") || a.equalsIgnoreCase("lineardensity") || a.equalsIgnoreCase("density")){
 			if(Tools.isNumeric(b)){
 				AUTOSIZE_LINEAR_DENSITY=Double.parseDouble(b);
 				AUTOSIZE_LINEAR=true;
 				AUTOSIZE=false;
+				SET_AUTOSIZE=true;
 			}else{
 				AUTOSIZE_LINEAR=Tools.parseBoolean(b);
 				if(AUTOSIZE_LINEAR){AUTOSIZE=false;}
+				SET_AUTOSIZE=true;
 			}
 		}else if(a.equalsIgnoreCase("maxGenomeFractionSmall")){
 			maxGenomeFractionSmall=Float.parseFloat(b);
@@ -141,6 +161,10 @@ public class SketchObject {
 		
 		else if(a.equalsIgnoreCase("killok")){
 			KILL_OK=Tools.parseBoolean(b);
+		}else if(a.equalsIgnoreCase("ssa") || a.equalsIgnoreCase("usessa")){
+			useSSA=Tools.parseBoolean(b);
+		}else if(a.equalsIgnoreCase("ssa3") || a.equalsIgnoreCase("usessa3")){
+			useSSA3=Tools.parseBoolean(b);
 		}else if(a.equalsIgnoreCase("exactani")){
 			EXACT_ANI=Tools.parseBoolean(b);
 		}else if(a.equalsIgnoreCase("translate") || a.equalsIgnoreCase("callgenes")){
@@ -152,9 +176,11 @@ public class SketchObject {
 			if(sixframes){
 				translate=defaultParams.translate=true;
 			}
+		}else if(a.equalsIgnoreCase("processSSU") || a.equalsIgnoreCase("process16S") || a.equalsIgnoreCase("16S") || a.equalsIgnoreCase("SSU")){
+			processSSU=Tools.parseBoolean(b);
 		}else if(a.equalsIgnoreCase("hashSeed")){
 			hashSeed=Long.parseLong(b);
-			remakeCodes(hashSeed);
+			//remakeCodes(hashSeed); //Happens in postParse
 		}
 		
 		else if(a.equalsIgnoreCase("forceDisableMultithreadedFastq")){
@@ -181,6 +207,16 @@ public class SketchObject {
 		
 		else if(a.equalsIgnoreCase("useToValue2") || a.equalsIgnoreCase("ToValue2")){
 			useToValue2=Tools.parseBoolean(b);
+		}else if(a.equals("ssu") || a.equals("ssufile")){
+			assert(false) : "ssufile is deprecated; please specify 16Sfile and/or 18Sfile independently";
+		}else if(a.equalsIgnoreCase("16Sfile")/* || a.equalsIgnoreCase("ssufile") || a.equalsIgnoreCase("silvafile")*/){
+			SSUMap.r16SFile=b;
+			if("auto".equalsIgnoreCase(SSUMap.r16SFile)){SSUMap.r16SFile=TaxTree.default16SFile();}
+			assert(SSUMap.r16SMap==null);
+		}else if(a.equalsIgnoreCase("18Sfile")){
+			SSUMap.r18SFile=b;
+			if("auto".equalsIgnoreCase(SSUMap.r18SFile)){SSUMap.r18SFile=TaxTree.default18SFile();}
+			assert(SSUMap.r18SMap==null);
 		}
 		
 //		else if(a.equalsIgnoreCase("minHashValue")){
@@ -236,10 +272,10 @@ public class SketchObject {
 				mode_=ONE_SKETCH;
 			}else if(b.equalsIgnoreCase("taxa") || b.equalsIgnoreCase("pertaxa")){
 				mode_=PER_TAXA;
-			}else if(b.equalsIgnoreCase("sequence") || b.equalsIgnoreCase("persequence")){
+			}else if(b.equalsIgnoreCase("sequence") || b.equalsIgnoreCase("persequence") || b.equalsIgnoreCase("header") || b.equalsIgnoreCase("perheader")){
 				mode_=PER_SEQUENCE;
-			}else if(b.equalsIgnoreCase("header") || b.equalsIgnoreCase("perheader")){
-				mode_=PER_HEADER;
+//			}else if(b.equalsIgnoreCase("header") || b.equalsIgnoreCase("perheader")){
+//				mode_=PER_HEADER;
 			}else if(b.equalsIgnoreCase("img")){
 				mode_=PER_IMG;
 			}else if(b.equalsIgnoreCase("perfile") || b.equalsIgnoreCase("file")){
@@ -251,10 +287,10 @@ public class SketchObject {
 			mode_=ONE_SKETCH;
 		}else if(a.equalsIgnoreCase("taxa") || a.equalsIgnoreCase("pertaxa")){
 			mode_=PER_TAXA;
-		}else if(a.equalsIgnoreCase("sequence") || a.equalsIgnoreCase("persequence")){
+		}else if(a.equalsIgnoreCase("sequence") || a.equalsIgnoreCase("persequence") || a.equalsIgnoreCase("header") || a.equalsIgnoreCase("perheader")){
 			mode_=PER_SEQUENCE;
-		}else if(a.equalsIgnoreCase("header") || a.equalsIgnoreCase("perheader")){
-			mode_=PER_HEADER;
+//		}else if(a.equalsIgnoreCase("header") || a.equalsIgnoreCase("perheader")){
+//			mode_=PER_HEADER;
 		}else if(a.equalsIgnoreCase("perfile") || a.equalsIgnoreCase("file")){
 			mode_=PER_FILE;
 		}
@@ -265,7 +301,7 @@ public class SketchObject {
 	/*----------------        Initialization        ----------------*/
 	/*--------------------------------------------------------------*/
 	
-	static synchronized void setTaxtree(String taxTreeFile){
+	static synchronized void setTaxtree(String taxTreeFile, PrintStream outstream){
 		if(taxTreeFile==null){
 			taxtree=null;
 			return;
@@ -275,7 +311,7 @@ public class SketchObject {
 			if(treefile.equals(taxTreeFile)){return;}
 			treefile=taxTreeFile;
 		}
-		taxtree=TaxTree.loadTaxTree(taxTreeFile, System.err, hashNames, false);
+		taxtree=TaxTree.loadTaxTree(taxTreeFile, outstream, hashNames, false);
 	}
 	
 	public static void reset(){
@@ -292,7 +328,7 @@ public class SketchObject {
 		if(postparsed){return;}
 		postparsed=true;
 		IntList3.defaultMode=IntList3.UNIQUE; //Not really safe, if Seal uses Sketch...
-
+		
 		if(defaultParams.amino()){amino=true;}
 		if(amino){Shared.AMINO_IN=true;}
 		
@@ -350,19 +386,21 @@ public class SketchObject {
 		
 //		hasher=k2<1 ? new Hasher1() : k2submask==0 ? new Hasher2() : new Hasher3();
 		
-		if(translate){
+		if(translate || processSSU){
 			if(pgmFile==null){
 				pgmFile=Data.findPath("?model.pgm");
 			}
 			pgm=GeneModelParser.loadModel(pgmFile);
-			GeneCaller.call16S=false;
+			GeneCaller.call16S=processSSU;
+			GeneCaller.call18S=false;//processSSU;
 			GeneCaller.call23S=false;
 			GeneCaller.call5S=false;
 			GeneCaller.calltRNA=false;
-			GeneCaller.callCDS=true;
+			GeneCaller.callCDS=translate;
 			
-			if(GeneCaller.call16S || GeneCaller.call23S){//Which, currently, is always false.
-				pgm.loadLongKmers();
+			if(GeneCaller.call16S || GeneCaller.call18S || GeneCaller.call23S){
+				ProkObject.loadLongKmers();
+				ProkObject.loadConsensusSequenceFromFile(true, true);
 			}
 		}
 		
@@ -375,6 +413,8 @@ public class SketchObject {
 //				", basesPerCycle="+basesPerCycle+", hashCycles="+hashCycles+", k2mask="+k2mask+", k2submask="+k2submask+", hashCycles2="+hashCycles2+
 //				", codeMax="+codeMax+", codeMax2="+codeMax2);
 //		structures.IntList3.ascending=false;//123 for testing
+		
+		alignerPool=new AlignmentThreadPool(Shared.threads());
 	}
 	
 	/*--------------------------------------------------------------*/
@@ -640,6 +680,61 @@ public class SketchObject {
 	/*----------------      Convenience Methods     ----------------*/
 	/*--------------------------------------------------------------*/
 	
+	public static final float align(byte[] query, byte[] ref){
+		SingleStateAlignerFlat2 ssa=GeneCaller.getSSA();
+		int a=0, b=ref.length-1;
+		int[] max=ssa.fillUnlimited(query, ref, a, b, -9999);
+		if(max==null){return 0;}
+		
+		final int rows=max[0];
+		final int maxCol=max[1];
+		final int maxState=max[2];
+		final float id=ssa.tracebackIdentity(query, ref, a, b, rows, maxCol, maxState, null);
+		return id;
+	}
+	
+	public static final float alignAndMakeMatch(Read r, byte[] ref){
+		byte[] query=r.bases;
+		SingleStateAlignerFlat2 ssa=GeneCaller.getSSA();
+		int a=0, b=ref.length-1;
+		int[] max=ssa.fillUnlimited(query, ref, a, b, -9999);
+		if(max==null){return 0;}
+		
+		final int rows=max[0];
+		final int maxCol=max[1];
+		final int maxState=max[2];
+		byte[] match=ssa.traceback(query, ref, a, b, rows, maxCol, 0);
+		int[] score=ssa.score(query, ref, a, b, rows, maxCol, 0);
+		final float id=Read.identity(match);
+		r.match=match;
+		r.start=score[1];
+		r.stop=score[2];
+		r.setMapped(true);
+		return id;
+	}
+	
+	public static final float alignAndMakeMatch(Read r, byte[] ref, float[] refWeights/*, float[] insWeights, float[] delWeights*/){
+		byte[] query=r.bases;
+		SingleStateAlignerFlatFloat ssa=GeneCaller.getSSAF();
+//		ssa.setWeights(refWeights, insWeights, delWeights);
+		ssa.setWeights(refWeights);
+		int a=0, b=ref.length-1;
+		int[] max=ssa.fillUnlimited(query, ref, a, b, -9999);
+		if(max==null){return 0;}
+		
+		final int rows=max[0];
+		final int maxCol=max[1];
+		final int maxState=max[2];
+		byte[] match=ssa.traceback(query, ref, a, b, rows, maxCol, 0);
+		int[] score=ssa.score(query, ref, a, b, rows, maxCol, 0);
+		final float id=Read.identity(match);
+		r.match=match;
+		r.start=score[1];
+		r.stop=score[2];
+		r.setMapped(true);
+		return id;
+	}
+	
 	static String fixMeta(String s){
 		if(s==null){return null;}
 		int colon=s.indexOf(':');
@@ -767,14 +862,14 @@ public class SketchObject {
 //		Shared.sort(kmers);
 //	}
 	
-	public static void hashArrayToSketchArray(long[] codes){
-		for(int i=0; i<codes.length; i++){
-			long hash=codes[i];
+	public static void hashArrayToSketchArray(long[] keys){
+		for(int i=0; i<keys.length; i++){
+			long hash=keys[i];
 			assert(hash>=minHashValue);
 			hash=Long.MAX_VALUE-hash;
-			codes[i]=hash;
+			keys[i]=hash;
 		}
-		Shared.sort(codes);
+		Shared.sort(keys);
 	}
 	
 	public static final long genomeSizeEstimate(long min, int length) {
@@ -843,7 +938,11 @@ public class SketchObject {
 				return (int)(Tools.min(10000000, Tools.min(maxGenomeFractionSmall, AUTOSIZE_LINEAR_DENSITY)*genomeSizeKmers));
 			}
 		}else{
-			return Tools.min((int)(2+maxGenomeFraction*genomeSizeKmers), maxSketchSize);
+			if(aminoOrTranslate()){//kmers are roughly 3x smaller...
+				return Tools.min((int)(2+3*maxGenomeFraction*genomeSizeKmers), maxSketchSize);
+			}else{
+				return Tools.min((int)(2+maxGenomeFraction*genomeSizeKmers), maxSketchSize);
+			}
 		}
 	}
 	
@@ -918,8 +1017,7 @@ public class SketchObject {
 		hexTable['x']=hexTable['X']=hexTable['-']=hexTable['+']=0;
 	}
 
-	public static final int ONE_SKETCH=1, PER_SEQUENCE=2, PER_TAXA=3, PER_IMG=4, PER_HEADER=5, PER_FILE=6;
-//	public static final int /*ONE_SKETCH=1,*/ PER_SEQUENCE=2, PER_TAXA=3, PER_IMG=4, PER_HEADER=5, PER_FILE=6;
+	public static final int ONE_SKETCH=1, PER_SEQUENCE=2, PER_TAXA=3, PER_IMG=4/*, PER_HEADER=5*/, PER_FILE=6;
 	
 	/*--------------------------------------------------------------*/
 	/*----------------      Default Locations       ----------------*/
@@ -939,14 +1037,36 @@ public class SketchObject {
 		return list;
 	}
 	
-	static final String IMG_PATH="/global/projectb/sandbox/gaag/bbtools/img/current/img#.sketch";
-	static final String NT_PATH="/global/projectb/sandbox/gaag/bbtools/nt/current/taxa#.sketch";
-	static final String NR_PATH="/global/projectb/sandbox/gaag/bbtools/nr/current/taxa#.sketch";
-	static final String REFSEQ_PATH="/global/projectb/sandbox/gaag/bbtools/refseq/current/taxa#.sketch";
-	static final String SILVA_PATH="/global/projectb/sandbox/gaag/bbtools/silva/latest/both_seq#.sketch";
-	static final String PROKPROT_PATH="/global/projectb/sandbox/gaag/bbtools/refseq/current/prot/taxa#.sketch";
-	static final String MITO_PATH="/global/projectb/sandbox/gaag/bbtools/mito2/taxa#.sketch";
-	static final String FUNGI_PATH="/global/projectb/sandbox/gaag/bbtools/mito2/fungi#.sketch";
+	private static final String IMG_PATH="/global/projectb/sandbox/gaag/bbtools/img/current/img#.sketch";
+	private static final String NT_PATH="/global/projectb/sandbox/gaag/bbtools/nt/current/taxa#.sketch";
+	private static final String NR_PATH="/global/projectb/sandbox/gaag/bbtools/nr/current/taxa#.sketch";
+	private static final String REFSEQ_PATH="/global/projectb/sandbox/gaag/bbtools/refseq/current/taxa#.sketch";
+	private static final String REFSEQ_PATH_BIG="/global/projectb/sandbox/gaag/bbtools/refseq/current/big#.sketch";
+	private static final String SILVA_PATH="/global/projectb/sandbox/gaag/bbtools/silva/latest/both_taxa#.sketch";
+	private static final String PROKPROT_PATH="/global/projectb/sandbox/gaag/bbtools/refseq/current/prot/taxa#.sketch";
+	private static final String PROKPROT_PATH_BIG="/global/projectb/sandbox/gaag/bbtools/refseq/current/prot/big#.sketch";
+	private static final String MITO_PATH="/global/projectb/sandbox/gaag/bbtools/mito2/taxa#.sketch";
+	private static final String FUNGI_PATH="/global/projectb/sandbox/gaag/bbtools/mito2/fungi#.sketch";
+
+	private static final String IMG_PATH_AWS=null;
+	private static final String NT_PATH_AWS="/test1/sketch/latest/nt/taxa#.sketch";
+	private static final String NR_PATH_AWS=null;
+	private static final String REFSEQ_PATH_AWS="/test1/sketch/latest/refseq/taxa#.sketch";
+	private static final String SILVA_PATH_AWS="/test1/sketch/latest/ribo/both_taxa#.sketch";
+	private static final String PROKPROT_PATH_AWS="/test1/sketch/latest/protein/taxa#.sketch";
+	private static final String MITO_PATH_AWS=null;
+	private static final String FUNGI_PATH_AWS=null;
+
+	public static final String IMG_PATH(){return Shared.AWS ? IMG_PATH_AWS : IMG_PATH;}
+	public static final String NT_PATH(){return Shared.AWS ? NT_PATH_AWS : NT_PATH;}
+	public static final String NR_PATH(){return Shared.AWS ? NR_PATH_AWS : NR_PATH;}
+	public static final String REFSEQ_PATH(){return Shared.AWS ? REFSEQ_PATH_AWS : REFSEQ_PATH;}
+	public static final String REFSEQ_PATH_BIG(){return REFSEQ_PATH_BIG;}
+	public static final String SILVA_PATH(){return Shared.AWS ? SILVA_PATH_AWS : SILVA_PATH;}
+	public static final String PROKPROT_PATH(){return Shared.AWS ? PROKPROT_PATH_AWS : PROKPROT_PATH;}
+	public static final String PROKPROT_PATH_BIG(){return PROKPROT_PATH_BIG;}
+	public static final String MITO_PATH(){return Shared.AWS ? MITO_PATH_AWS : MITO_PATH;}
+	public static final String FUNGI_PATH(){return Shared.AWS ? FUNGI_PATH_AWS : FUNGI_PATH;}
 	
 	/*--------------------------------------------------------------*/
 	/*----------------           Getters            ----------------*/
@@ -975,6 +1095,8 @@ public class SketchObject {
 	public static boolean amino8=false;
 	public static boolean translate=false;
 	public static boolean sixframes=false;
+	public static boolean processSSU=true;
+	public static int min_SSU_len=1000;
 	public static int HASH_VERSION=2;
 	public static String pgmFile=null;
 	public static GeneModel pgm=null;
@@ -991,8 +1113,8 @@ public class SketchObject {
 	private static int codeMax2;
 	
 	private static long hashSeed=12345;
-	private static long[][] codes=makeCodes(maxCycles, codeIncrement, hashSeed, false);
-	private static long[] codes1D=makeCodes1D(codes);
+	private static long[][] codes;//=makeCodes(maxCycles, codeIncrement, hashSeed, false);
+	private static long[] codes1D;//=makeCodes1D(codes);
 	static boolean useToValue2=true;
 	
 	//These are set in postParse()
@@ -1018,9 +1140,11 @@ public class SketchObject {
 	public static boolean AUTOSIZE=true;
 	public static float AUTOSIZE_FACTOR=1;
 	public static boolean SET_AUTOSIZE_FACTOR=false;
+	public static boolean SET_AUTOSIZE=false;
+	public static boolean SET_TARGET_SIZE=false;
 	public static boolean AUTOSIZE_LINEAR=false;
 	public static double AUTOSIZE_LINEAR_DENSITY=0.001;
-	public static float maxGenomeFraction=0.01f;
+	public static float maxGenomeFraction=0.04f;//Was 0.015, but that is often too sparse
 	public static float maxGenomeFractionSmall=0.10f;
 	public static int smallSketchSize=150;
 	public static boolean makeIndex=true;
@@ -1030,14 +1154,14 @@ public class SketchObject {
 	public static boolean skipCompare=false;
 	public static final int bitSetBits=2; //Needs to be 2 for unique counts.
 
-	private static double keyFraction=0.2;
+	private static double keyFraction=0.16;
 	private static double keyFraction2=keyFraction*1.2;
 	public static long minHashValue=setMinHashValue();
 	public static double keyFraction(){return keyFraction;}
 	public static void setKeyFraction(double d){
 		assert(d>0);
-		keyFraction=Tools.mid(0, d, 0.5);
-		keyFraction2=Tools.mid(0, keyFraction*1.2, 0.5);
+		keyFraction=Tools.mid(0.0001, d, 0.5);
+		keyFraction2=Tools.mid(0.0001, keyFraction*1.2, 0.5);
 	}
 	public static long setMinHashValue(){
 		double mult=1-2*keyFraction;
@@ -1053,6 +1177,13 @@ public class SketchObject {
 	public static boolean allowMultithreadedFastq=false;
 	static boolean forceDisableMultithreadedFastq=false;
 	
+	static boolean preferSSUMap=false;
+	static boolean preferSSUMapForEuks=true;
+	static boolean useSSUMapOnly=false;
+	static boolean useSSUMapOnlyForEuks=false;
+	static boolean ban18SForProks=true;
+	static boolean ban16SForEuks=true;
+	
 	static long sampleseed=-1L;
 	
 	public static TaxTree taxtree=null;
@@ -1062,10 +1193,14 @@ public class SketchObject {
 	private static boolean postparsed=false;
 	public static boolean KILL_OK=false;
 	public static boolean EXACT_ANI=true;
+	public static boolean useSSA=true;
+	public static boolean useSSA3=false;
 	
 	//Needs to be last due to dependencies.
 	public static DisplayParams defaultParams=new DisplayParams();
 	
+	static AlignmentThreadPool alignerPool=null;
+	
 	public static boolean verbose2=false;
 	public static boolean LOADER2=true;
 	
