@@ -12,6 +12,7 @@ import java.util.concurrent.atomic.AtomicLong;
 import shared.Shared;
 import shared.Tools;
 import structures.AbstractBitSet;
+import structures.ByteBuilder;
 import structures.Heap;
 import structures.IntHashMap;
 import tax.TaxNode;
@@ -25,6 +26,8 @@ public class SketchSearcher extends SketchObject {
 
 	public boolean parse(String arg, String a, String b, boolean addFileIfNotFound){
 		
+//		System.err.println("Parsing "+arg+"; ref="+refFiles); //123
+		
 		if(parseSketchFlags(arg, a, b)){
 			//Do nothing
 		}else if(defaultParams.parse(arg, a, b)){
@@ -33,10 +36,11 @@ public class SketchSearcher extends SketchObject {
 			verbose=Tools.parseBoolean(b);
 		}else if(a.equals("ref")){
 			addRefFiles(b);
-		}else if(arg.equalsIgnoreCase("nt") || arg.equalsIgnoreCase("RefSeq") || arg.equalsIgnoreCase("nr")
+		}else if(arg.equalsIgnoreCase("nt") || arg.equalsIgnoreCase("RefSeq") || arg.equalsIgnoreCase("refseqbig") || arg.equalsIgnoreCase("nr")
 				|| arg.equalsIgnoreCase("img") || arg.equalsIgnoreCase("silva") || arg.equalsIgnoreCase("ribo")
 				 || arg.equalsIgnoreCase("mito") || arg.equalsIgnoreCase("fungi") 
-				 || arg.equalsIgnoreCase("prokprot") || arg.equalsIgnoreCase("protein") || arg.equalsIgnoreCase("protien") || a.equalsIgnoreCase("prot")){
+				 || arg.equalsIgnoreCase("prokprot") || arg.equalsIgnoreCase("prokprotbig") || arg.equalsIgnoreCase("protein") || 
+				 arg.equalsIgnoreCase("protien") || a.equalsIgnoreCase("prot")){
 			addRefFiles(arg);
 		}else if(a.equals("threads") || a.equals("sketchthreads") || a.equals("t")){
 			threads=Integer.parseInt(b);
@@ -61,11 +65,13 @@ public class SketchSearcher extends SketchObject {
 		}else{
 			return false;
 		}
+//		System.err.println("Parsed "+arg+"; ref="+refFiles); //123
 		return true;
 	}
 
-	public boolean compare(ArrayList<Sketch> querySketches, StringBuilder sb, DisplayParams params, int maxThreads){
+	public boolean compare(ArrayList<Sketch> querySketches, ByteBuilder sb, DisplayParams params, int maxThreads){
 		assert(params.postParsed);
+		final boolean json=params.json();
 		ConcurrentHashMap<Integer, Comparison> map=new ConcurrentHashMap<Integer, Comparison>();
 		
 		SketchResults[] alca=new SketchResults[querySketches.size()];
@@ -90,7 +96,20 @@ public class SketchSearcher extends SketchObject {
 		for(int i=0; i<alca.length; i++){
 //			Sketch s=sketches.get(i);
 			SketchResults results=alca[i];
+
+			if(json && alca.length>1 && i==0){
+				sb.append('[');
+			}
+
 			sb.append(results.toText(params));
+
+			if(json && alca.length>1){
+				if(i<alca.length-1){
+					sb.append(',');
+				}else{
+					sb.append(']');
+				}
+			}
 		}
 		return success;
 	}
@@ -112,6 +131,7 @@ public class SketchSearcher extends SketchObject {
 		
 		@Override
 		public void run(){
+			if(a.length()<1 || a.length()<params.minHits || (params.requireSSU && !a.hasSSU())){return;}//TODO: Change to 'require16S'
 			assert(a.compareBitSet()==null || buffer.cbs!=null) : (a.compareBitSet()==null)+", "+(buffer.cbs==null); //Unsafe to use a.cbs multithreaded unless atomic
 			final AbstractBitSet cbs=(buffer.cbs==null ? a.compareBitSet() : buffer.cbs);
 			for(int i=pid; i<localRefSketches.size(); i+=incr){
@@ -138,7 +158,7 @@ public class SketchSearcher extends SketchObject {
 	
 	public SketchResults processSketch(Sketch a, CompareBuffer buffer, AtomicInteger fakeID, 
 			ConcurrentHashMap<Integer, Comparison> map, DisplayParams params, int maxThreads){
-		if(a.length()<1 || a.length()<params.minHits){return new SketchResults(a);}
+		if(a.length()<1 || a.length()<params.minHits || (params.requireSSU && !a.hasSSU())){return new SketchResults(a);}
 		//		Timer t=new Timer();
 		//		t.start("Began query.");
 		assert(a.compareBitSet()==null);
@@ -273,7 +293,7 @@ public class SketchSearcher extends SketchObject {
 			long seb=b.genomeSizeEstimate();
 			if(Tools.min(sea, seb)<params.minSizeRatio*Tools.max(sea, seb)){return false;}
 		}
-		Comparison c=compareOneToOne(a, b, buffer, abs, /*taxHits, params.contamLevel(),*/ params.minHits, params.minWKID, params.minANI, null);
+		Comparison c=compareOneToOne(a, b, buffer, abs, /*taxHits, params.contamLevel(),*/ params.minHits, params.minWKID, params.minANI, params.requireSSU, null);
 		if(c==null){return false;}
 		if(c.taxID()<1){c.taxID=fakeID.getAndIncrement();}
 		
@@ -338,10 +358,11 @@ public class SketchSearcher extends SketchObject {
 	
 	//TODO:  Interestingly, the heap never seems to be created by anything...  not sure what it's for.
 	private static Comparison compareOneToOne(final Sketch a, final Sketch b, CompareBuffer buffer, AbstractBitSet abs,
-			int minHits, float minWKID, float minANI, Heap<Comparison> heap){
+			int minHits, float minWKID, float minANI, boolean requireSSU, Heap<Comparison> heap){
 //		assert(heap!=null); //Optional, for testing.
 //		assert(a.refHitCounts!=null);
 		if(a==b && !compareSelf){return null;}
+		if(requireSSU && !a.sharesSSU(b)){return null;}
 		final int matches=a.countMatches(b, buffer, abs, true/*!makeIndex || !AUTOSIZE*/, null, -1);
 		assert(matches==buffer.hits());
 		if(matches<minHits){return null;}
@@ -366,34 +387,40 @@ public class SketchSearcher extends SketchObject {
 	
 	public void addRefFiles(String a){
 		if(a.equalsIgnoreCase("nr")){
-			addRefFiles(NR_PATH);
+			addRefFiles(NR_PATH());
 			if(blacklist==null){blacklist=Blacklist.nrBlacklist();}
 			if(defaultParams.dbName==null){defaultParams.dbName="nr";}
 			if(!setK){k=defaultKAmino; k2=defaultK2Amino;}
 		}else if(a.equalsIgnoreCase("nt")){
-			addRefFiles(NT_PATH);
+			addRefFiles(NT_PATH());
 			if(blacklist==null){blacklist=Blacklist.ntBlacklist();}
 			if(defaultParams.dbName==null){defaultParams.dbName="nt";}
 			if(!setK){k=defaultK; k2=defaultK2;}
 		}else if(a.equalsIgnoreCase("refseq")){
-			addRefFiles(REFSEQ_PATH);
+			addRefFiles(REFSEQ_PATH());
 			if(blacklist==null){blacklist=Blacklist.refseqBlacklist();}
 			if(defaultParams.dbName==null){defaultParams.dbName="RefSeq";}
 			if(!setK){k=defaultK; k2=defaultK2;}
 			if(!SET_AUTOSIZE_FACTOR){AUTOSIZE_FACTOR=2.0f;}
+		}else if(a.equalsIgnoreCase("refseqbig")){
+			addRefFiles(REFSEQ_PATH_BIG());
+			if(blacklist==null){blacklist=Blacklist.refseqBlacklist();}
+			if(defaultParams.dbName==null){defaultParams.dbName="RefSeq";}
+			if(!setK){k=defaultK; k2=defaultK2;}
+			if(!SET_AUTOSIZE_FACTOR){AUTOSIZE_FACTOR=4.5f;}
 		}else if(a.equalsIgnoreCase("silva")){
 //			TaxTree.SILVA_MODE=Tools.parseBoolean(b);
-			addRefFiles(SILVA_PATH);
+			addRefFiles(SILVA_PATH());
 			if(blacklist==null){blacklist=Blacklist.silvaBlacklist();}
 			if(defaultParams.dbName==null){defaultParams.dbName="Silva";}
 			if(!setK){k=defaultK; k2=defaultK2;}
 		}else if(a.equalsIgnoreCase("img")){
-			addRefFiles(IMG_PATH);
+			addRefFiles(IMG_PATH());
 			if(blacklist==null){blacklist=Blacklist.imgBlacklist();}
 			if(defaultParams.dbName==null){defaultParams.dbName="IMG";}
 			if(!setK){k=defaultK; k2=defaultK2;}
 		}else if(a.equalsIgnoreCase("prokprot") || a.equalsIgnoreCase("protein")){
-			addRefFiles(PROKPROT_PATH);
+			addRefFiles(PROKPROT_PATH());
 			if(blacklist==null){blacklist=Blacklist.prokProtBlacklist();}
 			if(defaultParams.dbName==null){defaultParams.dbName="ProkProt";}
 			if(!setK){k=defaultKAmino; k2=defaultK2Amino;}
@@ -402,13 +429,23 @@ public class SketchSearcher extends SketchObject {
 				System.err.println("Setting translate to true because a protein dataset is being used.");
 			}
 			if(!SET_AUTOSIZE_FACTOR){AUTOSIZE_FACTOR=3.0f;}
+		}else if(a.equalsIgnoreCase("prokprotbig") || a.equalsIgnoreCase("proteinbig")){
+			addRefFiles(PROKPROT_PATH_BIG());
+			if(blacklist==null){blacklist=Blacklist.prokProtBlacklist();}
+			if(defaultParams.dbName==null){defaultParams.dbName="ProkProt";}
+			if(!setK){k=defaultKAmino; k2=defaultK2Amino;}
+			if(!amino && !translate) {
+				translate=true;
+				System.err.println("Setting translate to true because a protein dataset is being used.");
+			}
+			if(!SET_AUTOSIZE_FACTOR){AUTOSIZE_FACTOR=7.5f;}
 		}else if(a.equalsIgnoreCase("mito") || a.equalsIgnoreCase("refseqmito")){
-			addRefFiles(MITO_PATH);
+			addRefFiles(MITO_PATH());
 			if(blacklist==null){blacklist=Blacklist.mitoBlacklist();}
 			if(defaultParams.dbName==null){defaultParams.dbName="RefSeqMito";}
 			if(!setK){k=defaultK; k2=defaultK2;}
 		}else if(a.equalsIgnoreCase("fungi") || a.equalsIgnoreCase("refseqfungi")){
-			addRefFiles(FUNGI_PATH);
+			addRefFiles(FUNGI_PATH());
 			if(blacklist==null){blacklist=Blacklist.fungiBlacklist();}
 			if(defaultParams.dbName==null){defaultParams.dbName="RefSeqFungi";}
 			if(!setK){k=defaultK; k2=defaultK2;}
@@ -417,7 +454,7 @@ public class SketchSearcher extends SketchObject {
 		}
 	}
 	
-	private void addFiles(String a, Set<String> list){
+	static void addFiles(String a, Set<String> list){
 		if(a==null){return;}
 		File f=new File(a);
 		assert(!list.contains(a)) : "Duplicate file "+a;
@@ -450,6 +487,7 @@ public class SketchSearcher extends SketchObject {
 	public void loadReferences(int mode_, int minKeyOccuranceCount, float minEntropy, float minProb, byte minQual) {
 		makeTool(minKeyOccuranceCount, false, false);
 		refSketches=tool.loadSketches_MT(mode_, 1f, -1, minEntropy, minProb, minQual, refFiles);
+		assert(refSketches!=null) : refFiles;
 		if(mode_==PER_FILE){
 			Collections.sort(refSketches, SketchIdComparator.comparator);
 		}
