@@ -183,7 +183,7 @@ public final class SketchTool extends SketchObject {
 	public Sketch toSketch(ArrayList<SketchHeap> heaps, boolean allowZeroSizeSketch){
 		if(heaps==null || heaps.isEmpty()){
 			if(allowZeroSizeSketch){
-				return new Sketch(new long[0], null, null, null);
+				return new Sketch(new long[0], null, null, null, null, null);
 			}else{
 				return null;
 			}
@@ -203,7 +203,7 @@ public final class SketchTool extends SketchObject {
 		list.shrinkToUnique();
 		list.reverse();
 		for(int i=0; i<list.size; i++){list.array[i]=Long.MAX_VALUE-list.array[i];}
-		return new Sketch(list.toArray(), null, null, null);
+		return new Sketch(list.toArray(), null, null, null, null, null);
 	}
 	
 	/*--------------------------------------------------------------*/
@@ -257,12 +257,12 @@ public final class SketchTool extends SketchObject {
 //	}
 	
 	public ArrayList<Sketch> loadSketches_MT(DisplayParams params, Collection<String> fnames){
-		return loadSketches_MT(params.mode, params.samplerate, params.reads,
+		return loadSketches_MT(params.mode, params.samplerate, params.maxReads,
 				params.minEntropy, params.minProb, params.minQual, fnames);
 	}
 	
 	public ArrayList<Sketch> loadSketches_MT(DisplayParams params, String...fnames){
-		return loadSketches_MT(params.mode, params.samplerate, params.reads,
+		return loadSketches_MT(params.mode, params.samplerate, params.maxReads,
 				params.minEntropy, params.minProb, params.minQual, fnames);
 	}
 	
@@ -272,6 +272,7 @@ public final class SketchTool extends SketchObject {
 	
 	//TODO: This is only multithreaded per file in persequence mode.
 	public ArrayList<Sketch> loadSketches_MT(int mode, float samplerate, long reads, float minEntropy, float minProb, byte minQual, String...fnames){
+		
 		ConcurrentLinkedQueue<StringNum> decomposedFnames=new ConcurrentLinkedQueue<StringNum>();
 		int num=0;
 		for(String s : fnames){
@@ -291,7 +292,7 @@ public final class SketchTool extends SketchObject {
 		
 		//Determine how many threads may be used
 		final int threads=Tools.min(Shared.threads(), decomposedFnames.size());
-
+		
 		//Fill a list with LoadThreads
 		ArrayList<LoadThread> allt=new ArrayList<LoadThread>(threads);
 		
@@ -353,7 +354,7 @@ public final class SketchTool extends SketchObject {
 //				allowMultithreadedFastq+", "+forceDisableMultithreadedFastq+", "+(mode==ONE_SKETCH);
 		
 		if(ff.fastq() && allowMultithreadedFastq && !forceDisableMultithreadedFastq && (mode==ONE_SKETCH || mode==PER_FILE) &&
-				maxThreads>1 && Shared.threads()>2 && (reads<1 || reads*samplerate*(mergePairs ? 2 : 1)>=80000)){
+				maxThreads>1 && Shared.threads()>2 && (reads<1 || reads*samplerate*(mergePairs ? 2 : 1)>=1000)){//limit is low due to PacBio long reads
 			
 			final boolean vic=Read.VALIDATE_IN_CONSTRUCTOR;
 			Read.VALIDATE_IN_CONSTRUCTOR=false;
@@ -364,7 +365,10 @@ public final class SketchTool extends SketchObject {
 			Read.VALIDATE_IN_CONSTRUCTOR=vic;
 			
 			ArrayList<Sketch> list=new ArrayList<Sketch>(1);
-			if(sketch!=null && (sketch.length()>0 || allowZeroSizeSketch)){list.add(sketch);}
+			if(sketch!=null && (sketch.length()>0 || allowZeroSizeSketch)){
+				sketch.loadSSU();
+				list.add(sketch);
+			}
 			return list;
 		}
 		if(smm==null){smm=new SketchMakerMini(this, mode, minEntropy, minProb, minQual);}
@@ -387,6 +391,10 @@ public final class SketchTool extends SketchObject {
 		long imgID=-1;
 		long genomeSizeBases=0, genomeSizeKmers=0, genomeSequences=0;
 		long[] baseCounts=null;
+		byte[] r16S=null;
+		byte[] r18S=null;
+		int r16SLen=0;
+		int r18SLen=0;
 		float probCorrect=-1;
 		String name=null, name0=null, fname=ff.simpleName();
 		LongList list=null;
@@ -397,11 +405,263 @@ public final class SketchTool extends SketchObject {
 		
 		for(byte[] line=bf.nextLine(); line!=null; line=bf.nextLine()){
 			if(line.length>0){
-//				System.err.println("Processing line "+new String(line));
+				//				System.err.println("Processing line "+new String(line));
 				if(line[0]=='#'){
-					lastHeader=line;
+					if(r16SLen>0 || r18SLen>0){
+						byte[] ssu=KillSwitch.copyOfRange(line, 5, line.length);
+						if(Tools.startsWith(line, "#16S:") || Tools.startsWith(line, "#SSU:")){
+							assert(r16SLen>0);
+							assert(ssu.length==r16SLen) : r16SLen+", "+line.length+"\n"+new String(line)+"\n";
+							r16S=ssu;
+							r16SLen=0;
+						}else if(Tools.startsWith(line, "#18S:")){
+							assert(r18SLen>0);
+							assert(ssu.length==r18SLen) : r18SLen+", "+line.length+"\n"+new String(line)+"\n";
+							r18S=ssu;
+							r18SLen=0;
+						}else{
+							assert(false) : new String(line);
+						}
+					}else{
+						lastHeader=line;
+						if(list!=null){
+							assert(list.size==list.array.length);
+							if(NUC || unsorted){
+								list.sort();
+								list.shrinkToUnique();
+							}else{
+								list.shrink();
+							}
+							if(list.size()>0 || allowZeroSizeSketch){
+								int[] keyCounts=countList==null ? null : countList.array;
+								Sketch sketch=new Sketch(list.array, keyCounts, baseCounts, r16S, r18S, taxID, imgID, 
+										genomeSizeBases, genomeSizeKmers, genomeSequences, probCorrect, name, name0, fname, meta);
+								sketch.spid=spid;
+								sketches.add(sketch);
+							}
+							//						System.err.println("Made sketch "+sketch);
+						}
+						name=name0=null;
+						fname=ff.simpleName();
+						list=null;
+						countList=null;
+						meta=null;
+						baseCounts=null;
+						r16S=null;
+						r18S=null;
+						r16SLen=0;
+						r18SLen=0;
+						sum=0;
+						taxID=-1;
+						imgID=-1;
+						genomeSizeBases=0;
+						genomeSizeKmers=0;
+						genomeSequences=0;
+						probCorrect=-1;
+						int k_sketch=defaultK;
+						int k2_sketch=0;
+						int hashVersion_sketch=1;
+
+						if(line.length>1){
+							String[] split=new String(line, 1, line.length-1).split("\t");
+							for(String s : split){
+								final int colon=s.indexOf(':');
+								final String sub=s.substring(colon+1);
+								if(s.startsWith("SZ:") || s.startsWith("SIZE:")){//Sketch length
+									currentSketchSize=Integer.parseInt(sub);
+								}else if(s.startsWith("CD:")){//Coding
+									A48=HEX=NUC=delta=counts=unsorted=false;
+
+									for(int i=0; i<sub.length(); i++){
+										char c=sub.charAt(i);
+										if(c=='A'){A48=true;}
+										else if(c=='H'){HEX=true;}
+										else if(c=='R'){A48=HEX=false;}
+										else if(c=='N'){NUC=true;}
+										else if(c=='D'){delta=true;}
+										else if(c=='C'){counts=true;}
+										else if(c=='U'){unsorted=true;}
+										else if(c=='M'){assert(aminoOrTranslate()) : "Amino sketch in non-amino mode: "+new String(line);}
+										else if(c=='8'){assert(amino8) : "Amino8 sketch in non-amino8 mode: "+new String(line);}
+										else{assert(false) : "Unknown coding symbol: "+c+"\t"+new String(line);}
+									}
+								}else if(s.startsWith("K:")){//Kmer length
+									if(sub.indexOf(',')>=0){
+										String[] subsplit=sub.split(",");
+										assert(subsplit.length==2) : "Bad header component "+s;
+										int x=Integer.parseInt(subsplit[0]);
+										int y=Integer.parseInt(subsplit[1]);
+										k_sketch=Tools.max(x, y);
+										k2_sketch=Tools.min(x, y);
+									}else{
+										k_sketch=Integer.parseInt(sub);
+										k2_sketch=0;
+									}
+								}else if(s.startsWith("H:")){//Hash version
+									hashVersion_sketch=Integer.parseInt(sub);
+								}else if(s.startsWith("BC:") || s.startsWith("BASECOUNTS:")){//ACGTN
+									baseCounts=Tools.parseLongArray(sub);
+								}else if(s.startsWith("GS:") || s.startsWith("GSIZE:")){//Genomic bases
+									genomeSizeBases=Long.parseLong(sub);
+								}else if(s.startsWith("GK:") || s.startsWith("GKMERS:")){//Genomic kmers
+									genomeSizeKmers=Long.parseLong(sub);
+								}else if(s.startsWith("GQ:")){
+									genomeSequences=Long.parseLong(sub);
+								}else if(s.startsWith("GE:")){//Genome size estimate kmers
+									//ignore
+								}else if(s.startsWith("PC:")){//Probability of correctness
+									probCorrect=Float.parseFloat(sub);
+								}else if(s.startsWith("ID:") || s.startsWith("TAXID:")){
+									taxID=Integer.parseInt(sub);
+								}else if(s.startsWith("IMG:")){
+									imgID=Long.parseLong(sub);
+								}else if(s.startsWith("SPID:")){
+									spid=Integer.parseInt(sub);
+								}else if(s.startsWith("NM:") || s.startsWith("NAME:")){
+									name=sub;
+								}else if(s.startsWith("FN:")){
+									fname=sub;
+								}else if(s.startsWith("NM0:")){
+									name0=sub;
+								}else if(s.startsWith("MT_")){
+									if(meta==null){meta=new ArrayList<String>(1);}
+									meta.add(s.substring(3));
+								}else if(s.startsWith("16S:") || s.startsWith("SSU:")){
+									r16SLen=Integer.parseInt(sub);
+								}else if(s.startsWith("18S:")){
+									r18SLen=Integer.parseInt(sub);
+								}else{
+									assert(false) : "Unsupported header tag "+s;
+								}
+							}
+						}
+
+						if(KILL_OK){
+							if(k_sketch!=k && !NUC){KillSwitch.kill("Sketch kmer length "+k_sketch+
+									" differs from loaded kmer length "+k+"\n"+new String(line)+"\nfile: "+ff.name());}
+							if(k2_sketch!=k2 && !NUC){KillSwitch.kill("Sketch kmer length "+k_sketch+","+k2_sketch+
+									" differs from loaded kmer length "+k+","+k2+"\n"+new String(line)+"\nfile: "+ff.name());}
+							if(hashVersion_sketch!=HASH_VERSION && !NUC){KillSwitch.kill("Sketch hash version "+hashVersion_sketch+
+									" differs from loaded hash version "+HASH_VERSION+".\n"
+									+ "You may need to download the latest version of BBTools.\n"+new String(line)+"\nfile: "+ff.name());}
+						}else{//Potential hang
+							assert(k_sketch==k || NUC) : "Sketch kmer length "+k_sketch+" differs from loaded kmer length "+k+"\n"+new String(line);
+							assert(k2_sketch==k2 || NUC) : "Sketch kmer length "+k_sketch+","+k2_sketch+" differs from loaded kmer length "+k+","+k2+"\n"+new String(line);
+							assert(hashVersion_sketch==HASH_VERSION || NUC) : "Sketch hash version "+hashVersion_sketch+
+							" differs from loaded hash version "+HASH_VERSION+".\n"
+							+ "You may need to download the latest version of BBTools.\n"+new String(line)+"\n";
+						}
+						if(currentSketchSize>0 || allowZeroSizeSketch){
+							list=new LongList(Tools.max(1, currentSketchSize));
+							if(counts){countList=new IntList(Tools.max(1, currentSketchSize));}
+						}
+					}
+				}else{
+					long x=(counts ? Sketch.parseA48C(line, countList) : A48 ? Sketch.parseA48(line) :
+						HEX ? Sketch.parseHex(line) : NUC ? Sketch.parseNuc(line) : Tools.parseLong(line));
+//					System.err.println("sum="+sum+", x="+x+" -> "+(sum+x));
+					sum+=x;
+					assert(x>=0 || NUC) : "x="+x+"\nline="+new String(line)+"\nheader="+(lastHeader==null ? "null" : new String(lastHeader))+"\nlineNum="+bf.lineNum()+"\n";
+					assert(sum>=0 || !delta) : "The sketch was made with delta compression off.  Please regenerate it.";
+					assert(list!=null) : new String(line);
+					long key=(delta ? sum : x);
+					if(key>=0){list.add(key);}
+				}
+			}
+		}
+		if(list!=null && (list.size>0 || allowZeroSizeSketch)){
+			assert(list.size==list.array.length || NUC || unsorted || (allowZeroSizeSketch && list.size==0)) : list.size+"!="+list.array.length;
+			if(NUC || unsorted){
+				list.sort();
+				list.shrinkToUnique();
+			}else{
+				list.shrink();
+			}
+			int[] keyCounts=countList==null ? null : countList.array;
+			Sketch sketch=new Sketch(list.array, keyCounts, baseCounts, r16S, r18S, taxID, imgID, 
+					genomeSizeBases, genomeSizeKmers, genomeSequences, probCorrect, name, name0, fname, meta);
+			sketch.spid=spid;
+			sketches.add(sketch);
+		}
+		if(verbose2){System.err.println("Loaded "+sketches.size()+" sketches from text.");}
+		return sketches;
+	}
+	
+	/** Usually much faster due to not manifesting the multithreaded load Java slowdown.  Should incur less garbage collection also. */
+	private ArrayList<Sketch> loadSketchesFromSketchFile2(final FileFormat ff, boolean allowZeroSizeSketch){
+		
+		boolean A48=(Sketch.CODING==Sketch.A48), HEX=(Sketch.CODING==Sketch.HEX), NUC=false, delta=true, counts=false, unsorted=false;
+		
+		if(verbose2){System.err.println("Loading sketches from text.");}
+		ArrayList<Sketch> sketches=new ArrayList<Sketch>();
+		
+		InputStream is=ReadWrite.getInputStream(ff.name(), BUFFERED_READER, false);
+		byte[] buffer=new byte[BUFLEN];
+		int start, limit=0;
+		try {
+			limit=is.read(buffer);
+		} catch (IOException e) {
+			KillSwitch.exceptionKill(e);
+		}
+		
+		
+		int currentSketchSize=stTargetSketchSize;
+		int taxID=-1;
+		long spid=-1;
+		long imgID=-1;
+		long genomeSizeBases=0, genomeSizeKmers=0, genomeSequences=0;
+		long[] baseCounts=null;
+		byte[] r16S=null;
+		byte[] r18S=null;
+		int r16SLen=0;
+		int r18SLen=0;
+		float probCorrect=-1;
+		String name=null, name0=null, fname=ff.simpleName();
+		LongList list=null;
+		IntList countList=null;
+		ArrayList<String> meta=null;
+		long sum=0;
+		byte[] lastHeader=null;
+		ByteBuilder bb=new ByteBuilder(256);
+		
+		for(start=0; start<limit;){
+			//				System.err.println("Processing line "+new String(line));
+			if(buffer[start]=='#'){
+				bb.clear();
+				try {
+					while(buffer[start]!='\n'){
+						bb.append(buffer[start]);
+						start++;
+						if(start>=limit){start=0; limit=is.read(buffer);}
+					}
+				} catch (IOException e) {
+					KillSwitch.exceptionKill(e);
+				}
+				start++;
+				
+				if(r16SLen>0 || r18SLen>0){
+					byte[] ssu=bb.toBytes(5, bb.length());
+					if(bb.startsWith("#16S:") || bb.startsWith("#SSU:")){
+						assert(r16SLen>0);
+						assert(ssu.length==r16SLen) : r16SLen+", "+bb.length+"\n"+bb+"\n";
+						r16S=ssu;
+						r16SLen=0;
+					}else if(bb.startsWith("#18S:")){
+						assert(r18SLen>0);
+						assert(ssu.length==r18SLen) : r18SLen+", "+bb.length+"\n"+bb+"\n";
+						r18S=ssu;
+						r18SLen=0;
+					}else{
+						assert(false) : bb;
+					}
+				}else{
+
+					//				byte[] line=lastHeader=bb.toBytes();
 					if(list!=null){
-						assert(list.size==list.array.length);
+
+						//This assertion fails sometimes for Silva per-sequence mode, but it's not important
+						//					assert(list.size==list.array.length) : list.size+", "+list.array.length+(lastHeader==null ? "" : ", "+new String(lastHeader));
+
 						if(NUC || unsorted){
 							list.sort();
 							list.shrinkToUnique();
@@ -410,18 +670,23 @@ public final class SketchTool extends SketchObject {
 						}
 						if(list.size()>0 || allowZeroSizeSketch){
 							int[] keyCounts=countList==null ? null : countList.array;
-							Sketch sketch=new Sketch(list.array, keyCounts, baseCounts, taxID, imgID, 
+							Sketch sketch=new Sketch(list.array, keyCounts, baseCounts, r16S, r18S, taxID, imgID, 
 									genomeSizeBases, genomeSizeKmers, genomeSequences, probCorrect, name, name0, fname, meta);
 							sketch.spid=spid;
 							sketches.add(sketch);
 						}
-//						System.err.println("Made sketch "+sketch);
+						//						System.err.println("Made sketch "+sketch);
 					}
 					name=name0=null;
 					fname=ff.simpleName();
 					list=null;
 					countList=null;
 					meta=null;
+					baseCounts=null;
+					r16S=null;
+					r18S=null;
+					r16SLen=0;
+					r18SLen=0;
 					sum=0;
 					taxID=-1;
 					imgID=-1;
@@ -432,9 +697,9 @@ public final class SketchTool extends SketchObject {
 					int k_sketch=defaultK;
 					int k2_sketch=0;
 					int hashVersion_sketch=1;
-					
-					if(line.length>1){
-						String[] split=new String(line, 1, line.length-1).split("\t");
+
+					if(bb.length>1){
+						String[] split=new String(bb.array, 1, bb.length-1).split("\t");
 						for(String s : split){
 							final int colon=s.indexOf(':');
 							final String sub=s.substring(colon+1);
@@ -442,7 +707,7 @@ public final class SketchTool extends SketchObject {
 								currentSketchSize=Integer.parseInt(sub);
 							}else if(s.startsWith("CD:")){//Coding
 								A48=HEX=NUC=delta=counts=unsorted=false;
-								
+
 								for(int i=0; i<sub.length(); i++){
 									char c=sub.charAt(i);
 									if(c=='A'){A48=true;}
@@ -452,9 +717,9 @@ public final class SketchTool extends SketchObject {
 									else if(c=='D'){delta=true;}
 									else if(c=='C'){counts=true;}
 									else if(c=='U'){unsorted=true;}
-									else if(c=='M'){assert(aminoOrTranslate()) : "Amino sketch in non-amino mode: "+new String(line);}
-									else if(c=='8'){assert(amino8) : "Amino8 sketch in non-amino8 mode: "+new String(line);}
-									else{assert(false) : "Unknown coding symbol: "+c+"\t"+new String(line);}
+									else if(c=='M'){assert(aminoOrTranslate()) : "Amino sketch in non-amino mode: "+bb;}
+									else if(c=='8'){assert(amino8) : "Amino8 sketch in non-amino8 mode: "+bb;}
+									else{assert(false) : "Unknown coding symbol: "+c+"\t"+bb;}
 								}
 							}else if(s.startsWith("K:")){//Kmer length
 								if(sub.indexOf(',')>=0){
@@ -497,236 +762,35 @@ public final class SketchTool extends SketchObject {
 							}else if(s.startsWith("MT_")){
 								if(meta==null){meta=new ArrayList<String>(1);}
 								meta.add(s.substring(3));
+							}else if(s.startsWith("16S:") || s.startsWith("SSU:")){
+								r16SLen=Integer.parseInt(sub);
+							}else if(s.startsWith("18S:")){
+								r18SLen=Integer.parseInt(sub);
 							}else{
 								assert(false) : "Unsupported header tag "+s;
 							}
 						}
 					}
-					
+
 					if(KILL_OK){
 						if(k_sketch!=k && !NUC){KillSwitch.kill("Sketch kmer length "+k_sketch+
-								" differs from loaded kmer length "+k+"\n"+new String(line)+"\nfile: "+ff.name());}
+								" differs from loaded kmer length "+k+"\n"+bb+"\nfile: "+ff.name());}
 						if(k2_sketch!=k2 && !NUC){KillSwitch.kill("Sketch kmer length "+k_sketch+","+k2_sketch+
-								" differs from loaded kmer length "+k+","+k2+"\n"+new String(line)+"\nfile: "+ff.name());}
+								" differs from loaded kmer length "+k+","+k2+"\n"+bb+"\nfile: "+ff.name());}
 						if(hashVersion_sketch!=HASH_VERSION && !NUC){KillSwitch.kill("Sketch hash version "+hashVersion_sketch+
 								" differs from loaded hash version "+HASH_VERSION+".\n"
-										+ "You may need to download the latest version of BBTools.\n"+new String(line)+"\nfile: "+ff.name());}
+								+ "You may need to download the latest version of BBTools.\nfile: "+ff.name());}
 					}else{//Potential hang
-						assert(k_sketch==k || NUC) : "Sketch kmer length "+k_sketch+" differs from loaded kmer length "+k+"\n"+new String(line);
-						assert(k2_sketch==k2 || NUC) : "Sketch kmer length "+k_sketch+","+k2_sketch+" differs from loaded kmer length "+k+","+k2+"\n"+new String(line);
+						assert(k_sketch==k || NUC) : "Sketch kmer length "+k_sketch+" differs from loaded kmer length "+k+"\n"+bb;
+						assert(k2_sketch==k2 || NUC) : "Sketch kmer length "+k_sketch+","+k2_sketch+" differs from loaded kmer length "+k+","+k2+"\n"+bb;
 						assert(hashVersion_sketch==HASH_VERSION || NUC) : "Sketch hash version "+hashVersion_sketch+
 						" differs from loaded hash version "+HASH_VERSION+".\n"
-								+ "You may need to download the latest version of BBTools.\n"+new String(line)+"\n";
+						+ "You may need to download the latest version of BBTools.\nfile: "+ff.name();
 					}
 					if(currentSketchSize>0 || allowZeroSizeSketch){
 						list=new LongList(Tools.max(1, currentSketchSize));
 						if(counts){countList=new IntList(Tools.max(1, currentSketchSize));}
 					}
-				}else{
-					long x=(counts ? Sketch.parseA48C(line, countList) : A48 ? Sketch.parseA48(line) :
-						HEX ? Sketch.parseHex(line) : NUC ? Sketch.parseNuc(line) : Tools.parseLong(line));
-//					System.err.println("sum="+sum+", x="+x+" -> "+(sum+x));
-					sum+=x;
-					assert(x>=0 || NUC) : "x="+x+"\nline="+new String(line)+"\nheader="+(lastHeader==null ? "null" : new String(lastHeader))+"\nlineNum="+bf.lineNum()+"\n";
-					assert(sum>=0 || !delta) : "The sketch was made with delta compression off.  Please regenerate it.";
-					assert(list!=null) : new String(line);
-					long key=(delta ? sum : x);
-					if(key>=0){list.add(key);}
-				}
-			}
-		}
-		if(list!=null && (list.size>0 || allowZeroSizeSketch)){
-			assert(list.size==list.array.length || NUC || unsorted || (allowZeroSizeSketch && list.size==0)) : list.size+"!="+list.array.length;
-			if(NUC || unsorted){
-				list.sort();
-				list.shrinkToUnique();
-			}else{
-				list.shrink();
-			}
-			int[] keyCounts=countList==null ? null : countList.array;
-			Sketch sketch=new Sketch(list.array, keyCounts, baseCounts, taxID, imgID, 
-					genomeSizeBases, genomeSizeKmers, genomeSequences, probCorrect, name, name0, fname, meta);
-			sketch.spid=spid;
-			sketches.add(sketch);
-		}
-		if(verbose2){System.err.println("Loaded "+sketches.size()+" sketches from text.");}
-		return sketches;
-	}
-	
-	/** Usually much faster due to not manifesting the multithreaded load Java slowdown.  Should incur less garbage collection also. */
-	private ArrayList<Sketch> loadSketchesFromSketchFile2(final FileFormat ff, boolean allowZeroSizeSketch){
-		
-		boolean A48=(Sketch.CODING==Sketch.A48), HEX=(Sketch.CODING==Sketch.HEX), NUC=false, delta=true, counts=false, unsorted=false;
-		
-		if(verbose2){System.err.println("Loading sketches from text.");}
-		ArrayList<Sketch> sketches=new ArrayList<Sketch>();
-		
-		InputStream is=ReadWrite.getInputStream(ff.name(), BUFFERED_READER, false);
-		byte[] buffer=new byte[BUFLEN];
-		int start, limit=0;
-		try {
-			limit=is.read(buffer);
-		} catch (IOException e) {
-			KillSwitch.exceptionKill(e);
-		}
-		
-		
-		int currentSketchSize=stTargetSketchSize;
-		int taxID=-1;
-		long spid=-1;
-		long imgID=-1;
-		long genomeSizeBases=0, genomeSizeKmers=0, genomeSequences=0;
-		long[] baseCounts=null;
-		float probCorrect=-1;
-		String name=null, name0=null, fname=ff.simpleName();
-		LongList list=null;
-		IntList countList=null;
-		ArrayList<String> meta=null;
-		long sum=0;
-		byte[] lastHeader=null;
-		ByteBuilder bb=new ByteBuilder(256);
-		
-		for(start=0; start<limit;){
-			//				System.err.println("Processing line "+new String(line));
-			if(buffer[start]=='#'){
-				bb.clear();
-				try {
-					while(buffer[start]!='\n'){
-						bb.append(buffer[start]);
-						start++;
-						if(start>=limit){start=0; limit=is.read(buffer);}
-					}
-				} catch (IOException e) {
-					KillSwitch.exceptionKill(e);
-				}
-				start++;
-				
-//				byte[] line=lastHeader=bb.toBytes();
-				if(list!=null){
-					
-					//This assertion fails sometimes for Silva per-sequence mode, but it's not important
-//					assert(list.size==list.array.length) : list.size+", "+list.array.length+(lastHeader==null ? "" : ", "+new String(lastHeader));
-					
-					if(NUC || unsorted){
-						list.sort();
-						list.shrinkToUnique();
-					}else{
-						list.shrink();
-					}
-					if(list.size()>0 || allowZeroSizeSketch){
-						int[] keyCounts=countList==null ? null : countList.array;
-						Sketch sketch=new Sketch(list.array, keyCounts, baseCounts, taxID, imgID, 
-								genomeSizeBases, genomeSizeKmers, genomeSequences, probCorrect, name, name0, fname, meta);
-						sketch.spid=spid;
-						sketches.add(sketch);
-					}
-					//						System.err.println("Made sketch "+sketch);
-				}
-				name=name0=null;
-				fname=ff.simpleName();
-				list=null;
-				countList=null;
-				meta=null;
-				sum=0;
-				taxID=-1;
-				imgID=-1;
-				genomeSizeBases=0;
-				genomeSizeKmers=0;
-				genomeSequences=0;
-				baseCounts=null;
-				probCorrect=-1;
-				int k_sketch=defaultK;
-				int k2_sketch=0;
-				int hashVersion_sketch=1;
-
-				if(bb.length>1){
-					String[] split=new String(bb.array, 1, bb.length-1).split("\t");
-					for(String s : split){
-						final int colon=s.indexOf(':');
-						final String sub=s.substring(colon+1);
-						if(s.startsWith("SZ:") || s.startsWith("SIZE:")){//Sketch length
-							currentSketchSize=Integer.parseInt(sub);
-						}else if(s.startsWith("CD:")){//Coding
-							A48=HEX=NUC=delta=counts=unsorted=false;
-
-							for(int i=0; i<sub.length(); i++){
-								char c=sub.charAt(i);
-								if(c=='A'){A48=true;}
-								else if(c=='H'){HEX=true;}
-								else if(c=='R'){A48=HEX=false;}
-								else if(c=='N'){NUC=true;}
-								else if(c=='D'){delta=true;}
-								else if(c=='C'){counts=true;}
-								else if(c=='U'){unsorted=true;}
-								else if(c=='M'){assert(aminoOrTranslate()) : "Amino sketch in non-amino mode: "+bb;}
-								else if(c=='8'){assert(amino8) : "Amino8 sketch in non-amino8 mode: "+bb;}
-								else{assert(false) : "Unknown coding symbol: "+c+"\t"+bb;}
-							}
-						}else if(s.startsWith("K:")){//Kmer length
-							if(sub.indexOf(',')>=0){
-								String[] subsplit=sub.split(",");
-								assert(subsplit.length==2) : "Bad header component "+s;
-								int x=Integer.parseInt(subsplit[0]);
-								int y=Integer.parseInt(subsplit[1]);
-								k_sketch=Tools.max(x, y);
-								k2_sketch=Tools.min(x, y);
-							}else{
-								k_sketch=Integer.parseInt(sub);
-								k2_sketch=0;
-							}
-						}else if(s.startsWith("H:")){//Hash version
-							hashVersion_sketch=Integer.parseInt(sub);
-						}else if(s.startsWith("BC:") || s.startsWith("BASECOUNTS:")){//ACGTN
-							baseCounts=Tools.parseLongArray(sub);
-						}else if(s.startsWith("GS:") || s.startsWith("GSIZE:")){//Genomic bases
-							genomeSizeBases=Long.parseLong(sub);
-						}else if(s.startsWith("GK:") || s.startsWith("GKMERS:")){//Genomic kmers
-							genomeSizeKmers=Long.parseLong(sub);
-						}else if(s.startsWith("GQ:")){
-							genomeSequences=Long.parseLong(sub);
-						}else if(s.startsWith("GE:")){//Genome size estimate kmers
-							//ignore
-						}else if(s.startsWith("PC:")){//Probability of correctness
-							probCorrect=Float.parseFloat(sub);
-						}else if(s.startsWith("ID:") || s.startsWith("TAXID:")){
-							taxID=Integer.parseInt(sub);
-						}else if(s.startsWith("IMG:")){
-							imgID=Long.parseLong(sub);
-						}else if(s.startsWith("SPID:")){
-							spid=Integer.parseInt(sub);
-						}else if(s.startsWith("NM:") || s.startsWith("NAME:")){
-							name=sub;
-						}else if(s.startsWith("FN:")){
-							fname=sub;
-						}else if(s.startsWith("NM0:")){
-							name0=sub;
-						}else if(s.startsWith("MT_")){
-							if(meta==null){meta=new ArrayList<String>(1);}
-							meta.add(s.substring(3));
-						}else{
-							assert(false) : "Unsupported header tag "+s;
-						}
-					}
-				}
-				
-				if(KILL_OK){
-					if(k_sketch!=k && !NUC){KillSwitch.kill("Sketch kmer length "+k_sketch+
-							" differs from loaded kmer length "+k+"\n"+bb+"\nfile: "+ff.name());}
-					if(k2_sketch!=k2 && !NUC){KillSwitch.kill("Sketch kmer length "+k_sketch+","+k2_sketch+
-							" differs from loaded kmer length "+k+","+k2+"\n"+bb+"\nfile: "+ff.name());}
-					if(hashVersion_sketch!=HASH_VERSION && !NUC){KillSwitch.kill("Sketch hash version "+hashVersion_sketch+
-							" differs from loaded hash version "+HASH_VERSION+".\n"
-									+ "You may need to download the latest version of BBTools.\nfile: "+ff.name());}
-				}else{//Potential hang
-					assert(k_sketch==k || NUC) : "Sketch kmer length "+k_sketch+" differs from loaded kmer length "+k+"\n"+bb;
-					assert(k2_sketch==k2 || NUC) : "Sketch kmer length "+k_sketch+","+k2_sketch+" differs from loaded kmer length "+k+","+k2+"\n"+bb;
-					assert(hashVersion_sketch==HASH_VERSION || NUC) : "Sketch hash version "+hashVersion_sketch+
-					" differs from loaded hash version "+HASH_VERSION+".\n"
-							+ "You may need to download the latest version of BBTools.\nfile: "+ff.name();
-				}
-				if(currentSketchSize>0 || allowZeroSizeSketch){
-					list=new LongList(Tools.max(1, currentSketchSize));
-					if(counts){countList=new IntList(Tools.max(1, currentSketchSize));}
 				}
 			}else{
 				bb.clear();
@@ -774,7 +838,7 @@ public final class SketchTool extends SketchObject {
 				list.shrink();
 			}
 			int[] keyCounts=countList==null ? null : countList.array;
-			Sketch sketch=new Sketch(list.array, keyCounts, baseCounts, taxID, imgID, 
+			Sketch sketch=new Sketch(list.array, keyCounts, baseCounts, r16S, r18S, taxID, imgID, 
 					genomeSizeBases, genomeSizeKmers, genomeSequences, probCorrect, name, name0, fname, meta);
 			sketch.spid=spid;
 			sketches.add(sketch);
@@ -800,6 +864,10 @@ public final class SketchTool extends SketchObject {
 		long imgID=-1;
 		long genomeSizeBases=0, genomeSizeKmers=0, genomeSequences=0;
 		long[] baseCounts=null;
+		byte[] r16S=null;
+		byte[] r18S=null;
+		int r16SLen=0;
+		int r18SLen=0;
 		float probCorrect=-1;
 		String name=null, name0=null, fname=null;
 		LongList list=null;
@@ -814,6 +882,21 @@ public final class SketchTool extends SketchObject {
 				if(line.charAt(0)=='#'){
 					if(line.length()>1 && line.charAt(1)=='#'){
 						//ignore
+					}else if(r16SLen>0 || r18SLen>0){
+						byte[] ssu=KillSwitch.copyOfRange(line.getBytes(), 5, line.length());
+						if(line.startsWith("#16S:") || line.startsWith("#SSU:")){
+							assert(r16SLen>0);
+							assert(ssu.length==r16SLen) : r16SLen+", "+line.length()+"\n"+line+"\n";
+							r16S=ssu;
+							r16SLen=0;
+						}else if(line.startsWith("#18S:")){
+							assert(r18SLen>0);
+							assert(ssu.length==r18SLen) : r18SLen+", "+line.length()+"\n"+line+"\n";
+							r18S=ssu;
+							r18SLen=0;
+						}else{
+							assert(false) : line;
+						}
 					}else{
 						if(list!=null){
 							assert(list.size==list.array.length);
@@ -825,7 +908,7 @@ public final class SketchTool extends SketchObject {
 							}
 							if(list.size()>0){
 								int[] keyCounts=countList==null ? null : countList.array;
-								Sketch sketch=new Sketch(list.array, keyCounts, baseCounts, taxID, imgID, 
+								Sketch sketch=new Sketch(list.array, keyCounts, baseCounts, r16S, r18S, taxID, imgID, 
 										genomeSizeBases, genomeSizeKmers, genomeSequences, probCorrect, name, name0, fname, meta);
 								sketch.spid=spid;
 								sketches.add(sketch);
@@ -837,13 +920,17 @@ public final class SketchTool extends SketchObject {
 						list=null;
 						countList=null;
 						meta=null;
+						baseCounts=null;
+						r16S=null;
+						r18S=null;
+						r16SLen=0;
+						r18SLen=0;
 						sum=0;
 						taxID=-1;
 						imgID=-1;
 						genomeSizeBases=0;
 						genomeSizeKmers=0;
 						genomeSequences=0;
-						baseCounts=null;
 						probCorrect=-1;
 						int k_sketch=defaultK;
 						int k2_sketch=0;
@@ -914,6 +1001,10 @@ public final class SketchTool extends SketchObject {
 								}else if(s.startsWith("MT_")){
 									if(meta==null){meta=new ArrayList<String>(1);}
 									meta.add(s.substring(3));
+								}else if(s.startsWith("16S:") || s.startsWith("SSU:")){
+									r16SLen=Integer.parseInt(sub);
+								}else if(s.startsWith("18S:")){
+									r18SLen=Integer.parseInt(sub);
 								}else{
 									assert(false) : "Unsupported header tag "+s;
 								}
@@ -961,7 +1052,7 @@ public final class SketchTool extends SketchObject {
 				list.shrink();
 			}
 			int[] keyCounts=countList==null ? null : countList.array;
-			Sketch sketch=new Sketch(list.array, keyCounts, baseCounts, taxID, imgID, 
+			Sketch sketch=new Sketch(list.array, keyCounts, baseCounts, r16S, r18S, taxID, imgID, 
 					genomeSizeBases, genomeSizeKmers, genomeSequences, probCorrect, name, name0, fname, meta);
 			sketch.spid=spid;
 			sketches.add(sketch);
@@ -991,7 +1082,7 @@ public final class SketchTool extends SketchObject {
 			for(StringNum sn=queue.poll(); sn!=null; sn=queue.poll()){
 				ArrayList<Sketch> temp=null;
 				try {
-					temp=loadSketchesFromFile(sn.s, smm, 1, reads, smm.mode, samplerate, smm.minEntropy(), smm.minProb, smm.minQual, false);
+					temp=loadSketchesFromFile(sn.s, smm, 1, reads, smm.mode, samplerate, smm.minEntropy(), smm.minProb(), smm.minQual(), false);
 				} catch (Throwable e) {
 					System.err.println("Failure loading "+sn+":\n"+e);
 					e.printStackTrace();
@@ -1171,8 +1262,9 @@ public final class SketchTool extends SketchObject {
 //			if(verbose){outstream.println("Started cris");}
 		}
 		
-		final int threads=Tools.min(maxThreads,
-				(compressed ? 1 : 2)*(ffin2==null ? 4 : 8)*(mergePairs ? 3 : minEntropy>0 ? 2 : 1));
+		//TODO: bgzip actually decompresses fast.
+		final int threads=(int)Tools.min(maxThreads,
+				1.75f*(compressed ? 1 : 2)*(ffin2==null ? 4 : 8)*(mergePairs ? 3 : minEntropy>0 ? 2 : 1));
 		
 		if(verbose2){System.err.println("Starting "+threads+" load threads.");}
 		ArrayList<LoadThread2> list=new ArrayList<LoadThread2>(threads);
