@@ -75,6 +75,7 @@ public class SortByName {
 		
 		//Set shared static variables
 		Shared.capBuffers(4);
+		Shared.setBufferData(1000000);
 		ReadWrite.USE_PIGZ=ReadWrite.USE_UNPIGZ=true;
 		ReadWrite.MAX_ZIP_THREADS=Shared.threads();
 		FASTQ.TEST_INTERLEAVED=true; //TEST_INTERLEAVED must explicitly be disabled with int=f to sort corrupt files.
@@ -100,6 +101,8 @@ public class SortByName {
 //				verbose2=Tools.parseBoolean(b);
 			}else if(a.equals("delete")){
 				delete=Tools.parseBoolean(b);
+			}else if(a.equalsIgnoreCase("deleteearly")){
+				deleteEarly=Tools.parseBoolean(b);
 			}else if(a.equals("allowtemp") || a.equals("usetemp")){
 				allowTempFiles=Tools.parseBoolean(b);
 			}else if(a.equals("memmult") || a.equals("mult")){
@@ -153,6 +156,8 @@ public class SortByName {
 				taxTreeFile=b;
 			}else if(a.equals("maxfiles") || a.equals("files")){
 				maxFiles=Integer.parseInt(b);
+				assert(maxFiles>1 && maxFiles<=10000000) : "Invalid value for maxfiles: "+maxFiles+"; range is 2-10000000";
+				setMaxFiles=true;
 			}else if(a.equals("parse_flag_goes_here")){
 				//Set a variable here
 			}else if(parser.parse(arg, a, b)){//Parse standard flags in the parser
@@ -300,6 +305,7 @@ public class SortByName {
 			}
 		}
 		
+		qualityObserved=!ffin1.fasta();
 	}
 	
 	/*--------------------------------------------------------------*/
@@ -370,7 +376,7 @@ public class SortByName {
 		final long currentLimit=(long)(maxMem*memMult);
 		final int readLimit=2000000000;
 		long currentMem=0;
-		long dumped=0;
+		long bytesDumped=0;
 		long dumps=0;
 //		IntList dumpCount=new IntList();
 		AtomicLong outstandingMem=new AtomicLong();
@@ -419,7 +425,7 @@ public class SortByName {
 //					dumpCount.add(storage.size());
 					sortAndDump(storage, currentMem, outstandingMem, null, false);
 					storage=new ArrayList<Read>();
-					dumped+=currentMem;
+					bytesDumped+=currentMem;
 					dumps++;
 					currentMem=0;
 					if(verbose){outstream.println("Waiting on memory; outstandingMem="+outstandingMem);}
@@ -460,11 +466,6 @@ public class SortByName {
 			waitOnMemory(outstandingMem, 0);
 			outstream.println("Merging "+(dumps+1)+" files.");
 			ReadWrite.ZIPLEVEL=ziplevel0;
-			if(maxLengthObserved*(dumped+1)>200000000L){
-				outstream.println("Reduced buffer sizes prior to merging due to low memory.");
-				Shared.capBufferLen(4);
-				Shared.capBuffers(1);
-			}
 			mergeAndDump(outTemp, /*dumpCount, */useSharedHeader, false);
 		}
 		
@@ -489,15 +490,45 @@ public class SortByName {
 	/*--------------------------------------------------------------*/
 	/*----------------         Inner Methods        ----------------*/
 	/*--------------------------------------------------------------*/
+
+	private final long maxSizeObserved(){//Includes some extra for a margin of error
+		return Tools.max(maxSizeObservedStatic(), Tools.max(maxLengthObserved, 150)*(qualityObserved ? 2 : 1)+500);
+	}
+	final static long maxSizeObservedStatic(){//Includes some extra for a margin of error
+		return Tools.max(maxLengthObservedStatic, 150)*(qualityObservedStatic ? 2 : 1)+500;
+	}
+	private static long mergeMemNeeded(long maxSizeObserved, int inFiles){
+		return (long)(maxSizeObserved*2*(inFiles+1)*1.6);
+	}
+	private static int maxMergeFiles(long maxSizeObserved){
+		final long memAvailable=Shared.memAvailableAdvanced();
+		int files=(int)Tools.max(3, memAvailable/Tools.max(maxSizeObserved, 500))-1;
+		return Tools.min(files, 10000);
+	}
+	private static void adjustBuffers(long maxSizeObserved, int inFiles, PrintStream outstream){
+		if(Shared.numBuffers()<3){return;}
+		long memNeeded=mergeMemNeeded(maxSizeObserved, inFiles);
+		long memAvailable=Shared.memAvailableAdvanced();
+		if(memNeeded>memAvailable*0.8){
+			outstream.println("Reduced buffers prior to merging due to low memory.");
+			Shared.capBufferLen(40);
+			Shared.capBuffers(2);
+		}
+	}
+	
+	/*--------------------------------------------------------------*/
 	
 	private ArrayList<String> mergeRecursive(final ArrayList<String> inList, boolean allowSubprocess){
 		assert(maxFiles>1);
+		
 		ArrayList<String> currentList=inList;
 		final int oldZL=ReadWrite.ZIPLEVEL;
+		final boolean deleteAfterMerge=(delete && deleteEarly);
+		final int limit=Tools.min(mergeFileLimit, maxFiles, maxMergeFiles(maxSizeObserved()));
 		while(currentList.size()>maxFiles){
 			ReadWrite.ZIPLEVEL=Tools.min(ReadWrite.ZIPLEVEL, 4);
 			final int size=currentList.size();
-			final int groups=(size+maxFiles-1)/maxFiles;
+			final int groups=(size+limit-1)/limit;
 			assert(groups>0 && groups<size);
 			ArrayList<ArrayList<String>> listList=new ArrayList<ArrayList<String>>();
 			ArrayList<String> outList=new ArrayList<String>();
@@ -509,18 +540,25 @@ public class SortByName {
 			}
 			for(ArrayList<String> subList : listList){
 				String temp=getTempFile();
-				FileFormat ff=FileFormat.testOutput(temp, FileFormat.FASTQ, null, true, false, false, false);
-				merge(subList, ff, null, allowSubprocess);
+				FileFormat ffoutTemp=FileFormat.testOutput(temp, FileFormat.FASTQ, null, true, false, false, false);
+				mergeRecursiveInner(subList, ffoutTemp, null, deleteAfterMerge, allowSubprocess);
 				outList.add(temp);
 			}
+			if(delete && !deleteEarly){//delete them now that this pass is finished
+				System.err.println("Deleting "+size+" temp files...");
+				for(ArrayList<String> subList : listList){
+					for(String fname : subList){new File(fname).delete();}
+				}
+				System.err.println(" done.");
+			}
 			currentList=outList;
 		}
 		ReadWrite.ZIPLEVEL=oldZL;
 		return currentList;
 	}
 	
-	public void merge(ArrayList<String> inList, FileFormat ff1, FileFormat ff2, boolean allowSubprocess){
-		errorState|=mergeAndDump(inList, /*null, */ff1, ff2, delete, useSharedHeader, allowSubprocess, outstream, maxLengthObserved);
+	public void mergeRecursiveInner(ArrayList<String> inList, FileFormat ffoutTemp1, FileFormat ffoutTemp2, boolean deleteAfterMerge, boolean allowSubprocess){
+		errorState|=mergeAndDump(inList, /*null, */ffoutTemp1, ffoutTemp2, deleteAfterMerge, useSharedHeader, allowSubprocess, outstream, maxSizeObserved());
 	}
 	
 	private String getTempFile(){
@@ -539,25 +577,28 @@ public class SortByName {
 	}
 	
 	private boolean mergeAndDump(ArrayList<String> fnames, /*IntList dumpCount, */boolean useHeader, boolean allowSubprocess) {
-		if(fnames.size()*maxLengthObserved>2000000000 || fnames.size()>64){
+		final long maxSizeObserved=maxSizeObserved();
+		final int limit=Tools.min(mergeFileLimit, maxMergeFiles(maxSizeObserved));
+		if(fnames.size()>limit){
 			outstream.println("Performing recursive merge to reduce open files.");
 			fnames=mergeRecursive(fnames, allowSubprocess);
 		}
-		return mergeAndDump(fnames, /*dumpCount,*/ ffout1, ffout2, delete, useHeader, allowSubprocess, outstream, maxLengthObserved);
+		return mergeAndDump(fnames, /*dumpCount,*/ ffout1, ffout2, delete, useHeader, allowSubprocess, outstream, maxSizeObserved);
 	}
 	
 	public static boolean mergeAndDump(ArrayList<String> fnames, FileFormat ffout1, FileFormat ffout2, 
-			boolean delete, boolean useHeader, boolean allowSubprocess, PrintStream outstream, long maxLengthObserved) {
+			boolean deleteAfterMerge, boolean useHeader, boolean allowSubprocess, PrintStream outstream, final long maxSizeObserved) {
 		
 		final int oldBuffers=Shared.numBuffers();
 		final int oldBufferLen=Shared.bufferLen();
-		
-		if(fnames.size()>3 && (maxLengthObserved<1 || maxLengthObserved>100000)){
-			outstream.println("Reduced buffer sizes prior to merging.");
-			Shared.capBufferLen(4);
-			Shared.capBuffers(1);
+		if(!FileFormat.isFastaFile(fnames.get(0))){
+			synchronized(SortByName.class){
+				qualityObservedStatic=true;
+			}
 		}
 		
+		adjustBuffers(maxSizeObserved, fnames.size(), outstream);
+		
 		System.err.println("Merging "+fnames);
 //		outstream.println("zl="+ReadWrite.ZIPLEVEL);
 //		outstream.println("ztd="+ReadWrite.ZIP_THREAD_DIVISOR);
@@ -594,7 +635,7 @@ public class SortByName {
 		for(CrisContainer cc : cclist){
 			errorState|=cc.close();
 		}
-		if(delete){
+		if(deleteAfterMerge){
 			for(String fname : fnames){
 				new File(fname).delete();
 			}
@@ -866,7 +907,9 @@ public class SortByName {
 	/*--------------------------------------------------------------*/
 
 	long maxLengthObserved=0;
-	static long maxLengthObservedStatic=0;
+	private static long maxLengthObservedStatic=0;
+	private final boolean qualityObserved;
+	private static boolean qualityObservedStatic;
 	
 	/** Number of reads processed */
 	protected long readsProcessed=0;
@@ -875,8 +918,9 @@ public class SortByName {
 
 	/** Quit after processing this many input reads; -1 means no limit */
 	private long maxReads=-1;
-	
+
 	private boolean delete=true;
+	private boolean deleteEarly=false;
 	
 	private boolean useSharedHeader=false;
 	
@@ -888,6 +932,9 @@ public class SortByName {
 	
 	/** Max files to merge per pass */
 	private int maxFiles=12;
+	private boolean setMaxFiles=false;
+	
+	private static int mergeFileLimit=24;
 	
 	/*--------------------------------------------------------------*/
 	/*----------------         Final Fields         ----------------*/
@@ -921,7 +968,5 @@ public class SortByName {
 	private boolean overwrite=false;
 	/** Append to existing output files */
 	private boolean append=false;
-	/** This flag has no effect on singlethreaded programs */
-	private final boolean ordered=false;
 	
 }
