@@ -558,7 +558,7 @@ public class SamLine implements Serializable {
 		assert((seq==bytestar)==(Tools.equals(seq, bytestar)));
 		assert((qual==bytestar)==(Tools.equals(qual, bytestar)));
 		
-		if(mapped() && strand()==Shared.MINUS){
+		if(mapped() && strand()==Shared.MINUS && FLIP_ON_LOAD){
 			if(seq!=bytestar){AminoAcid.reverseComplementBasesInPlace(seq);}
 			if(qual!=bytestar){Tools.reverseInPlace(qual);}
 		}
@@ -837,6 +837,10 @@ public class SamLine implements Serializable {
 		return calcCigarLength(cigar, includeSoftClip, includeHardClip);
 	}
 	
+	public int calcCigarReadLength(boolean includeSoftClip, boolean includeHardClip){
+		return calcCigarReadLength(cigar, includeSoftClip, includeHardClip);
+	}
+	
 	/** Reference length of cigar string */
 	public static int calcCigarLength(String cigar, boolean includeSoftClip, boolean includeHardClip){
 		if(cigar==null){return 0;}
@@ -872,6 +876,41 @@ public class SamLine implements Serializable {
 		return len;
 	}
 	
+	/** Reference length of cigar string */
+	public static int calcCigarReadLength(String cigar, boolean includeSoftClip, boolean includeHardClip){
+		if(cigar==null){return 0;}
+		int len=0;
+		int current=0;
+		for(int i=0; i<cigar.length(); i++){
+			char c=cigar.charAt(i);
+			if(Tools.isDigit(c)){
+				current=(current*10)+(c-'0');
+			}else{
+				if(c=='M' || c=='=' || c=='X' || c=='I'){
+					len+=current;
+				}else if(c=='S'){
+					if(includeSoftClip){len+=current;}
+				}else if (c=='H'){
+					//In this case, the base string is the wrong length since letters were truncated.
+					//Therefore, the bases cannot be used for calling variations after mapping.
+					//Hard clipping messes up original location verification.
+					//Therefore...  len+=current would be best in practice, but for GRADING purposes, leaving it disabled is best.
+
+					if(includeHardClip){len+=current;}
+				}else if(c=='D' || c=='N'){
+					//do nothing
+				}else if(c=='P'){
+					throw new RuntimeException("Unhandled cigar symbol: "+c+"\n"+cigar+"\n");
+					//'P' is currently poorly defined
+				}else{
+					throw new RuntimeException("Unhandled cigar symbol: "+c+"\n"+cigar+"\n");
+				}
+				current=0;
+			}
+		}
+		return len;
+	}
+	
 	/** Number of query bases in cigar string */
 	public static int calcCigarBases(String cigar, boolean includeSoftClip, boolean includeHardClip){
 		if(cigar==null){return 0;}
@@ -1100,6 +1139,33 @@ public class SamLine implements Serializable {
 		}
 		return dels-inss;
 	}
+
+	public int mappedNonClippedBases() {
+		if(!mapped() || cigar==null || seq==null){return 0;}
+		
+		int len=0;
+		int current=0;
+		for(int i=0; i<cigar.length(); i++){
+			char c=cigar.charAt(i);
+			if(Tools.isDigit(c)){
+				current=(current*10)+(c-'0');
+			}else{
+				if(c=='M' || c=='='){
+					len+=current;
+				}else if(c=='X'){
+					len+=current;
+				}else if(c=='D' || c=='N'){
+					
+				}else if(c=='I'){
+					len+=current;
+				}else if(c=='S' || c=='H' || c=='P'){
+					
+				}
+				current=0;
+			}
+		}
+		return len;
+	}
 	
 	/**
 	 * @param cigar
@@ -1131,6 +1197,37 @@ public class SamLine implements Serializable {
 		}
 		return msdic;
 	}
+
+	public float calcIdentity() {
+		assert(cigar!=null);
+		int match=0, other=0;
+		
+		int current=0;
+		for(int i=0; i<cigar.length(); i++){
+			char c=cigar.charAt(i);
+			if(Tools.isDigit(c)){
+				current=(current*10)+(c-'0');
+			}else{
+				if(c=='='){
+					match+=current;
+				}else if(c=='M'){
+					
+				}else if(c=='X'){
+					other+=current;
+				}else if(c=='D'){
+					other+=current;
+				}else if(c=='N'){
+					
+				}else if(c=='I'){
+					other+=current;
+				}else if(c=='S' || c=='H' || c=='P'){
+					
+				}
+				current=0;
+			}
+		}
+		return match/(float)Tools.max(match+other, 1);
+	}
 	
 	/**
 	 * @param cigar
@@ -2278,7 +2375,8 @@ public class SamLine implements Serializable {
 	public byte strand(){
 		return ((flag&0x10)==0x10 ? (byte)1 : (byte)0);
 	}
-	
+
+	public byte mateStrand(){return nextStrand();}
 	public byte nextStrand(){
 		return ((flag&0x20)==0x20 ? (byte)1 : (byte)0);
 	}
@@ -2316,6 +2414,11 @@ public class SamLine implements Serializable {
 		return (flag&0x800)==0x800;
 	}
 	
+	public boolean leftmost(){
+		if(!pairedOnSameChrom() || tlen==0){return true;}
+		return tlen>0;
+	}
+	
 	/*--------------------------------------------------------------*/
 	/*----------------             ?             ----------------*/
 	/*--------------------------------------------------------------*/
@@ -2617,6 +2720,8 @@ public class SamLine implements Serializable {
 	public static boolean PARSE_OPTIONAL=true;
 	public static boolean PARSE_OPTIONAL_MD_ONLY=false;
 	
+	public static boolean FLIP_ON_LOAD=true;
+	
 	public static boolean verbose=false;
 	
 }
