@@ -70,8 +70,8 @@ public class EntropyTracker {
 		countCounts[0]=(short)windowKmers;
 		bases=new byte[windowBases];
 		
-//		assert(false) : amino+", "+windowBases+", "+entropyCutoff+", "+bitsPerBase+", "+
-//			Integer.toBinaryString(mask)+", "+kmerSpace+", "+entropy+", "+entropyMult;
+//		assert(false) : "\namino="+amino+", windowBases="+windowBases+", windowKmers="+windowKmers+", cutoff="+entropyCutoff+", bpb="+bitsPerBase+", mask="+
+//			Integer.toBinaryString(mask)+", space="+kmerSpace+", mult="+entropyMult+"\n"+"entropy="+Arrays.toString(entropy);
 		
 //		entropyDeltaPlus=makeEntropyDeltaPlus(entropy, entropyMult);
 //		entropyDeltaMinus=makeEntropyDeltaMinus(entropy, entropyMult);
@@ -256,13 +256,17 @@ public class EntropyTracker {
 		//Loop over all nonzero kmer counts
 		for(int count : counts){
 			//Prevent NaN and INF
+//			System.err.println("count="+count);
 			if(count>0){
 				//Fraction of total represented by this kmer
 				double pk=count/(double)windowKmers;
+//				System.err.println("pk="+pk);
 
 				//Entropy contribution for this kmer's count
 				double npklogpk=-pk*Math.log(pk);
+//				System.err.println("npklogpk="+npklogpk);
 				eSum+=npklogpk;
+//				System.err.println("eSum="+eSum);
 			}
 		}
 		//eSum now holds entropy in bits.
@@ -272,6 +276,7 @@ public class EntropyTracker {
 		
 		//Adjust entropy to 0-1 scale based on window size
 		float e=(float)(eSum*multiplier);
+//		System.err.println("e="+e);
 		assert(e>=0 && e<=1) : e+", "+eSum+", "+entropyMult+"\n"+Arrays.toString(entropy)+"\n"+this;
 		
 		//Get rid of negative zero
@@ -368,11 +373,15 @@ public class EntropyTracker {
 	 * @param b Base to add.
 	 */
 	public void add(final byte b){
+		
 		//Test initial state
 		assert(!verify || verify()) : this;
 		
 		final byte oldBase=bases[pos]; //Leftmost base, about to be overwritten
 		
+		if(verbose){System.err.println("\nAdding "+Character.toString((char)b)+
+				"; evicting "+Character.toString((char)oldBase)+"; counts="+Arrays.toString(counts)+"; countcounts="+Arrays.toString(countCounts)+", pos="+pos+", pos2="+pos2);}
+		
 		//Increment length
 		len++;
 		
@@ -421,15 +430,18 @@ public class EntropyTracker {
 				assert(!verify || (Tools.sum(countCounts)>0 && (Tools.sum(countCounts)<=windowKmers+1))) : this;
 			}
 		}
+		if(verbose){System.err.println("B: counts="+Arrays.toString(counts)+"; countcounts="+Arrays.toString(countCounts));}
 		
 		//At this point the state is inconsistent as it may have one too many kmers.
 		
 		if(pos2>=0){//Remove the leftmost base
-			final byte b2=bases[pos2];//This is not the leftmost base, but the base to the right of the leftmost kmer
+			final byte b2=(k>1 ? bases[pos2] : oldBase);//This is not the leftmost base, but the base to the right of the leftmost kmer
 			final int n2=symbolToNumber0[b2];
 			//Generate old leftmost kmer using an internal base near the left end
 			kmer2=((kmer2<<bitsPerBase)|n2)&mask;
 
+			if(verbose){System.err.println("B2: pos="+pos+", pos2="+pos2+"; b2="+Character.toString((char)b2)+"; kmer2="+kmer2);}
+
 			if(len>windowBases){//Remove a kmer, only if a base is leaving the window
 				//System.err.println("Removing "+kmer2);
 				
@@ -469,6 +481,7 @@ public class EntropyTracker {
 				assert(!verify || (Tools.sum(countCounts)>=0 && (Tools.sum(countCounts)<=windowKmers))) : this;
 			}
 		}
+		if(verbose){System.err.println("C: counts="+Arrays.toString(counts)+"; countcounts="+Arrays.toString(countCounts));}
 		
 		//Update position pointers
 		//Can use modulo, but this is faster because the branch is normally skipped.
@@ -477,6 +490,7 @@ public class EntropyTracker {
 		pos2++;
 		if(pos>=windowBases){pos=0;}
 		if(pos2>=windowBases){pos2=0;}
+		assert(k==1 || pos!=pos2) : "pos="+pos+", pos2="+pos2;
 		
 		//Test final state.
 		assert(!verify || verify()) : this;
@@ -488,6 +502,7 @@ public class EntropyTracker {
 	public void clear(){
 		//Reset scalars
 		kmer=kmer2=len=pos=unique=ns=0;
+		pos=0;
 		pos2=0-windowBases+k-1;
 		currentEsum=0;
 		
