@@ -93,7 +93,7 @@ public class PrintTaxonomy {
 				accessionFile=b;
 			}else if(a.equals("tree") || a.equals("taxtree")){
 				taxTreeFile=b;
-			}else if(a.equals("level") || a.equals("taxlevel")){
+			}else if(a.equals("level") || a.equals("lv") || a.equals("taxlevel") || a.equals("tl")){
 				taxLevel=TaxTree.parseLevel(b);
 			}else if(a.equals("minlevel")){
 				minLevel=TaxTree.parseLevel(b);
@@ -105,6 +105,8 @@ public class PrintTaxonomy {
 				reverseOrder=Tools.parseBoolean(b);
 			}else if(a.equals("silva")){
 				TaxTree.SILVA_MODE=Tools.parseBoolean(b);
+			}else if(a.equals("unite")){
+				TaxTree.UNITE_MODE=Tools.parseBoolean(b);
 			}else if(a.equals("simple")){
 				skipNonCanonical=Tools.parseBoolean(b);
 			}else if(a.equals("column")){
