@@ -68,8 +68,9 @@ public class RenameGiToTaxid {
 			}else if(a.equals("server") || a.equals("useserver")){
 				if(b!=null && b.startsWith("http")){
 					useServer=true;
-					TaxClient.path=b;
-					if(!TaxClient.path.endsWith("/")){TaxClient.path+="/";}
+					String path=b;
+					if(!path.endsWith("/")){path+="/";}
+					Shared.setTaxServer(path);
 				}else{
 					useServer=Tools.parseBoolean(b);
 				}
@@ -83,6 +84,8 @@ public class RenameGiToTaxid {
 				taxTreeFile=b;
 			}else if(a.equals("invalid")){
 				outInvalid=b;
+			}else if(a.equals("deleteinvalid")){
+				deleteInvalid=Tools.parseBoolean(b);
 			}else if(a.equals("badheaders")){
 				badHeaders=b;
 			}else if(a.equals("maxbadheaders") || a.equals("maxinvalidheaders")){
@@ -91,8 +94,8 @@ public class RenameGiToTaxid {
 				keepAll=Tools.parseBoolean(b);
 			}else if(a.equals("shrinknames")){
 				shrinkNames=Tools.parseBoolean(b);
-			}else if(a.equals("silva")){
-				TaxTree.SILVA_MODE=Tools.parseBoolean(b);
+			}else if(a.equals("warn")){
+				warnBadHeaders=Tools.parseBoolean(b);
 			}
 			
 			else if(a.equals("maxpigzprocesses")){
@@ -108,6 +111,9 @@ public class RenameGiToTaxid {
 					mode=Integer.parseInt(b);
 				}else if("accession".equalsIgnoreCase(b)){
 					mode=ACCESSION_MODE;
+				}else if("unite".equalsIgnoreCase(b)){
+					mode=UNITE_MODE;
+					TaxTree.UNITE_MODE=true;
 				}else if("gi".equalsIgnoreCase(b)){
 					mode=GI_MODE;
 				}else if("header".equalsIgnoreCase(b)){
@@ -148,7 +154,7 @@ public class RenameGiToTaxid {
 		if(useServer){
 			giTableFile=null;
 			accessionFile=null;
-			taxTreeFile=null;
+			if(mode!=UNITE_MODE){taxTreeFile=null;}
 		}
 		
 		{//Process parser fields
@@ -189,7 +195,8 @@ public class RenameGiToTaxid {
 		ffoutInvalid=FileFormat.testOutput(outInvalid, FileFormat.FA, null, true, overwrite, append, false);
 		ffin1=new ArrayList<FileFormat>(in1.size());
 		for(String s : in1){
-			ffin1.add(FileFormat.testInput(s, FileFormat.FA, null, true, true));
+			FileFormat ff=FileFormat.testInput(s, FileFormat.FA, null, true, true);
+			ffin1.add(ff);
 		}
 		
 		if(ffoutInvalid!=null){keepAll=false;}
@@ -234,28 +241,51 @@ public class RenameGiToTaxid {
 		
 		final HashArray1D counts=(countTable && !prefix) ? new HashArray1D(256000, -1L, true) : null;
 		
+		gffIn=false;
 		for(FileFormat ffin : ffin1){
+			gffIn=gffIn||ffin.gff();
 			ByteFile bf=ByteFile.makeByteFile(ffin);
 			if(useServer){
-				processInner_server(bf, bsw, bswInvalid, bswBadHeaders, counts);
+				processInner_server(bf, bsw, bswInvalid, bswBadHeaders, counts, ffin.format());
 			}else{
-				IntList list=(useServer ? getIds(bf) : null);
-				processInner(bf, bsw, bswInvalid, bswBadHeaders, counts, list);
+//				IntList list=(useServer ? getIds(bf) : null);
+				processInner(bf, bsw, bswInvalid, bswBadHeaders, counts, null);
 			}
 		}
 		
-		if(bsw!=null){errorState|=bsw.poisonAndWait();}
+		if(bsw!=null){
+			errorState|=bsw.poisonAndWait();
+			if(deleteInvalid && invalidReads>0 && !ffout1.stdio()){
+				try {
+					System.err.println("Deleting "+out1);
+					new File(out1).delete();
+				} catch (Exception e) {
+					System.err.println("An error occured while attempting to delete "+out1);
+					e.printStackTrace();
+				}
+			}
+		}
 		if(bswInvalid!=null){errorState|=bswInvalid.poisonAndWait();}
 		if(bswBadHeaders!=null){errorState|=bswBadHeaders.poisonAndWait();}
 		
 		t.stop();
-		outstream.println(Tools.timeReadsBasesProcessed(t, readsProcessed, basesProcessed, 8));
-		
-		outstream.println();
-		outstream.println("Valid Sequences:   \t"+validReads);
-		outstream.println("Valid Bases:       \t"+validBases);
-		outstream.println("Invalid Sequences: \t"+invalidReads);
-		outstream.println("Invalid Bases:     \t"+invalidBases);
+		if(!gffIn) {
+			outstream.println(Tools.timeReadsBasesProcessed(t, readsProcessed, basesProcessed, 8));
+
+			outstream.println();
+			outstream.println("Valid Sequences:   \t"+validReads);
+			outstream.println("Valid Bases:       \t"+validBases);
+			outstream.println("Invalid Sequences: \t"+invalidReads);
+			outstream.println("Invalid Bases:     \t"+invalidBases);
+		}else{
+			outstream.println(Tools.timeLinesBytesProcessed(t, linesIn, basesProcessed, 8));
+
+			outstream.println();
+			outstream.println("Valid Lines:       \t"+validLines);
+			outstream.println("Valid Bytes:       \t"+validBases);
+			outstream.println("Invalid Lines:     \t"+invalidLines);
+			outstream.println("Invalid Bytes:     \t"+invalidBases);
+		}
 		if(counts!=null){
 			outstream.println("Unique Taxa:       \t"+taxaCounted);
 		}
@@ -265,59 +295,60 @@ public class RenameGiToTaxid {
 		}
 	}
 	
-	public IntList getIds(ByteFile bf){
-		IntList ids=new IntList();
-		
-		int readsProcessedInner=0;
-		
-		byte[] line=bf.nextLine();
-		ByteBuilder bb=new ByteBuilder();
-		while(line!=null){
-			if(line.length>0 && line[0]=='>'){
-				readsProcessedInner++;
-				if(maxReads>0 && readsProcessedInner>maxReads){break;}
-				
-				for(int i=1; i<line.length; i++){
-					byte b=line[i];
-					if(b==' ' || b=='.'){break;}
-					else{bb.append(b);}
-				}
-				bb.append(',');
-				if(bb.length()>100000){
-					bb.setLength(bb.length()-1);
-					int[] ret;
-					if(mode==ACCESSION_MODE){
-						ret=TaxClient.accessionToTaxidArray(bb.toString());
-					}else if(mode==GI_MODE){
-						ret=TaxClient.giToTaxidArray(bb.toString());
-					}else{
-						ret=TaxClient.headerToTaxidArray(bb.toString());
-					}
-					assert(ret!=null) : bb.toString();
-					for(int i : ret){ids.add(i);}
-					bb.clear();
-				}
-			}
-			line=bf.nextLine();
-		}
-		if(bb.length()>0){
-			bb.setLength(bb.length()-1);
-			int[] ret;
-			if(mode==ACCESSION_MODE){
-				ret=TaxClient.accessionToTaxidArray(bb.toString());
-			}else if(mode==GI_MODE){
-				ret=TaxClient.giToTaxidArray(bb.toString());
-			}else{
-				ret=TaxClient.headerToTaxidArray(bb.toString());
-			}
-			assert(ret!=null) : bb.toString();
-			for(int i : ret){ids.add(i);}
-			bb.clear();
-		}
-		
-		bf.reset();
-		return ids;
-	}
+	//Unused; not efficient
+//	public IntList getIds(ByteFile bf){
+//		IntList ids=new IntList();
+//		
+//		int readsProcessedInner=0;
+//		
+//		byte[] line=bf.nextLine();
+//		ByteBuilder bb=new ByteBuilder();
+//		while(line!=null){
+//			if(line.length>0 && line[0]=='>'){
+//				readsProcessedInner++;
+//				if(maxReads>0 && readsProcessedInner>maxReads){break;}
+//				
+//				for(int i=1; i<line.length; i++){
+//					byte b=line[i];
+//					if(b==' ' || b=='.'){break;}
+//					else{bb.append(b);}
+//				}
+//				bb.append(',');
+//				if(bb.length()>100000){
+//					bb.setLength(bb.length()-1);
+//					int[] ret;
+//					if(mode==ACCESSION_MODE){
+//						ret=TaxClient.accessionToTaxidArray(bb.toString());
+//					}else if(mode==GI_MODE){
+//						ret=TaxClient.giToTaxidArray(bb.toString());
+//					}else{
+//						ret=TaxClient.headerToTaxidArray(bb.toString());
+//					}
+//					assert(ret!=null) : bb.toString();
+//					for(int i : ret){ids.add(i);}
+//					bb.clear();
+//				}
+//			}
+//			line=bf.nextLine();
+//		}
+//		if(bb.length()>0){
+//			bb.setLength(bb.length()-1);
+//			int[] ret;
+//			if(mode==ACCESSION_MODE){
+//				ret=TaxClient.accessionToTaxidArray(bb.toString());
+//			}else if(mode==GI_MODE){
+//				ret=TaxClient.giToTaxidArray(bb.toString());
+//			}else{
+//				ret=TaxClient.headerToTaxidArray(bb.toString());
+//			}
+//			assert(ret!=null) : bb.toString();
+//			for(int i : ret){ids.add(i);}
+//			bb.clear();
+//		}
+//		
+//		bf.reset();
+//		return ids;
+//	}
 	
 	private void processInner(ByteFile bf, ByteStreamWriter bsw, ByteStreamWriter bswInvalid, ByteStreamWriter bswBadHeaders, HashArray1D counts, IntList ids){
 
@@ -391,6 +422,10 @@ public class RenameGiToTaxid {
 					bsw.println();
 				}else{
 					invalidReads++;
+					if(deleteInvalid){
+						System.err.println("Invalid sequence detected; aborting.\n");
+						break;
+					}
 					if(bswBadHeaders!=null){bswBadHeaders.println(line);}
 					if(maxInvalidHeaders>=0 && invalidReads>maxInvalidHeaders){
 						KillSwitch.kill("Maximum bad headers exceeded: "+maxInvalidHeaders+"\n"+new String(line));
@@ -456,16 +491,57 @@ public class RenameGiToTaxid {
 		return true;
 	}
 	
-	private void updateHeadersFromServer(ArrayList<byte[]> lines, HashArray1D counts, ByteStreamWriter bswBadHeaders){
+	void appendHeaderLine(byte[] line, ByteBuilder bb){
+		assert(line[0]=='>' || line[0]=='@') : new String(line);
+		
+		if(mode==ACCESSION_MODE){
+			for(int i=1; i<line.length; i++){
+				byte b=line[i];
+				if(b==' ' || b=='.'){break;}
+				else{bb.append(b);}
+			}
+		}else if(mode==GI_MODE){
+			for(int i=1; i<line.length; i++){
+				byte b=line[i];
+				if(b==' ' || b=='|'){break;}
+				else{bb.append(b);}
+			}
+		}else if(mode==UNITE_MODE){
+			int initial=Tools.indexOf(line, '|');
+			for(int i=initial+1; i<line.length; i++){
+				byte b=line[i];
+				if(b==' ' || b=='.' || b=='|'){break;}
+				else{bb.append(b);}
+			}
+		}else{
+			for(int i=1; i<line.length; i++){
+				byte b=line[i];
+				bb.append(b);
+			}
+		}
+		bb.append(',');
+	}
+	
+	private void updateHeadersFromServer(ArrayList<byte[]> lines, HashArray1D counts, ByteStreamWriter bswBadHeaders, int format){
+		if(format==FileFormat.FA){
+			updateHeadersFromServer_fasta(lines, counts, bswBadHeaders);
+		}else if(format==FileFormat.GFF){
+			updateHeadersFromServer_gff(lines, counts, bswBadHeaders);
+		}else{
+			assert(false) : "Unsupported type: "+format;
+		}
+	}
+	
+	private void updateHeadersFromServer_fasta(ArrayList<byte[]> lines, HashArray1D counts, ByteStreamWriter bswBadHeaders){
 		ByteBuilder bb=new ByteBuilder();
+		ArrayList<String> names=new ArrayList<String>();
 		for(byte[] line : lines){
 			if(line[0]=='>' && !Tools.startsWith(line, ">tid")){
-				for(int i=1; i<line.length; i++){
-					byte b=line[i];
-					if(b==' ' || b=='.'){break;}
-					else{bb.append(b);}
+				appendHeaderLine(line, bb);
+				if(mode==UNITE_MODE){
+					int bar=Tools.indexOf(line, '|');
+					names.add(new String(line, 1, bar-1));
 				}
-				bb.append(',');
 			}
 		}
 		if(bb.length()<1){return;}
@@ -476,7 +552,7 @@ public class RenameGiToTaxid {
 //		System.err.println("Sending '"+bb+"'");
 		
 		final int[] serverIds;
-		if(mode==ACCESSION_MODE){
+		if(mode==ACCESSION_MODE || mode==UNITE_MODE){
 			serverIds=TaxClient.accessionToTaxidArray(bb.toString());
 		}else if(mode==GI_MODE){
 			serverIds=TaxClient.giToTaxidArray(bb.toString());
@@ -486,6 +562,25 @@ public class RenameGiToTaxid {
 		assert(serverIds!=null) : "Null response for '"+bb.toString()+"'";
 		bb.clear();
 		
+		if(!names.isEmpty()){
+			assert(tree!=null) : "Need to load a TaxTree.";
+			assert(names.size()==serverIds.length);
+			for(int i=0; i<serverIds.length; i++){
+				final String name=names.get(i);
+				if(serverIds[i]<0){
+					TaxNode tn=tree.getNodeByName(name);
+					if(tn!=null){serverIds[i]=tn.id;}
+//					else {
+//						assert(false) : names.get(i);
+//					}
+				}else{
+					//Sometimes the species gets renamed.
+//					TaxNode tn=tree.getNodeByName(name);
+//					if(tn==null || tn.id==serverIds[i]) {System.err.println(name+", "+serverIds[i]+", "+tn+", "+tree.getNodesByName(name));}
+				}
+			}
+		}
+		
 		for(int lineNum=0, serverNum=0; lineNum<=lines.size(); lineNum++){
 			byte[] line=lines.get(lineNum);
 			if(line[0]=='>' && !Tools.startsWith(line, ">tid")){
@@ -497,7 +592,7 @@ public class RenameGiToTaxid {
 						bswBadHeaders.print(tid).tab();
 						bswBadHeaders.print(looksLikeRealAccession(line)).tab();
 						bswBadHeaders.println(line);
-					}else{
+					}else if(warnBadHeaders){
 						System.err.println(tid+"\t"+looksLikeRealAccession(line)+"\t"+new String(line));
 					}
 				}
@@ -535,7 +630,97 @@ public class RenameGiToTaxid {
 		}
 	}
 	
-	private void processInner_server(ByteFile bf, ByteStreamWriter bsw, ByteStreamWriter bswInvalid, ByteStreamWriter bswBadHeaders,  HashArray1D counts){
+	private void updateHeadersFromServer_gff(ArrayList<byte[]> lines, HashArray1D counts, ByteStreamWriter bswBadHeaders){
+		ByteBuilder bb=new ByteBuilder();
+		ArrayList<String> names=new ArrayList<String>();
+		for(byte[] line : lines){
+			if(line[0]!='#' && !Tools.startsWith(line, "tid")){
+				if(bb.length()>0){bb.append(',');}
+				for(byte b : line){
+					if(b=='\t'){break;}
+					bb.append(b);
+				}
+			}
+		}
+		if(bb.length()<1){return;}
+		
+//		assert(false) : bb;
+		
+//		System.err.println("Sending '"+bb+"'");
+		
+		int[] serverIds;
+		if(mode==ACCESSION_MODE || mode==UNITE_MODE){
+			serverIds=TaxClient.accessionToTaxidArray(bb.toString());
+		}else if(mode==GI_MODE){
+			serverIds=TaxClient.giToTaxidArray(bb.toString());
+		}else{
+			serverIds=TaxClient.headerToTaxidArray(bb.toString());
+		}
+		if(serverIds==null){
+			KillSwitch.kill("Null response for '"+bb.toString()+"'");
+		}
+//		assert(serverIds!=null) : "Null response for '"+bb.toString()+"'";
+		bb.clear();
+		
+		if(!names.isEmpty()){
+			assert(tree!=null) : "Need to load a TaxTree.";
+			assert(names.size()==serverIds.length);
+			for(int i=0; i<serverIds.length; i++){
+				final String name=names.get(i);
+				if(serverIds[i]<0){
+					TaxNode tn=tree.getNodeByName(name);
+					if(tn!=null){serverIds[i]=tn.id;}
+//					else {
+//						assert(false) : names.get(i);
+//					}
+				}else{
+					//Sometimes the species gets renamed.
+//					TaxNode tn=tree.getNodeByName(name);
+//					if(tn==null || tn.id==serverIds[i]) {System.err.println(name+", "+serverIds[i]+", "+tn+", "+tree.getNodesByName(name));}
+				}
+			}
+		}
+		
+		for(int lineNum=0, serverNum=0; lineNum<=lines.size(); lineNum++){
+			byte[] line=lines.get(lineNum);
+			if(line[0]!='#' && !Tools.startsWith(line, "tid")){
+				bb.clear();
+				final int tid=serverIds[serverNum];
+				if(tid<0){
+					//WARN
+					if(bswBadHeaders!=null){
+						bswBadHeaders.print(tid).tab();
+						bswBadHeaders.print(looksLikeRealAccession(line)).tab();
+						bswBadHeaders.println(line);
+					}else if(warnBadHeaders){
+						System.err.println(tid+"\t"+looksLikeRealAccession(line)+"\t"+new String(line));
+					}
+				}
+				
+				bb.append("tid|");
+				bb.append(tid);
+				if(prefix){
+					bb.append('|');
+					bb.append(line);
+				}else if(counts!=null && tid>=0){
+					bb.append('|');
+					int count=counts.increment(tid, 1);
+					bb.append(count);
+					if(count==1){taxaCounted++;}
+				}
+				
+				lines.set(lineNum, bb.toBytes());
+				
+				serverNum++;
+				if(serverNum>=serverIds.length){break;}
+			}
+		}
+		if(maxInvalidHeaders>=0 && invalidReads>maxInvalidHeaders){
+			KillSwitch.kill("Maximum bad headers exceeded: "+maxInvalidHeaders);
+		}
+	}
+	
+	private void processInner_server(ByteFile bf, ByteStreamWriter bsw, ByteStreamWriter bswInvalid, ByteStreamWriter bswBadHeaders, HashArray1D counts, int format){
 		
 		ArrayList<byte[]> lines=new ArrayList<byte[]>();
 		byte[] line=bf.nextLine();
@@ -545,20 +730,28 @@ public class RenameGiToTaxid {
 		while(line!=null){
 			
 			if(line.length>0){
+				linesIn++;
 				lines.add(line);
 				storedBytes+=line.length;
 				if(storedBytes>=maxStoredBytes){
-					updateHeadersFromServer(lines, counts, bswBadHeaders);
+					updateHeadersFromServer(lines, counts, bswBadHeaders, format);
 					valid=dumpBuffer(lines, valid, bsw, bswInvalid);
 					lines=new ArrayList<byte[]>();
 					storedBytes=0;
+					if(deleteInvalid && invalidReads>0){
+							System.err.println("Invalid sequence detected; aborting.\n"
+									+ "Input file:  \t"+bf.name()+"\n"
+									+ "Output file: \t"+(bsw==null ? "null" : bsw.fname)+"\n"
+									+ "Line:        \t"+new String(line)+"\n");
+						break;
+					}
 				}
 			}
 			line=bf.nextLine();
 		}
 		
 		if(storedBytes>0){
-			updateHeadersFromServer(lines, counts, bswBadHeaders);
+			updateHeadersFromServer(lines, counts, bswBadHeaders, format);
 			valid=dumpBuffer(lines, valid, bsw, bswInvalid);
 			lines=new ArrayList<byte[]>();
 			storedBytes=0;
@@ -578,17 +771,32 @@ public class RenameGiToTaxid {
 				if(Tools.startsWith(line, invalidTitle)){
 					valid=false;
 					invalidReads++;
+					invalidLines++;
+					if(deleteInvalid){break;}
 				}else{
 					assert(Tools.startsWith(line, title));
 					valid=true;
 					validReads++;
+					validLines++;
+				}
+			}else if(gffIn){
+				basesProcessed+=line.length;
+				valid=!Tools.startsWith(line, invalidGffTitle);
+				if(valid){
+					validBases+=line.length;
+					validLines++;
+				}else{
+					invalidBases+=line.length;
+					invalidLines++;
 				}
 			}else{
 				basesProcessed+=line.length;
 				if(valid){
 					validBases+=line.length;
+					validLines++;
 				}else{
 					invalidBases+=line.length;
+					invalidLines++;
 				}
 			}
 			
@@ -624,6 +832,10 @@ public class RenameGiToTaxid {
 	private long invalidReads=0;
 	private long invalidBases=0;
 	private long taxaCounted=0;
+
+	private long linesIn=0;
+	private long validLines=0;
+	private long invalidLines=0;
 	
 	private long maxStoredBytes=10000000;
 	
@@ -633,12 +845,17 @@ public class RenameGiToTaxid {
 	private boolean countTable=true;
 	private boolean keepAll=true;
 	private boolean shrinkNames=false;
+	private boolean warnBadHeaders=true;
 	private boolean useServer=false;
 	/** Crash if the number of invalid headers exceeds this */
 	private long maxInvalidHeaders=-1;
+	/** Delete the output file if there are any invalid headers */
+	private boolean deleteInvalid=false;
 	
 	private int mode;
-	private static final int ACCESSION_MODE=0, GI_MODE=1, HEADER_MODE=2;
+	private static final int ACCESSION_MODE=0, GI_MODE=1, HEADER_MODE=2, UNITE_MODE=3;
+	
+	private boolean gffIn=false;
 	
 	/*--------------------------------------------------------------*/
 	
@@ -657,5 +874,6 @@ public class RenameGiToTaxid {
 
 	private static byte[] title=">tid|".getBytes();
 	private static byte[] invalidTitle=">tid|-1".getBytes();
+	private static byte[] invalidGffTitle="tid|-1".getBytes();
 	
 }
