@@ -74,7 +74,7 @@ public class TaxFilter {
 				taxTreeFile=b;
 			}else if(a.equals("accession")){
 				accessionFile=b;
-			}else if(a.equals("level") || a.equals("taxlevel")){
+			}else if(a.equals("level") || a.equals("lv") || a.equals("taxlevel") || a.equals("tl")){
 				taxLevelE=TaxTree.parseLevelExtended(b);
 //				System.err.println("Set taxLevelE = "+TaxTree.levelToStringExtended(taxLevelE)); //123
 			}else if(a.equals("reqlevel") || a.equals("requiredlevel") || a.equals("reqlevels") || a.equals("requiredlevels")){
@@ -102,7 +102,7 @@ public class TaxFilter {
 			}else if(a.equals("requirepresent")){
 				REQUIRE_PRESENT=Tools.parseBoolean(b);
 				TaxTree.SHOW_WARNINGS=REQUIRE_PRESENT;
-			}else if(a.equals("id") || a.equals("ids") || a.equals("taxid") || a.equals("taxids")){
+			}else if(a.equals("id") || a.equals("ids") || a.equals("taxid") || a.equals("tid") || a.equals("taxids")){
 				ids=b;
 			}
 		}
@@ -196,7 +196,7 @@ public class TaxFilter {
 		}else if(a.equals("tree") || a.equals("taxtree")){
 		}else if(a.equals("accession")){
 		}else if(a.equals("taxpath")){
-		}else if(a.equals("level") || a.equals("taxlevel")){
+		}else if(a.equals("level") || a.equals("lv") || a.equals("taxlevel") || a.equals("tl")){
 		}else if(a.equals("name") || a.equals("names")){
 		}else if(a.equals("regex")){
 		}else if(a.equals("contains")){
@@ -205,7 +205,7 @@ public class TaxFilter {
 		}else if(a.equals("promote")){
 		}else if(a.equals("exclude")){
 		}else if(a.equals("printnodesadded") || a.equals("printnodes")){
-		}else if(a.equals("id") || a.equals("ids") || a.equals("taxid") || a.equals("taxids")){
+		}else if(a.equals("id") || a.equals("ids") || a.equals("taxid") || a.equals("tid") || a.equals("taxids")){
 		}else if(a.equals("requirepresent")){
 		}else if(a.equals("reqlevel") || a.equals("requiredlevel") || a.equals("reqlevels") || a.equals("requiredlevels")){
 		}else{
