@@ -131,8 +131,6 @@ public class TaxServer {
 				sizeFile=b;
 			}else if(a.equalsIgnoreCase("img")){
 				imgFile=b;
-			}else if(a.equals("reverse")){
-				reverseOrder=Tools.parseBoolean(b);
 			}else if(a.equals("domain")){
 				domain=b;
 				while(domain!=null && domain.endsWith("/")){domain=domain.substring(0, domain.length()-1);}
@@ -285,6 +283,8 @@ public class TaxServer {
 //			if(searcher.refFiles.isEmpty()){System.gc();}
 		}
 		
+//		assert(false) : searcher.refFileCount();
+		
 		//Load reference sketches
 		hasSketches=searcher.refFileCount()>0;
 		if(hasSketches){
@@ -296,7 +296,7 @@ public class TaxServer {
 		}
 		
 		SketchObject.allowMultithreadedFastq=(maxConcurrentSketchLoadThreads>1);
-		SketchObject.defaultParams.reads=defaultSketchReads;
+		SketchObject.defaultParams.maxReads=defaultSketchReads;
 		ReadWrite.USE_UNPIGZ=false;
 //		ReadWrite.USE_UNBGZIP=false;
 		
@@ -351,11 +351,11 @@ public class TaxServer {
 			result=ServerTools.sendAndReceive(oldKillCode.getBytes(), oldAddress);
 		} catch (Exception e) {
 			// TODO Auto-generated catch block
-			e.printStackTrace();
+			if(e!=null){e.printStackTrace();}
 			System.err.println("\nException suppressed; continuing.\n");
 			return;
 		}
-		if(result==null || !result.s.equals("Success.")){
+		if(result==null || result.s==null || !"Success.".equals(result.s)){
 //			KillSwitch.kill("Bad kill result: "+result+"\nQuitting.\n");
 			System.err.println("Bad kill result: "+result+"\nContinuing.\n");
 		}
@@ -561,7 +561,7 @@ public class TaxServer {
 			}
 			
 			if(verbose2){System.err.println("Loading sketches from file "+fname);}
-			sketches=tool.loadSketchesFromFile(fname, (SketchMakerMini)null, maxConcurrentSketchLoadThreads, params.reads, params.mode, params, true);
+			sketches=tool.loadSketchesFromFile(fname, (SketchMakerMini)null, maxConcurrentSketchLoadThreads, params.maxReads, params.mode, params, true);
 			if(verbose2){System.err.println("Loaded "+(sketches==null ? "null" : sketches.size())+" sketches from file "+fname);}
 			return sketches;
 		}
@@ -789,7 +789,7 @@ public class TaxServer {
 								fileMode=true;
 								trimmed+=5;
 								break;
-							}else if(a.equals("ref") || a.equals("taxid")){
+							}else if(a.equals("ref") || a.equals("taxid") || a.equals("tid")){
 								refMode=true;
 								trimmed+=4;
 								break;
@@ -855,7 +855,7 @@ public class TaxServer {
 			
 			if(sketchOnly){
 				ServerTools.reply("\nERROR: This server is tunning in sketch mode and should not be used for taxonomic lookups.\n"
-						+ "The taxonomy server is at https://taxonomy.jgi-psf.org/\n", "text/plain", t, verbose2, 400, true);
+						+ "The taxonomy server is at "+Shared.taxServer()+"\n", "text/plain", t, verbose2, 400, true);
 				return;
 			}
 			
@@ -1727,8 +1727,8 @@ public class TaxServer {
 					+ "For example:\n"
 					+ "/simple/id/1234\n"
 					+ "\nTo print taxonomy from the command line in Linux, use curl:\n"
-					+ "curl https://taxonomy.jgi-psf.org/id/9606\n"
-					+ "\nQueries longer than around 8kB can be sent via POST: curl https://taxonomy.jgi-psf.org/POST"
+					+ "curl https://taxonomy.jgi.doe.gov/id/9606\n"
+					+ "\nQueries longer than around 8kB can be sent via POST: curl https://taxonomy..doe.gov/POST"
 					+ "\n...where the data sent is, for example: name/e.coli,h.sapiens,c.lupus\n"
 					+ "\nLast restarted "+startTime+"\n"
 					+ "Running BBMap version "+Shared.BBMAP_VERSION_STRING+"\n";
@@ -1985,7 +1985,7 @@ public class TaxServer {
 		final int inSize=inSketches.size();
 		querySketches.addAndGet(inSize);
 		if(Shared.threads()<2 || maxConcurrentSketchCompareThreads<2 || inSize<4){
-			StringBuilder sb=new StringBuilder();
+			ByteBuilder sb=new ByteBuilder();
 			success=searcher.compare(inSketches, sb, params, maxConcurrentSketchCompareThreads);
 			return sb.toString();
 		}else{//More sketches than threads, and more than one thread
@@ -2042,6 +2042,7 @@ public class TaxServer {
 		public void run(){
 			success=false;
 			final int inLim=inSketches.size();
+			final boolean json=params.json();
 			
 			for(int inNum=nextSketch.getAndIncrement(); inNum<inLim; inNum=nextSketch.getAndIncrement()){
 				Sketch a=inSketches.get(inNum);
@@ -2051,6 +2052,16 @@ public class TaxServer {
 
 				ByteBuilder bb=sr.toText(params);
 				if(out!=null){
+					if(json && inLim>1){
+						if(inNum==0){
+							bb.insert(0, (byte)'[');
+						}
+						if(inNum<inLim-1){
+							bb.append(',');
+						}else{
+							bb.append(']');
+						}
+					}
 					synchronized(out){
 						out[inNum]=bb;
 					}
@@ -2192,9 +2203,6 @@ public class TaxServer {
 	/** Maps common organism names to scientific names */
 	private final HashMap<String, String> commonMap;
 	
-	/** Reverse order for tax lines */
-	private boolean reverseOrder=true;
-	
 	/** Hash taxonomic names for lookup */
 	private boolean hashNames=true;
 	private boolean hashDotFormat=true;
@@ -2205,7 +2213,7 @@ public class TaxServer {
 	private String oldAddress=null;
 
 	/** Address of current server instance (optional) */
-	public String domain=null;//"taxonomy.jgi-psf.org";
+	public String domain=null;
 
 	public int maxConcurrentSketchCompareThreads=8;//TODO: This might be too high when lots of concurrent sessions are active
 	public int maxConcurrentSketchLoadThreads=4;//TODO: This might be too high when lots of concurrent sessions are active
