@@ -326,13 +326,8 @@ public class A_SampleMT implements Accumulator<A_SampleMT.ProcessThread> {
 			alpt.add(new ProcessThread(cris, ros, i));
 		}
 		
-		//Start the threads
-		for(ProcessThread pt : alpt){
-			pt.start();
-		}
-		
-		//Wait for threads to finish
-		boolean success=ThreadWaiter.waitForThreads(alpt, this);
+		//Start the threads and wait for them to finish
+		boolean success=ThreadWaiter.startAndWait(alpt, this);
 		errorState&=!success;
 		
 		//Do anything necessary after processing
