@@ -289,6 +289,8 @@ public class KmerTableSetU extends AbstractKmerTableSet {
 		merge=merge_;
 		minProb=(float)minProb_;
 		rcomp=rcomp_;
+		Kmer.rcomp=rcomp;
+//		System.err.println("rcomp="+rcomp);
 		estimatedKmerCapacity=(long)((tableMemory*1.0/bytesPerKmer)*((prealloc ? preallocFraction : 0.81))*(HashArrayU.maxLoadFactorFinal*0.97));
 		KmerCountAbstract.minProb=(minProbPrefilter ? minProb : 0);
 		
